SIGuiModel subclass: #SIInterfaceFinder
	instanceVariableNames: 'interfaceHome interfaceList interfaceListSelection classList classListSelection iMode cMode inMorphic '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallInterfaces Squeak GUI'!
!SIInterfaceFinder commentStamp: '<historical>' prior: 0!
I can be used for finding defined interfaces in system.

I support:
* interface & class filtering
* create stubed class & interface
* find interfaces conforming classes
* find interfaces implemented by classes
* extend/unextend interfaces
* seal/unseal interfaces

---
#creator
[:masashi | ^umezawa] 
#version 2.0 MU 5/12/2000 17:16!


!SIInterfaceFinder methodsFor: 'initialize-release' stamp: 'MU 4/20/2000 00:11'!
initialize
	super initialize.
	inMorphic := false.
	interfaceHome := Interface home.
	self filterAllInterfaces.! !


!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 5/13/2000 02:20'!
createStubClass
	| if defaultStubClassName defaultSuperClassName stubClassName superClassName superClass cls |
	if := self selectedInterface ifNil:[^self].
	defaultStubClassName := if defaultClassName.
	defaultSuperClassName := #Object.
	stubClassName := self defaultDialogClass request: 'enter stub class name' initialAnswer: defaultStubClassName.
	stubClassName isEmpty ifTrue: [stubClassName := defaultStubClassName].
	superClassName := self defaultDialogClass request: ' enter superclass name' initialAnswer: defaultSuperClassName.
	superClassName isEmpty ifTrue: [superClassName := defaultSuperClassName].
	(stubClassName isEmpty or: [superClassName isEmpty])
		ifTrue: [^ self].
	superClass := self environment at: superClassName asSymbol ifAbsent: [^ self].
	self defaultInformerClass informUser: 'Creating...' during: [cls := if asClassNamed: stubClassName asSymbol super: superClass].
	self addClassList: cls! !

!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 5/13/2000 02:23'!
createStubInterface
	| cls defaultStubInterfaceName  stubInterfaceName if  |
	cls := self selectedClass ifNil:[^self].
	defaultStubInterfaceName := cls defaultInterfaceName.
	stubInterfaceName := self defaultDialogClass request: 'enter stub interface name' initialAnswer: defaultStubInterfaceName.
	(stubInterfaceName isEmpty) ifTrue:[^self].
	self defaultInformerClass informUser: 'Creating...' during: [if := cls asInterfaceNamed: stubInterfaceName asSymbol].
	self addInterfaceList: if.! !

!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 5/13/2000 02:21'!
extendInterfaces
	| selectedIf candidates extendTargets |
	selectedIf := self selectedInterface ifNil:[^self].
	candidates := ((Interface home allInterfaces - selectedIf extended) remove: selectedIf; yourself) asSortedCollection: Class sortBlock.
	extendTargets := self defaultDialogClass chooseMultiple: 'select interfaces to extend' from: candidates.
	extendTargets isEmpty ifTrue: [^ self].
	self defaultInformerClass informUser: 'Extending...' during: [extendTargets do: [:each | selectedIf extend: each]].
	self browseInterfaceDefinition.! !

!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 5/12/2000 18:12'!
forceInMorphic
	self inMorphic: true! !

!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 1/23/2001 23:58'!
implementInterfaces
	| cls nonImplemented ifs |
	cls := self selectedClass ifNil:[^self].
	nonImplemented := Interface home allInterfaces - cls allInterfaces asSortedCollection: Class sortBlock.
	ifs := self defaultDialogClass
				chooseMultiple: 'select interfaces to be implemented'
				from:nonImplemented.
	ifs isEmpty ifTrue: [^ self].
	self defaultInformerClass informUser: 'Implementing...' during: [ifs do: [:each | cls implement: each]].
	self classList: (Array with:(self nameFromClass: cls)).
	self listAllImplementedInterfaces.! !

!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 5/12/2000 18:12'!
popClassFilterMenu
	| menu selector |
	menu := self defaultMenuClass new addList: self class classFilterMenuArray.
	menu title: 'class filter'.
	selector := menu startUp.
	selector isNil ifFalse: [self perform: selector].! !

!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 5/12/2000 18:11'!
popInterfaceFilterMenu
	| menu selector |
	menu := self defaultMenuClass new addList: self class interfaceFilterMenuArray.
	menu title: 'interface filter'.
	selector := menu startUp.
	selector isNil ifFalse: [self perform: selector].! !

!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 5/13/2000 02:39'!
toggleSealInterface
	| selectedIf arr index |
	selectedIf := self selectedInterface ifNil: [^ self].
	arr := self interfaceList.
	index := arr indexOf: self interfaceListSelection.
	selectedIf isSealed
		ifTrue: [selectedIf beMutable]
		ifFalse: [selectedIf beSealed].
	arr at: index put: (self nameFromInterface: selectedIf).
	self interfaceList: arr.! !

!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 5/13/2000 02:22'!
unextendInterfaces
	| selectedIf candidates unextendTargets |
	selectedIf :=  self selectedInterface ifNil:[^self].
	candidates := selectedIf extended asSortedCollection: Class sortBlock.
	unextendTargets := self defaultDialogClass chooseMultiple: 'select interfaces to unextend' from: candidates.
	unextendTargets isEmpty ifTrue: [^ self].
	self defaultInformerClass informUser: 'Unextending...' during: [unextendTargets do: [:each | selectedIf unimplement: each]].
	self browseInterfaceDefinition.! !

!SIInterfaceFinder methodsFor: 'actions' stamp: 'MU 5/13/2000 02:53'!
unimplementInterfaces
	| cls implemented ifs |
	cls := self selectedClass ifNil:[^self].
	implemented := cls allInterfaces asSortedCollection: Class sortBlock.
	ifs := self defaultDialogClass
				chooseMultiple: 'select interfaces to be unimplemented'
				from: implemented.
	ifs isEmpty ifTrue: [^ self].
	self defaultInformerClass informUser: 'Unimplementing...' during: [ifs do: [:each | cls unimplement: each]].
	self classList: (Array with:(self nameFromClass: cls)).
	self listAllImplementedInterfaces.! !


!SIInterfaceFinder methodsFor: 'actions listing' stamp: 'MU 5/13/2000 02:40'!
addClassList: aClass
	| sz arr |
	sz := self classList size.
	arr := Array new: sz + 1.
	1 to: sz do: [:idx | arr at: idx put: (self classList at: idx)].
	arr at: sz + 1 put: (self nameFromClass: aClass).
	self classList: arr.! !

!SIInterfaceFinder methodsFor: 'actions listing' stamp: 'MU 5/13/2000 02:40'!
addInterfaceList: anInterface	
	| sz arr |
	sz := self interfaceList size.
	arr := Array new: sz + 1.
	1 to: sz do: [:idx | arr at: idx put: (self interfaceList at: idx)].
	arr at: sz + 1 put: (self nameFromInterface: anInterface).
	self interfaceList: arr.! !

!SIInterfaceFinder methodsFor: 'actions listing' stamp: 'MU 5/13/2000 02:33'!
listAllConformerClasses
	| if |
	if :=  self selectedInterface ifNil:[^self].
	self classList: (self namesFromClasses: if allConformers).
	self cMode: #allConformerClasses! !

!SIInterfaceFinder methodsFor: 'actions listing' stamp: 'MU 5/13/2000 02:34'!
listAllImplementedInterfaces
	| cls |
	cls := self selectedClass ifNil:[^self].
	self interfaceList: (self namesFromInterfaces: cls allInterfaces).
	self iMode: #allImplementedInterfaces.! !

!SIInterfaceFinder methodsFor: 'actions listing' stamp: 'MU 5/13/2000 02:33'!
listConformerClasses
	| if |
	if := self selectedInterface ifNil:[^self].
	self classList: (self namesFromClasses: if conformers).
	self cMode: #conformerClasses! !

!SIInterfaceFinder methodsFor: 'actions listing' stamp: 'MU 5/13/2000 02:34'!
listImplementedInterfaces
	| cls |
	cls := self selectedClass ifNil:[^self].
	self interfaceList: (self namesFromInterfaces: cls interfaces).
	self iMode: #implementedInterfaces.! !


!SIInterfaceFinder methodsFor: 'actions browsing' stamp: 'MU 5/13/2000 02:24'!
browseClass
	| cls |
	cls := self selectedClass ifNil:[^self].
	Browser newOnClass: cls.! !

!SIInterfaceFinder methodsFor: 'actions browsing' stamp: 'MU 5/13/2000 02:25'!
browseClassDefinition
	| cls |
	cls := self selectedClass ifNil:[^self].
	self privBrowseDefinitionOf: cls
	
	! !

!SIInterfaceFinder methodsFor: 'actions browsing' stamp: 'MU 5/13/2000 02:25'!
browseClassRepertoire
	| cls |
	cls := self selectedClass ifNil:[^self].
	self privBrowseRepertoireOf: cls.
	! !

!SIInterfaceFinder methodsFor: 'actions browsing' stamp: 'MU 5/13/2000 02:22'!
browseInterface
	| if |
	if := self selectedInterface ifNil:[^self].
	Browser newOnClass: if.! !

!SIInterfaceFinder methodsFor: 'actions browsing' stamp: 'MU 5/13/2000 02:22'!
browseInterfaceDefinition
	| if |
	if := self selectedInterface ifNil:[^self].
	self privBrowseDefinitionOf: if
	
	! !

!SIInterfaceFinder methodsFor: 'actions browsing' stamp: 'MU 5/13/2000 02:22'!
browseInterfaceRepertoire
	| if |
	if := self selectedInterface ifNil:[^self].
	self privBrowseRepertoireOf: if.
	! !


!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'MU 5/13/2000 02:33'!
filterAllClasses
	self classList: (self namesFromClasses: self environment allNonInterfaceClasses).
	self cMode: ''! !

!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'MU 5/13/2000 02:33'!
filterAllInterfaces
	self interfaceList: (self namesFromInterfaces: self interfaceHome allInterfaces).
	self iMode: ''! !

!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'M.U 6/20/1999 02:17'!
filterAllSubclasses
	self privDialogFilterClassesUsing: [:cls | cls allSubclasses]. ! !

!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'M.U 6/20/1999 01:56'!
filterCategoryNamedClasses
	| categoryName categoryStrings clsStrings |
	categoryName := self defaultDialogClass request: 'enter category name' initialAnswer: '*'.
	categoryStrings := (SystemOrganization categories select: [:each | categoryName match: each]).
	clsStrings := OrderedCollection new: (categoryStrings size * 4).
	categoryStrings do:[:each | clsStrings addAll: (SystemOrganization listAtCategoryNamed: each)].
	self classList: clsStrings.
	self cMode: ''! !

!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'MU 5/13/2000 02:33'!
filterExtendedInterfaces
	self interfaceList: (self namesFromInterfaces: self interfaceHome allExtendedInterfaces).
	self iMode: ''! !

!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'MU 5/13/2000 02:34'!
filterExtendingInterfaces
	self interfaceList: (self namesFromInterfaces: self interfaceHome allExtendingInterfaces).
	self iMode: ''! !

!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'MU 5/13/2000 02:34'!
filterLeafInterfaces
	self interfaceList: (self namesFromInterfaces: self interfaceHome leafInterfaces).
	self iMode: ''! !

!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'MU 5/13/2000 02:34'!
filterRootInterfaces
	self interfaceList: (self namesFromInterfaces: self interfaceHome rootInterfaces).
	self iMode: ''! !

!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'M.U 6/20/1999 02:16'!
filterSubclasses
	self privDialogFilterClassesUsing: [:cls | cls subclasses].! !

!SIInterfaceFinder methodsFor: 'actions filtering' stamp: 'M.U 6/20/1999 02:19'!
filterSuperclasses
	self privDialogFilterClassesUsing: [:cls | cls allSuperclasses].! !


!SIInterfaceFinder methodsFor: 'menu constants' stamp: 'M.U 6/19/1999 15:36'!
classMenuTitle
	^self class classMenuTitle! !

!SIInterfaceFinder methodsFor: 'menu constants' stamp: 'M.U 6/19/1999 15:37'!
interfaceMenuTitle
	^self class interfaceMenuTitle! !


!SIInterfaceFinder methodsFor: 'list menu creation' stamp: 'M.U 6/21/1999 23:07'!
classListMenu: aCustomMenu 
	^self privBuildMenu: aCustomMenu with: self class classListMenuArray! !

!SIInterfaceFinder methodsFor: 'list menu creation' stamp: 'M.U 6/21/1999 23:07'!
interfaceListMenu: aCustomMenu 
	^self privBuildMenu: aCustomMenu with: self class interfaceListMenuArray.! !


!SIInterfaceFinder methodsFor: 'self update' stamp: 'MU 5/12/2000 17:40'!
updateClassList
	self classList: #().
	self cMode == #conformerClasses ifTrue: [self listConformerClasses].
	self cMode == #allConformerClasses ifTrue: [self listAllConformerClasses].! !

!SIInterfaceFinder methodsFor: 'self update' stamp: 'MU 5/12/2000 17:56'!
updateInterfaceList
	self interfaceList: #().
	self iMode == #implementedInterfaces ifTrue: [self listImplementedInterfaces].
	self iMode == #allImplementedInterfaces ifTrue: [self listAllImplementedInterfaces].
	! !


!SIInterfaceFinder methodsFor: 'change notification' stamp: 'M.U 6/19/1999 23:44'!
classListSelectionChanged
	self changed: #classListSelection.
	self updateInterfaceList.
	
	! !

!SIInterfaceFinder methodsFor: 'change notification' stamp: 'M.U 6/19/1999 19:52'!
interfaceListSelectionChanged
	self changed: #interfaceListSelection.
	self updateClassList.
	! !


!SIInterfaceFinder methodsFor: 'factory' stamp: 'MU 5/12/2000 18:18'!
defaultDialogClass
	^SIDialog! !

!SIInterfaceFinder methodsFor: 'factory' stamp: 'M.U 6/20/1999 00:13'!
defaultEditorClass
	^Workspace! !

!SIInterfaceFinder methodsFor: 'factory' stamp: 'M.U 6/20/1999 02:23'!
defaultInformerClass
	^Utilities! !

!SIInterfaceFinder methodsFor: 'factory' stamp: 'M.U 6/19/1999 18:03'!
defaultMenuClass
	^CustomMenu! !

!SIInterfaceFinder methodsFor: 'factory' stamp: 'M.U 6/19/1999 22:54'!
environment
	^Smalltalk! !


!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/20/1999 00:38'!
cMode

	^cMode! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/20/1999 00:46'!
cMode: aSymbol

	cMode := aSymbol.
	self changed: #cMode.! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 14:55'!
classList

	^classList! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 18:23'!
classList: aValue

	classList := aValue.
	self changed: #classList.! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 14:55'!
classListSelection

	^classListSelection! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 19:47'!
classListSelection: aValue

	classListSelection := aValue.
	self classListSelectionChanged.
	! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/21/1999 23:16'!
forceInMorphic: aBolean

	forceInMorphic := aBolean.
	! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/20/1999 00:38'!
iMode

	^iMode! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/20/1999 00:47'!
iMode: aSymbol

	iMode := aSymbol.
	self changed: #iMode! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/21/1999 23:18'!
inMorphic
	^inMorphic! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/21/1999 23:19'!
inMorphic: aBoolean
	inMorphic := aBoolean! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 15:04'!
interfaceHome

	^interfaceHome! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 15:04'!
interfaceHome: aValue

	interfaceHome := aValue! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 14:56'!
interfaceList

	^interfaceList! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 18:23'!
interfaceList: aValue

	interfaceList := aValue.
	self changed: #interfaceList.! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 14:56'!
interfaceListSelection

	^interfaceListSelection! !

!SIInterfaceFinder methodsFor: 'accessing' stamp: 'M.U 6/19/1999 19:47'!
interfaceListSelection: aValue

	interfaceListSelection := aValue.
	self interfaceListSelectionChanged.! !


!SIInterfaceFinder methodsFor: 'private' stamp: 'MU 5/13/2000 02:11'!
classFromName: aClassName
	^self environment at: aClassName ifAbsent: [].! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'MU 5/13/2000 02:17'!
interfaceFromName: anInterfaceName
	| nm |
	anInterfaceName ifNil:[^nil].
	nm := anInterfaceName copyUpTo: Character space.
	^self environment at: nm asSymbol ifAbsent: [].
	! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'MU 5/13/2000 02:30'!
nameFromClass: aClass
	^aClass name! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'MU 5/13/2000 02:31'!
nameFromInterface: aInterface 
	^ aInterface isSealed
				ifTrue: [aInterface name , ' <S>']
				ifFalse: [aInterface name]! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'MU 5/13/2000 02:32'!
namesFromClasses: aCollectionOfClass
	^(aCollectionOfClass collect: [:each | self nameFromClass: each]) asArray.! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'MU 5/13/2000 02:33'!
namesFromInterfaces: aCollectionOfInterface 
	^ (aCollectionOfInterface
		collect: [:each | self nameFromInterface: each]) asArray.! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'M.U 6/20/1999 00:15'!
privBrowseDefinitionOf: aClassOrInterface
	| editor |
	editor := self defaultEditorClass new.
	editor contents: aClassOrInterface definitionWithInterfaces.
	editor openLabel: aClassOrInterface name. ' definition' 
	
	! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'M.U 6/19/1999 23:20'!
privBrowseRepertoireOf: aClassOfInterface
	| menu selectors result |
	menu := self defaultMenuClass new.
	selectors := aClassOfInterface repertoire collect:[ :each | Array with: each with:each].
	menu addList: selectors.
	result := menu startUpWithCaption: 'repertoire'.
	result isNil ifFalse:[ Browser newOnClass: aClassOfInterface selector: result] 
	! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'M.U 6/21/1999 23:23'!
privBuildMenu: aCustomMenu with: aMenuArray
	"Morhic should supoort #- syntax!! M.U 6/21/1999 23:06"
	(self environment isMorphic or: [self inMorphic])
		ifTrue: [aMenuArray do: [:each | each == #-
					ifTrue: [aCustomMenu addLine]
					ifFalse: [aCustomMenu add: each first action: each last]]]
		ifFalse: [aCustomMenu addList: aMenuArray].
	^aCustomMenu! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'MU 5/13/2000 02:33'!
privDialogFilterClassesUsing: aQueryBlock
	| ans className cls |
	ans := self classListSelection isNil
				ifTrue: ['']
				ifFalse: [self classListSelection].
	className := self defaultDialogClass request: 'enter class name' initialAnswer: ans.
	cls := self environment at: className asSymbol ifAbsent: [^ self].
	self classList: (self namesFromClasses: (aQueryBlock value: cls)).
	self cMode: ''.! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'MU 5/13/2000 02:19'!
selectedClass
	^self classFromName: self classListSelection! !

!SIInterfaceFinder methodsFor: 'private' stamp: 'MU 5/13/2000 02:21'!
selectedInterface
	^self interfaceFromName: self interfaceListSelection! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SIInterfaceFinder class
	instanceVariableNames: ''!

!SIInterfaceFinder class methodsFor: 'interface opening' stamp: 'MU 5/8/2000 01:04'!
openInMVC
	"SIInterfaceFinder openInMVC"
	"# model init"
	| model interfaceButtonView interfaceModeView interfaceListView classButtonView classModeView classListView interfaceView classView topView |
	model := self new.
	"# build parts"
	interfaceButtonView := self
				buildButtonViewBy: PluggableButtonView
				label: 'Interfaces'
				action: [model popInterfaceFilterMenu].
	interfaceModeView := self buildPluggableInterfaceModeViewBy: PluggableTextView on: model.
	interfaceListView := self buildPluggableInterfaceListViewBy: PluggableListViewByItem on: model.
	classButtonView := self
				buildButtonViewBy: PluggableButtonView
				label: 'Classes'
				action: [model popClassFilterMenu].
	classModeView := self buildPluggableClassModeViewBy: PluggableTextView on: model.
	classListView := self buildPluggableClassListViewBy: PluggableListViewByItem on: model.
	"# set parts border & size"
	interfaceButtonView borderWidth: 1;
	 window: (0 @ 0 corner: 100 @ 14).
	interfaceModeView borderWidth: 1;
	 window: (0 @ 0 corner: 100 @ 8).
	interfaceListView borderWidth: 1;
	 window: (0 @ 0 corner: 100 @ 78).
	classButtonView borderWidth: 1;
	 window: (0 @ 0 corner: 100 @ 14).
	classModeView borderWidth: 1;
	 window: (0 @ 0 corner: 100 @ 8).
	classListView borderWidth: 1;
	 window: (0 @ 0 corner: 100 @ 78).
	"# coloring"
	interfaceButtonView backgroundColor: Color yellow.
	interfaceModeView backgroundColor: Color gray.
	classButtonView backgroundColor: Color red.
	classModeView backgroundColor: Color gray.
	"# assembling"
	interfaceView := View new model: model.
	interfaceView addSubView: interfaceButtonView;
	 addSubView: interfaceListView below: interfaceButtonView;
	 addSubView: interfaceModeView below: interfaceListView.
	classView := View new model: model.
	classView addSubView: classButtonView;
	 addSubView: classListView below: classButtonView;
	 addSubView: classModeView below: classListView.
	interfaceModeView controller: ReadOnlyTextController new.
	classModeView controller: ReadOnlyTextController new.
	topView := StandardSystemView new label: self name;
			 minimumSize: 200 @ 100;
			 addSubView: interfaceView;
			 addSubView: classView toRightOf: interfaceView.
	topView borderWidth: 3.
	topView controller open.! !

!SIInterfaceFinder class methodsFor: 'interface opening' stamp: 'MU 5/8/2000 01:04'!
openInMorphic
	"SIInterfaceFinder openInMorphic openInMVC"
	"# model init"
	| model interfaceButtonView interfaceModeView interfaceListView classButtonView classModeView classListView window |
	model := self new forceInMorphic.
	"# build parts"
	interfaceButtonView := self
				buildButtonViewBy: PluggableButtonMorph
				label: 'Interfaces'
				action: [model popInterfaceFilterMenu].
	interfaceModeView := self buildPluggableInterfaceModeViewBy: PluggableTextMorph on: model.
	interfaceListView := self buildPluggableInterfaceListViewBy: PluggableListMorphByItem on: model.
	classButtonView := self
				buildButtonViewBy: PluggableButtonMorph
				label: 'Classes'
				action: [model popClassFilterMenu].
	classModeView := self buildPluggableClassModeViewBy: PluggableTextMorph on: model.
	classListView := self buildPluggableClassListViewBy: PluggableListMorphByItem on: model.
	"# coloring"
	interfaceButtonView onColor: Color gray offColor: Color yellow.
	interfaceModeView color: Color gray.
	classButtonView onColor: Color gray offColor: Color red.
	classModeView color: Color gray.
	"# assembling"
	window := (SystemWindow labelled: self name) model: model;
			 addMorph: interfaceButtonView frame: (0.0 @ 0.0 extent: 0.5 @ 0.1);
			 addMorph: interfaceListView frame: (0.0 @ 0.1 extent: 0.5 @ 0.9);
			 addMorph: interfaceModeView frame: (0.0 @ 0.9 extent: 0.5 @ 0.1);
			 addMorph: classButtonView frame: (0.5 @ 0.0 extent: 0.5 @ 0.1);
			 addMorph: classListView frame: (0.5 @ 0.1 extent: 0.5 @ 0.9);
			 addMorph: classModeView frame: (0.5 @ 0.9 extent: 0.5 @ 0.1).
	window borderWidth: 3.
	window openInWorldExtent: 500@250.
	^ window.! !


!SIInterfaceFinder class methodsFor: 'builder' stamp: 'M.U 6/19/1999 16:26'!
buildPluggableClassListViewBy: aViewClass on: aModel
	| view |
	view := aViewClass on: aModel
			list: #classList
			selected: #classListSelection
			changeSelected: #classListSelection:
			menu: #classListMenu:.
	view  menuTitleSelector: #classMenuTitle.
	^view! !

!SIInterfaceFinder class methodsFor: 'builder' stamp: 'M.U 6/20/1999 01:11'!
buildPluggableClassModeViewBy: aViewClass on: aModel
	| view |
	view := aViewClass on: aModel
			text: #cMode
			accept: #cMode:.
	^view! !

!SIInterfaceFinder class methodsFor: 'builder' stamp: 'M.U 6/19/1999 16:26'!
buildPluggableInterfaceListViewBy: aViewClass on: aModel
	| view |
	view := aViewClass on: aModel
			list: #interfaceList
			selected: #interfaceListSelection
			changeSelected: #interfaceListSelection:
			menu: #interfaceListMenu:.
	view  menuTitleSelector: #interfaceMenuTitle.
	^view! !

!SIInterfaceFinder class methodsFor: 'builder' stamp: 'M.U 6/20/1999 01:11'!
buildPluggableInterfaceModeViewBy: aViewClass on: aModel
	| view |
	view := aViewClass on: aModel
			text: #iMode
			accept: #iMode:.
	^view! !


!SIInterfaceFinder class methodsFor: 'menu arrays' stamp: 'M.U 6/20/1999 02:04'!
classFilterMenuArray
	^#(
	('all classes' #filterAllClasses)
	('classes in category named...' #filterCategoryNamedClasses)
	('subclasses of...' #filterSubclasses)
	('all subclasses of...' #filterAllSubclasses)
	('superclasses of...' #filterSuperclasses)
	)! !

!SIInterfaceFinder class methodsFor: 'menu arrays' stamp: 'MU 5/12/2000 19:16'!
classListMenuArray
	^#(
	('browse' #browseClass)
	('repertoire' #browseClassRepertoire)
	('definition' #browseClassDefinition)
	-
	('interfaces' #listImplementedInterfaces)
	('all interfaces' #listAllImplementedInterfaces)
	-
	('create stub interface...' #createStubInterface)
	('implement interfaces...' #implementInterfaces)
	('unimplement interfaces...' #unimplementInterfaces)
	)! !

!SIInterfaceFinder class methodsFor: 'menu arrays' stamp: 'M.U 6/19/1999 22:58'!
interfaceFilterMenuArray
	^#(
	('all interfaces' #filterAllInterfaces)
	('root interfaces' #filterRootInterfaces)
	('leaf interfaces' #filterLeafInterfaces)
	('extended interfaces' #filterExtendedInterfaces)
	('extending interfaces' #filterExtendingInterfaces)
	)! !

!SIInterfaceFinder class methodsFor: 'menu arrays' stamp: 'MU 5/13/2000 00:48'!
interfaceListMenuArray
	^#(
	('browse' #browseInterface)
	('repertoire' #browseInterfaceRepertoire)
	('definition' #browseInterfaceDefinition)
	-
	('conformers' #listConformerClasses)
	('all conformers' #listAllConformerClasses)
	-
	('create stub class...' #createStubClass)
	('extend interface...' #extendInterfaces)
	('unextend interface...' #unextendInterfaces)
	-
	('toggle seal' #toggleSealInterface)
	)! !


!SIInterfaceFinder class methodsFor: 'constants' stamp: 'M.U 6/19/1999 15:13'!
classMenuTitle
	^'Class menu'! !

!SIInterfaceFinder class methodsFor: 'constants' stamp: 'M.U 6/19/1999 15:13'!
interfaceMenuTitle
	^'Interface menu'! !


SIGuiModel subclass: #SIInterfaceSettingPanel
	instanceVariableNames: 'configuration configEntryList configEntryListSelection '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallInterfaces Squeak GUI'!
!SIInterfaceSettingPanel commentStamp: '<historical>' prior: 0!
Below properties can be set:

* defaultCategory  <String> ('Interfaces') indicates the interface default directory.

* discardInterfacesOnShutDown	<Boolean> (true) if true, cached interface repertoire will be released in snapshot.
* notifyOnDerivationDiscovery		<Boolean> (true) if true, user will be notifyied in a discovery of derivation from an interface. 
* removeOnlyStubs <Boolean> (true) if true, only stubbed methods will be removed.

* selectorForClassInterfaces <Symbol> (#interfaces) which interfaces should be shown in the definition string.
* selectorForInterfaceInheritance <Symbol> (#parents) which interfaces should be shown in the definition string.

* synchronize <Boolean> (true) if true, an interface change be synchronized with affected relatives (conformers & extenders).
* synchronizeSelectively <Boolean> (true) if true, you can selectively choose which relatives to upgrade/downgrade.
* synchronizeWithConformers <Boolean> (true) synchronize an interface change with conforming behaviors.
* synchronizeWithExtenders <Boolean> (true) synchronize an interface change with extending interfaces

---
#creator
[:masashi | ^umezawa] 
#version 2.0 MU 5/12/2000 17:16!


!SIInterfaceSettingPanel methodsFor: 'initialize-release' stamp: 'MU 4/20/2000 00:12'!
initialize
	super initialize.
	configuration := Interface configuration.
	configEntryList := self class configurationEntryList.
	! !


!SIInterfaceSettingPanel methodsFor: 'selection changed'!
configEntryListSelection: anSelection

	configEntryListSelection := anSelection.
	self changed: #configEntryListSelection.
	self changed: #configEntryValue.! !


!SIInterfaceSettingPanel methodsFor: 'actions'!
configEntryValue
	self configEntryListSelection isNil ifTrue:[^''].
	^(self configuration perform: (self configEntryListSelection asSymbol) ) printString.! !

!SIInterfaceSettingPanel methodsFor: 'actions' stamp: 'MU 4/20/2000 00:04'!
openHelp
	
	^(StringHolder new contents: self helpString) openLabel: 'SmallInterfaces Configuration Help'! !

!SIInterfaceSettingPanel methodsFor: 'actions' stamp: 'M.U 6/19/1999 15:05'!
perform: menuCallBack orSendTo: otherTarget 
	| value |
	value := menuCallBack == #fillInTheBlank
				ifTrue: [FillInTheBlank request: 'value?']
				ifFalse: [menuCallBack].
	value == #true ifTrue: [value := true].
	value == #false ifTrue: [value := false].
	self configuration perform: (self configEntryListSelection , ':') asSymbol with: value.
	self changed: #configEntryValue.! !


!SIInterfaceSettingPanel methodsFor: 'menu constants' stamp: 'M.U 6/19/1999 15:35'!
configMenuTitle
	^self class configMenuTitle! !


!SIInterfaceSettingPanel methodsFor: 'list menu creation'!
configEntryMenu: aCustomMenu 
	| menuArrGetSelector |
	menuArrGetSelector := self configEntryListSelection , 'MenuArray'.
	^ aCustomMenu addList: (self class perform: menuArrGetSelector asSymbol).! !


!SIInterfaceSettingPanel methodsFor: 'accessing'!
configEntryList

	^configEntryList! !

!SIInterfaceSettingPanel methodsFor: 'accessing'!
configEntryList: aValue
	
	configEntryList := aValue.
	! !

!SIInterfaceSettingPanel methodsFor: 'accessing'!
configEntryListSelection

	^configEntryListSelection! !

!SIInterfaceSettingPanel methodsFor: 'accessing'!
configuration

	^configuration! !

!SIInterfaceSettingPanel methodsFor: 'accessing'!
configuration: aValue

	configuration := aValue! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SIInterfaceSettingPanel class
	instanceVariableNames: ''!

!SIInterfaceSettingPanel class methodsFor: 'interface opening' stamp: 'MU 5/6/2000 23:09'!
openInMVC
	"SIInterfaceSettingPanel openInMVC"

	| model entryView  valueView topView |
	model := self new.
	entryView :=
		PluggableListViewByItem on: model
			list: #configEntryList
			selected: #configEntryListSelection
			changeSelected: #configEntryListSelection:
			menu: #configEntryMenu:.
	entryView  menuTitleSelector: #configMenuTitle.
	valueView :=
		PluggableTextView on: model
			text: #configEntryValue
			accept: #configEntryValue:.
	valueView controller: ReadOnlyTextController new.
	topView := StandardSystemView new
		label: self name;
		minimumSize: 300@200;
		addSubView: entryView;
		addSubView: valueView toRightOf: entryView.
		
	topView borderWidth: 3.
	topView controller open.! !

!SIInterfaceSettingPanel class methodsFor: 'interface opening' stamp: 'MU 5/6/2000 23:09'!
openInMorphic
	"SIInterfaceSettingPanel openInMorphic openInMVC"
	| model enrtyMorph valueMorph window |
	model := self new.
	enrtyMorph := PluggableListMorphByItem
				on: model
				list: #configEntryList
				selected: #configEntryListSelection
				changeSelected: #configEntryListSelection:
				menu: #configEntryMenu:.
	enrtyMorph menuTitleSelector: #configMenuTitle.
	valueMorph := PluggableTextMorph
				on: model
				text: #configEntryValue
				accept: #configEntryValue:.
	window := (SystemWindow labelled: self name)
				model: model.
	window addMorph: enrtyMorph frame: (0 @ 0 corner: 0.5 @ 1).
	window addMorph: valueMorph frame: (0.5 @ 0 corner: 1 @ 1).
	window borderWidth: 3.
	window openInWorldExtent: 400 @ 200.
	^ window! !


!SIInterfaceSettingPanel class methodsFor: 'constants'!
configMenuTitle
	^'choose them'! !

!SIInterfaceSettingPanel class methodsFor: 'constants' stamp: 'MU 4/28/2000 02:07'!
configurationEntryList
	^InterfaceConfiguration configurationArray! !

!SIInterfaceSettingPanel class methodsFor: 'constants' stamp: 'MU 4/19/2000 23:59'!
helpString

	^Interface configuration helpString! !


!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:33'!
defaultCategoryMenuArray
	^#(
	('Interfaces(default)' 'Interfaces')
	('others' #fillInTheBlank)
	)! !

!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:11'!
discardInterfacesOnShutDownMenuArray
	^#(
	('true(default)' true)
	('false' false)
	)! !

!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:12'!
notifyOnDerivationDiscoveryMenuArray
	^#(
	('true' true)
	('false(default)' false)
	)! !

!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:12'!
removeOnlyStubsMenuArray
	^#(
	('true(default)' true)
	('false' false)
	)! !

!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:22'!
selectorForClassInterfacesMenuArray
	^#(
	( 'interfaces(default)' interfaces)
	( 'allInterfaces' allInterfaces)
	( 'repertoire' repertoire)
	)! !

!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:20'!
selectorForInterfaceInheritanceMenuArray
	^#(
	('conformers' conformers )
	('allConformers' allConformers)
	('parents(default)' parents )
	('siblings' siblings )
	('twins' twins )
	('children' children )
	('ancestors' ancestors )
	('descendants' descendants )
	('extended' extended )
	('extenders' extenders )
	('repertoire' repertoire)
	)! !

!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:26'!
synchronizeMenuArray
	^#(
	('true(default)' true)
	('false' false)
	)! !

!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:27'!
synchronizeSelectivelyMenuArray
	^#(
	('true(default)' true)
	('false' false)
	)! !

!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:26'!
synchronizeWithConformersMenuArray
	^#(
	('true(default)' true)
	('false' false)
	)! !

!SIInterfaceSettingPanel class methodsFor: 'menu arrays' stamp: 'MU 4/28/2000 02:26'!
synchronizeWithExtendersMenuArray
	^#(
	('true(default)' true)
	('false' false)
	)! !
