'From Squeak2.8alpha of 19 January 2000 [latest update: #2158] on 24 May 2000 at 12:51:25 am'!
"Name:		SmallInterfaces Squeak SysChanges
Date:			25 May 2000
Author:			Benny Sadeh
Porter:			Masashi Umezawa
Version:	2.0.0
Distribution:	World
Prerequisites: #('SmallInterfaces Squeak Util.st' )
Description:

Squeak system class changes for SmallInterfaces Squeak implementation.
Note that this changes are based on Squeak *2.8alpha*.

Future Squeak version might produce some conflicts with this change set.
But you can resolve with some minor fixes (I hope).

Copyright (C) 1999-2000 Benny Sadeh
Copyright (C) 1999-2000 Masashi Umezawa

URL: http://brain.cs.uiuc.edu/VisualWorks/SmallInterfaces (original)
URL: http://www.mars.dti.ne.jp/~umejava/smalltalk/stClasses/smallinterfaces/ (Squeak Port)

"!

!Object methodsFor: 'accessing'!
environment

	^self class environment! !

!Object methodsFor: 'accessing'!
starts: aSequenceableCollection
	"Answer whether the first element of aSequenceableCollection is equal to the receiver."

	^aSequenceableCollection first = self! !

!Object methodsFor: 'accessing'!
types
	"Which are all the types which I embody?
	Definition: Type = (Class | Interface)"

	^self class allSuperclasses 
		add: self class;
		addAll: self class allInterfaces;
		yourself! !

!Object methodsFor: 'testing'!
conformsTo: anInterface
	"Can I respond to any message in <anInterface> repertoire?"

	^self class understands: anInterface! !

!Object methodsFor: 'testing'!
dialog
	"#SmallInterfaces2.0.0#Squeak(added)"
	^SIDialog! !

!Object methodsFor: 'testing'!
ends: aSequenceableCollection
	"Answer whether the last element of aSequenceableCollection is equal to the receiver."

	^aSequenceableCollection last = self! !

!Object methodsFor: 'testing'!
equalsLiteral: literal
	"Answer whether the receiver which is a literal of a method
	 equals the literal."

	^self = literal! !

!Object methodsFor: 'testing'!
isClass
	"Answer whether the receiver is a class."

	^false! !

!Object methodsFor: 'testing'!
isInterface

	^false! !

!Object methodsFor: 'testing'!
isTypeOf: aClassOrInterface

	^aClassOrInterface isEmbodiedIn: self! !

!Object methodsFor: 'error handling'!
commentOnly
	"This message sets up a framework for the behavior of the class' subclasses.
	Raise a signal indicates that the subclass should have implemented this message."
	"#SmallInterfaces2.0.0#Squeak(added)"

	^self error: 'comment only'! !

!Object methodsFor: 'indicating'!
shouldImplementFor: anInterface 
	"This method is used to mark a method as one generated (and stubbed) for an interface."

	^anInterface implementorsResponsibility! !

!Object methodsFor: 'indicating'!
subclassesShouldOverride
	"Unlike #subclassResponsibility, this is a recommendation only."! !


!Behavior methodsFor: 'testing'!
specialSelectorIndexFor: selector
	"Private - Answer the special selector index for the vm."
	"#SmallInterfaces2.0.0#Squeak(not used)"
	^CompiledMethod specialSelectorIndexFor: selector! !

!Behavior methodsFor: 'accessing method dictionary'!
removeSelector: selector ifAbsent: exceptionHandler

	(self methodDictionary includesKey: selector) ifFalse: [^exceptionHandler value].
	^self removeSelector: selector! !

!Behavior methodsFor: 'testing method dictionary'!
whichMethodsReferenceLiteral: literal
	"Answer the methods defined by the receiver which reference the
	 literal object."

	| methods |
	methods := OrderedCollection new.
	self methodsDo: [:cm |
		(cm referencesLiteral: literal) ifTrue: [
			methods add: cm]].
	^methods! !

!Behavior methodsFor: 'private'!
errorAbsentSelector: selector
	"Private - Handle the error of the receiver looking for a method named
	 selector when it does not define one."

	^self error: self name, '>>#', selector, ' is not defined'! !


!BlockContext methodsFor: 'SmallInterfaces'!
equalsLiteral: literal
	"#SmallInterfaces2.0.0#Squeak(modified)"
	"Answer whether the receiver's method references the literal."

	^self == literal or: [
		self method equalsLiteral: literal]! !


!ChangeSet methodsFor: 'change logging' stamp: 'MU 5/22/2000 01:03'!
noteNewMethod: newMethod forClass: class selector: selector priorMethod: methodOrNil

	| changedEvent |
	"#SmallInterfaces2.0.0#Squeak(appended)"
	changedEvent := SIClassDefinitionChangedEvent source: self aspect: #addSelector:class:.
	changedEvent behavior: class.
	changedEvent selector: selector. 
	self class changed: changedEvent.
	
	class wantsChangeSetLogging ifFalse: [^ self].
	isolationSet ifNotNil:
		["If there is an isolation layer above me, inform it as well."
		isolationSet noteNewMethod: newMethod forClass: class selector: selector
				priorMethod: methodOrNil].
	(self changeRecorderFor: class)
		noteNewMethod: newMethod selector: selector priorMethod: methodOrNil
! !

!ChangeSet methodsFor: 'change logging' stamp: 'di 3/29/2000 12:29'!
removeSelector: selector class: class priorMethod: priorMethod lastMethodInfo: info
	| changedEvent |
	"Include indication that a method has been forgotten.
	info is a pair of the source code pointer and message category
	for the method that was removed."

	"#SmallInterfaces2.0.0#Squeak(appended)"
	changedEvent := SIClassDefinitionChangedEvent source: self aspect: #removeSelector:class:.
	changedEvent behavior: class.
	changedEvent selector: selector. 
	self class changed: changedEvent.
	
	class wantsChangeSetLogging ifFalse: [^ self].
	isolationSet ifNotNil:
		["If there is an isolation layer above me, inform it as well."
		isolationSet removeSelector: selector class: class
				priorMethod: priorMethod lastMethodInfo: info].
	(self changeRecorderFor: class)
		noteRemoveSelector: selector priorMethod: priorMethod lastMethodInfo: info
! !

!ChangeSet methodsFor: 'class changes' stamp: 'di 4/1/2000 23:17'!
noteRemovalOf: class
	| changedEvent |
	"The class is about to be removed from the system.
	Adjust the receiver to reflect that fact."

	"#SmallInterfaces2.0.0#Squeak(appended)"
	changedEvent := SIClassDefinitionChangedEvent source: self aspect: #removeClass:.
	changedEvent behavior: class.
	self class changed: changedEvent.

	(self changeRecorderFor: class)
		noteChangeType: #remove fromClass: class.
	changeRecords removeKey: class class name ifAbsent: [].! !


!ClassBuilder methodsFor: 'validation'!
validateClassName: aString
	"Validate the new class name"
	"#SmallInterfaces2.0.0#Squeak(modified)"
	((self environment at: #Interface ifAbsent:[] ) isNil and:[ aString first isLowercase ]) ifTrue:[
		self error: 'Class names must be capitalized'.
		^false].
	environ at: aString ifPresent:[:old|
		(old isKindOf: Behavior) ifFalse:[
			self notify: aString asText allBold, 
						' already exists!!\Proceed will store over it.' withCRs]].
	^true! !


!ClassDescription methodsFor: 'printing'!
definitionWithInterfaces 

	| writer classInterfacesSelector classInterfaces |
	writer := String newLarge writeStream.
	writer
		nextPutAll: self definition;
		crtab;
		nextPutAll: 'interfaces: ';
		nextPut: $'.
	classInterfacesSelector := Interface configuration selectorForClassInterfaces.
	classInterfaces := (self perform: classInterfacesSelector) asSortedCollection: Class sortBlock.
	classInterfaces do: [:each | writer nextPutAll: each name asString; space].
	writer nextPut: $'.
	^writer contents! !

!ClassDescription methodsFor: 'printing'!
notify: requestor onDiscoveryOf: interfaces
	"#SmallInterfaces2.0.0#Squeak(modified)"
	(Interface configuration notifyOnDerivationDiscovery and:
	[interfaces notEmpty and: 
	[requestor notNil]])
		ifTrue: 
			[| writer |
			writer := String new writeStream.
			writer nextPutAll: self printString; nextPutAll: ' is now also conforming to: '.
			(interfaces asSortedCollection: Class sortBlock) do: 
				[:each | writer cr; nextPutAll: '   - '; nextPutAll: each printString].
			self dialog warn: writer contents]! !

!ClassDescription methodsFor: 'organization'!
allSelectorsReferencing: literal

	^(self whichMethodsReferenceLiteral: literal) collect: [:each | each selector]! !

!ClassDescription methodsFor: 'organization'!
removeAllEmptyCategories

	self organization removeEmptyCategories! !

!ClassDescription methodsFor: 'repertoire'!
repertoire
	"Note: the original implementation was - ^self allSelectors.
	However, since this was very slow, a new caching scheme can be used instead."

	^RepertoireHome current repertoireFor: self! !

!ClassDescription methodsFor: 'repertoire'!
repertoireImplemented

	^self selectors! !

!ClassDescription methodsFor: 'repertoire'!
resetRepertoire

	RepertoireHome current clearEntryFor: self! !

!ClassDescription methodsFor: 'repertoire'!
selectorsForRepertoire

	^self subclassResponsibility! !

!ClassDescription methodsFor: 'synchronizing'!
selectToSynchronizeFrom: behaviors
	"#SmallInterfaces2.0.0#Squeak(modified)"
	^Interface configuration synchronize 
		ifTrue: 
			[| candidates | 
			candidates := behaviors asSortedCollection: Class sortBlock.
			(candidates notEmpty and:
			[Interface configuration synchronizeSelectively])
				ifTrue: 
					[self dialog
						chooseMultiple: 'Choose which candidates to synchronize:'
						from: candidates]
			ifFalse: [candidates]]
		ifFalse: [#()].! !

!ClassDescription methodsFor: 'synchronizing'!
synchronizeChangeOf: aspect usingSelector: selector

	(aspect == #removeClass:) 
		ifTrue: 
			[RepertoireHome current 
				clearEntryFor: self;
				clearEntryFor: self class]! !

!ClassDescription methodsFor: 'synchronizing'!
synchronizeWith: anInterface removalOf: selector 

	((anInterface methodsStubedIn: self) includes: selector) 
		ifTrue: [self removeMethods: (Array with: selector)]! !

!ClassDescription methodsFor: 'message creation-removal'!
detachFromAllInterfacesBut: interfaces
	"Note: Since interfaces can overlap, we need to remove only the non-overlapping portion."

	| interfacesToDiscard discardable collective |
	interfacesToDiscard := self allInterfaces - interfaces.
	discardable := interfacesToDiscard 
		inject: Set new
		into: [:repertoire :each | repertoire addAll: (self detachableMethodsFor: each); yourself].
	collective := interfaces
		inject: Set new
		into: [:repertoire :each | repertoire addAll: (self detachableMethodsFor: each); yourself].
	self removeMethods: (discardable - collective).! !

!ClassDescription methodsFor: 'message creation-removal'!
detachableMethodsFor: anInterface

	^Interface configuration removeOnlyStubs
		ifTrue: [anInterface methodsStubedIn: self]
		ifFalse: [anInterface methodsImplementedBy: self]! !

!ClassDescription methodsFor: 'message creation-removal'!
implement: anInterface 
	"Create implementations for all of <anInterface> methods which I do not understand yet."

	self implement: anInterface enforce: false! !

!ClassDescription methodsFor: 'message creation-removal'!
implement: anInterface enforce: enforcing
	"Create methods implementations on behalf of <anInterface>.
	If <enforcing>, create implementations for all of <anInterface>'s repetoire (rather then merely 
	understand it). If not, create implementations only for the part I do not understand yet."

	| selectorsToBeImplemented |
	selectorsToBeImplemented := (enforcing or: [anInterface mustBeImplemented])
		ifTrue: [anInterface methodsNotImplementedBy: self]
		ifFalse: [anInterface methodsNotUnderstoodBy: self].
	selectorsToBeImplemented do: [:each | anInterface createStubedMethodFor: each from: self]! !

!ClassDescription methodsFor: 'message creation-removal'!
implementAll: interfaces 

	interfaces do: [:each | self implement: each]! !

!ClassDescription methodsFor: 'message creation-removal'!
implementAll: interfaces discardExisting: discarding 
	"Create implementations for <interfaces>.
	If <discarding>, discard existing implementations of interfaces not in <interfaces>."

	discarding ifTrue: [self detachFromAllInterfacesBut: interfaces].
	self implementAll: interfaces.
	self resetRepertoire.! !

!ClassDescription methodsFor: 'message creation-removal'!
implementInterfacesNamed: stringOfInterfaces 

	self 
		implementAll: (Interface interfacesFromString: stringOfInterfaces)
		discardExisting: true.! !

!ClassDescription methodsFor: 'message creation-removal'!
implementationStringForInterface

	^String newLarge writeStream
		nextPutAll: '^self';
		space;
		nextPutAll: Interface selectorForImplementation asString;
		contents! !

!ClassDescription methodsFor: 'message creation-removal'!
removeMethods: selectors

	selectors do: [:each | self removeSelector: each ifAbsent: []].
	self removeAllEmptyCategories.! !

!ClassDescription methodsFor: 'message creation-removal'!
unimplement: anInterface
	"Remove implementations for all of <anInterface> repertoire I already implement.
	If <removeOnlyStubbs>, remove only the the stubbed implementations.
	Otherwise, remove all methods implemented on behalf of <anInterface>.
	Note: methods I understand but do not implement would not be removed"

	self removeMethods: (self detachableMethodsFor: anInterface).! !

!ClassDescription methodsFor: 'interfaces creation'!
asInterface

	^self asInterfaceNamed: self defaultInterfaceName! !

!ClassDescription methodsFor: 'interfaces creation'!
asInterfaceNamed: aSymbol

	(Interface home includesInterfaceNamed: aSymbol) 
		ifTrue: [^self error: aSymbol asString, ' interface already exists!!'].

	^Interface newNamed: aSymbol from: self! !

!ClassDescription methodsFor: 'querying'!
commentFor: selector
		"Return the comment entered for selector."
	"#SmallInterfaces2.0.0#Squeak(not used)"
	| cm comments |
	cm := (self compiledMethodAt: selector ifAbsent: [^'']).
	comments := (Parser new) parseMethodComment: cm getSource setPattern: [:x |].
	^comments notEmpty
		ifTrue: [comments first]
		ifFalse: ['']! !

!ClassDescription methodsFor: 'querying'!
selectorsInCategory: categoryName 

	Symbol 
		hasInterned: categoryName 
		ifTrue: [:sym | ^self organization listAtCategoryNamed: sym].
	^#()! !

!ClassDescription methodsFor: 'interfaces'!
allInterfaces

	^Interface home allInterfaces select: [:each | self understands: each]! !

!ClassDescription methodsFor: 'interfaces'!
interfaces
	"The minimal set of interfaces which fully describe the repertoire of all interfaces I understand."

	^Interface consolidate: self allInterfaces! !

!ClassDescription methodsFor: 'interfaces'!
understands: anInterface
	"Can my instances respond to all of <anInterface> repertoire?
	Note the difference between implementing and understanding."

	^anInterface isUnderstoodBy: self! !

!ClassDescription methodsFor: 'interfaces'!
understandsAll: collectionOfInterfaces
	"This method aims to help with design validation"
	
	^collectionOfInterfaces
		inject: true
		into: 
			[:truth :interface |
			truth and: [self understands: interface]]! !

!ClassDescription methodsFor: 'interfaces'!
understandsRepertoire: repertoire
	"Can I understand all of repertoire (collection of messages)?"

	^self repertoire includesAll: repertoire! !

!ClassDescription methodsFor: 'documentation'!
providedInterfaces
	"This method is a template which aims to facilitate documenting the interfaces a class should be providing.
	At that point the question <aBehavior suppliesProvidedInterfaces> becomes meaningful."
	
	self subclassesShouldOverride.
	^#()! !

!ClassDescription methodsFor: 'documentation'!
requiredInterfaces
	"This method is a template which aims to facilitate documenting the interfaces a class should be able to understand.
	At that point the question <aBehavior understandsRequiredInterfaces> becomes meaningful."
	
	self subclassesShouldOverride.
	^#()! !

!ClassDescription methodsFor: 'documentation'!
understandsRequiredInterfaces
	"This method aims to help with design validation"
	
	^self requiredInterfaces
		inject: true
		into: 
			[:truth :interface |
			truth and: [self understands: interface]]! !


!Class methodsFor: 'accessing'!
homeClass
	"I am the home class, so me."

	^self! !

!Class methodsFor: 'testing'!
isClass
	"Answer whether the receiver is a class."

	^true! !

!Class methodsFor: 'testing'!
isEmbodiedIn: anObject

	^anObject isKindOf: self! !

!Class methodsFor: 'compiling'!
compile: code classified: heading notifying: requestor
	"#SmallInterfaces2.0.0#Squeak(modified)"
	| before result after |
	before := self interfaces.
	result := super compile: code classified: heading notifying: requestor. 
	after := self interfaces.
	self 
		notify: requestor 
		onDiscoveryOf: (after - before).
	self resetRepertoire.
	^result! !

!Class methodsFor: 'subclass creation'!
subclass: className
	instanceVariableNames: stringOfInstVars
	classVariableNames: stringOfClassVars
	poolDictionaries: stringOfPoolNames
	category: category
 	interfaces: stringOfInterfaces
	"Create or modify class so it would adhere to the declared Interfaces."

	| class |
	class := self
		subclass: className
		instanceVariableNames: stringOfInstVars
		classVariableNames: stringOfClassVars
		poolDictionaries: stringOfPoolNames
		category: category.
	class implementInterfacesNamed: stringOfInterfaces.		
	^class! !

!Class methodsFor: 'subclass creation'!
variableByteSubclass: className
	instanceVariableNames: stringOfInstVars
	classVariableNames: stringOfClassVars
	poolDictionaries: stringOfPoolNames
	category: category
 	interfaces: stringOfInterfaces
	"Create or modify class so it would adhere to the declared Interfaces."

	| class |
	class := self
		variableByteSubclass: className
		instanceVariableNames: stringOfInstVars
		classVariableNames: stringOfClassVars
		poolDictionaries: stringOfPoolNames
		category: category.
	class implementInterfacesNamed: stringOfInterfaces.		
	^class! !

!Class methodsFor: 'subclass creation'!
variableSubclass: className
	instanceVariableNames: stringOfInstVars
	classVariableNames: stringOfClassVars
	poolDictionaries: stringOfPoolNames
	category: category
 	interfaces: stringOfInterfaces
	"Create or modify class so it would adhere to the declared Interfaces."

	| class |
	class := self
		variableSubclass: className
		instanceVariableNames: stringOfInstVars
		classVariableNames: stringOfClassVars
		poolDictionaries: stringOfPoolNames
		category: category.
	class implementInterfacesNamed: stringOfInterfaces.		
	^class! !

!Class methodsFor: 'repertoire'!
selectorsForRepertoire

	^self allSelectors asIdentitySet! !

!Class methodsFor: 'updating'!
synchronizeWith: anInterface removalOf: selector 

	((anInterface methodsStubedIn: self) includes: selector) 
		ifTrue: 
			[self removeSelector: selector ifAbsent: [].
			self removeAllEmptyCategories]! !

!Class methodsFor: 'interfaces creation'!
defaultInterfaceName

	^(self name, 'Interface') asSymbol! !


!Class class methodsFor: 'instance creation'!
templateWithInterfaces
	"Answer an expression that can be edited and evaluated in order to define a new interface."

^'Interface subclass: #NameOfInterface
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''''
	interfaces: '''' '! !

!Class class methodsFor: 'instance creation'!
templateWithInterfaces: category
	"Answer an expression that can be edited and evaluated in order to define a new interface."

	^(self template: category),
'	interfaces: '''' '! !

!Class class methodsFor: 'sorting'!
sortBlock
		"Answer a sort block for sorting classes alphabetically."

	^[:a :b | a name <= b name]! !


!Collection methodsFor: 'adding'!
, aCollection 
	"Answer a copy of the receiver agregated with the argument, aCollection."

	^self copy 
		addAll: aCollection;
		yourself! !

!Collection methodsFor: 'converting'!
asIdentitySet

	^IdentitySet withAll: self! !

!Collection methodsFor: 'converting'!
flattened
	"Answer a new collection containing all my elements, flattened,
	which means: if an element is a collection, then flatten it as well (recursively).
	<#(1 #(2 3 4) #(10 #(12 13 14 #(#(100) 200 #(251 252) 300 400)))) flattened>"

	| newCollection |
	newCollection := self species new: self size.
	self do: 
		[:each | 
		each isCollection
			ifTrue: [newCollection addAll: each flattened]
			ifFalse: [newCollection add: each]].
	^newCollection! !

!Collection methodsFor: 'enumerating'!
keysAndValuesDo: a2ArgBlock 
	"#SmallInterfaces2.0.0#Squeak(example bug fixed) "
	"Evaluate <a2ArgBlock> with each of the receiver's key/value pairs as the arguments,
	where an arbitrary index is the key, and element is the value.
	<'ABC' asSet keysAndValuesDo: [:index :each | Transcript show: index printString; tab; show: each printString; cr]>"

	| index |
	index := 0.
	self do: [:each | a2ArgBlock value: (index := index + 1) value: each]! !

!Collection methodsFor: 'testing'!
consistsOf: aCollection 
	"Returns true if the contents of the receiver and <aCollection> 
	are exactly the same - order doesn't matter."

	^self size = aCollection size and: [self includesAllOccurrences: aCollection]! !

!Collection methodsFor: 'testing'!
contains: aBlock 
	"#SmallInterfaces2.0.0#Squeak(added)"

	self detect: aBlock ifNone: [^false].
	^true! !

!Collection methodsFor: 'testing'!
includesAll: aCollection
	"Answer whether the receiver includes each element of aCollection."

	self size < aCollection size ifTrue: [^false].
	aCollection do: [:each | (self includes: each) ifFalse: [^false]].
	^true! !

!Collection methodsFor: 'testing'!
includesAllOccurrences: aCollection 
	"Return true if the receiver includes all elements in aCollection. 
	If aCollection contains duplicate elements, the receiver must have one for each of them."

	| meAsBag otherBag |
	aCollection size > self size ifTrue: [^false].
	otherBag := aCollection asBag.
	meAsBag := self asBag.
	otherBag do: [:each | (meAsBag occurrencesOf: each) < (otherBag occurrencesOf: each) ifTrue: [^false]].
	^true! !

!Collection methodsFor: 'testing'!
includesAny: aCollection
	"Return true if the receiver includes any of the elements in aCollection."

	aCollection do: [:each | (self includes: each) ifTrue: [^true]].
	^false! !

!Collection methodsFor: 'set operations'!
intersectionWith: anotherCollection
	"Return the elements in common with receiver and anotherCollection 
	as an appropriate collection with one instance of each element."

	^anotherCollection intersectionWithSet: self asSet! !

!Collection methodsFor: 'set operations'!
intersectionWithIdentitySet: identitySet
	"Return the elements in common with receiver and identitySet 
	as a set with one instance of each element."

	^self intersectionWithSet: identitySet! !

!Collection methodsFor: 'set operations'!
intersectionWithSet: aSet
	"Return the elements in common with receiver and anotherCollection 
	as an appropriate collection with one instance of each element."

	^self asSet intersectionWithSet: aSet! !


!Collection class methodsFor: 'instance creation'!
newLarge

	^self new: 100! !


!LookupKey methodsFor: 'testing'!
equalsLiteral: literal
	"Answer whether the receiver, its key or value is equal to the literal.
	 Assumes the key and literal would both be symbols."

	^self == literal or: [self key == literal or: [self value == literal]]! !


!Metaclass methodsFor: 'accessing'!
homeClass
	"I am a metaclass - return the class I am meta for."

	^self soleInstance! !

!Metaclass methodsFor: 'testing'!
conformsTo: anInterface
	"Can I respond to any message in <anInterface> repertoire?"

	^self understands: anInterface! !

!Metaclass methodsFor: 'interfaces creation'!
defaultInterfaceName

	^(self homeClass name, 'Factory') asSymbol! !

!Metaclass methodsFor: 'repertoire'!
selectorsForRepertoire

	^RepertoireHome genericMetaRepertoire 
		addAll: self selectors;
		yourself! !

!Metaclass methodsFor: 'compiling-st80'!
classImplementationErrorMessage 

	^'Sorry, but the semantics of an Interface prohibit it from having any implementation on its class side...'! !

!Metaclass methodsFor: 'compiling-st80'!
compile: code classified: heading notifying: requestor 
	"#SmallInterfaces2.0.0#Squeak(modified)"
	| before result after selector |
	(self homeClass isInterface and: 
	[selector := SIParserUtil parseMethodSelectorOf: code.
	(Interface compilableMetaSelectors includes: selector) not])
		ifTrue: 
			[requestor notNil ifTrue: [self dialog warn: self classImplementationErrorMessage].
			^nil].

	before := self interfaces.
	result := super compile: code classified: heading notifying: requestor. 
	after := self interfaces.
	self 
		notify: requestor 
		onDiscoveryOf: (after - before).
	self resetRepertoire.
	^result! !

!Metaclass methodsFor: 'compiling-st80'!
instanceVariableNames: stringOfInstVars interfaces: stringOfInterfaces 
	"#SmallInterfaces2.0.0#Squeak(modified)"
	self homeClass isInterface 
		ifTrue: 
			[self dialog warn: self classImplementationErrorMessage.
			^nil].
	
	self instanceVariableNames: stringOfInstVars.
	self implementInterfacesNamed: stringOfInterfaces.! !


!SequenceableCollection methodsFor: 'testing'!
ends: aSequenceableCollection 
	"Answer whether the last elements of aSequenceableCollection are equal to the elements of the reciever."

	^self reverse starts: aSequenceableCollection reverse! !

!SequenceableCollection methodsFor: 'testing'!
endsWith: aSequenceableCollectionOrObject  
	"if aSequenceableCollectionOrObject is a <SequenceableCollection>, answer whether the 
	last elements of the receiver, respectively, are equal to the elements of the <SequenceableCollection>.
	if aSequenceableCollectionOrObject is any other kind of object, answer whether the 
	last element of the receiver equals to that <Object>."

	^aSequenceableCollectionOrObject ends: self! !

!SequenceableCollection methodsFor: 'testing'!
starts: aSequenceableCollection 
	"Answer whether the first elements of aSequenceableCollection are equal to the elements of the reciever.
	<'in' starts: 'in the beginning...' >"

	(self isEmpty or:
	[aSequenceableCollection size < self size]) ifTrue: [^false].
		
	1 to: self size do: [:each | (aSequenceableCollection at: each) = (self at: each) ifFalse: [^false]].
	^true! !


!ArrayedCollection methodsFor: 'converting'!
flattened
	"Answer a new collection containing all my elements, flattened,
	which means: if an element is a collection, then flatten it as well (recursively).
	Note: overriding super since it is adding to an ArrayedCollection is not appropriate."

	| newCollection |
	newCollection := OrderedCollection new: self size.
	self do: 
		[:each | 
		each isCollection
			ifTrue: [newCollection addAll: each flattened]
			ifFalse: [newCollection add: each]].
	^newCollection! !


!Array methodsFor: 'testing'!
equalsLiteral: literal
	"Answer whether the receiver is the same as the literal or
	 contains it."

	self = literal ifTrue: [^true].
	self do: [:element | element = literal ifTrue: [^true]].
	^false! !


!CompiledMethod methodsFor: 'accessing'!
methodClass
	"Answer the class which contains the receiver."

	^mclass! !

!CompiledMethod methodsFor: 'accessing'!
sourceString
	"Return the source code for the receiver."

	^self getSource! !

!CompiledMethod methodsFor: 'printing'!
commentString
	"#SmallInterfaces2.0.0#Squeak(modified)(need RB)"
	^SIParserUtil parseMethodCommentOfWithQuote: self sourceString! !

!CompiledMethod methodsFor: 'printing'!
headerString
	"#SmallInterfaces2.0.0#Squeak(modified)(need RB)"
	^self sourceString isEmpty 
		ifTrue: [self selector asSelectorStringWithArgumentsString]
		ifFalse:[ SIParserUtil parseMethodHeaderOf: self sourceString]! !

!CompiledMethod methodsFor: 'SmallInterfaces'!
equalsLiteral: literal
	"#SmallInterfaces2.0.0#Squeak(added)"
	"Answer whether the receiver or a contained block
	 references literal. The object can either be an element
	 of an array or the key or value of an association."

	^self == literal or: [
		self referencesLiteral: literal]! !

!CompiledMethod methodsFor: 'SmallInterfaces'!
getSource
	"Return the source code for the receiver."
	"#SmallInterfaces2.0.0#Squeak(modified)(RB)"

	^self getSourceFromFile asString! !

!CompiledMethod methodsFor: 'SmallInterfaces'!
referencesLiteral: literal
	"#SmallInterfaces2.0.0#Squeak(modified)"
	"Answer whether the receiver or a contained block
	 references literal. The object can either be an element
	 of an array or the key or value of an association."

	self literalsDo: [:each |
		(each equalsLiteral: literal) ifTrue: [^true]].
	^false! !


!Set methodsFor: 'adding'!
, aCollection 
	"Answer a copy of the receiver concatenated with the argument, aCollection."
	
	^self copy
		addAll: aCollection asSet;
		yourself! !

!Set methodsFor: 'removing'!
- aSet
	"#SmallInterfaces2.0.0#Squeak(added)"

	^self reject: [ :i | aSet includes: i ]! !

!Set methodsFor: 'set operations'!
& aSet 
	"Set intersection"

	^self intersectionWith: aSet! !

!Set methodsFor: 'set operations'!
intersectionWith: aSet
	"Intersection of two sets."

	^self select: [:each | aSet includes: each]! !

!Set methodsFor: 'set operations'!
intersectionWithSet: aSet
	"Return the elements in common with receiver and aSet as a set with one instance of each element.
	<#(1 2 3) asSet intersectionWithSet: #(2 3 4) asSet>"

	^(self size <= aSet size)
		ifTrue: [self select: [:each | aSet includes: each]]
		ifFalse: [aSet intersectionWithSet: self]! !


!Dictionary methodsFor: 'set operations'!
intersectionWithSet: aSet
	"Return the elements in common with receiver and aSet 
	as a set with one instance of each value in receiver that is equal to one in aSet"

	^self values asSet intersectionWithSet: aSet! !


!IdentitySet methodsFor: 'converting'!
asIdentitySet

	^self! !


!Stream class methodsFor: 'instance creation'!
newLarge

	^self on: self contentsSpecies newLarge! !


!String methodsFor: 'converting'!
asArrayOfSubstrings
		"Answer an array of substrings from the
		 receiver.  The receiver is divided into
		 substrings at the occurrences of one or
		 more space characters."

	| aStream answer index |
	answer := OrderedCollection new.
	aStream := ReadStream on: self.
	[aStream atEnd]
		whileFalse: [
			[aStream atEnd ifTrue: [^answer asArray].
			 aStream peek > (TextConstants at: #Space)]
				whileFalse: [aStream next].
			index := aStream position + 1.
			[aStream atEnd or: [aStream peek <= (TextConstants at: #Space)]]
				whileFalse: [aStream next].
			answer add:
				(self copyFrom: index to: aStream position)].
	^answer asArray! !

!String methodsFor: 'converting'!
asClass
	"Answer the class I designate."

	^self asSymbol asClass! !

!String methodsFor: 'converting'!
asSequenceableCollection: aChar 
	"Answer a Collection of lines in me. 
	The end-of-line character is aChar."

	| lines lastEOL |
	lines := OrderedCollection new.
	lastEOL := 0.
	1 to: self size do: 
		[:i | 
		(self at: i) = aChar
			ifTrue: 
				[lines addLast: (self copyFrom: lastEOL + 1 to: i - 1).
				lastEOL := i]].
		"deal with last line if not terminated"
	lastEOL = self size ifFalse: [lines addLast: (self copyFrom: lastEOL + 1 to: self size)].
	^lines! !

!String methodsFor: 'converting'!
subStrings
	"Answer an array which contains the words in the receiver."

	^self asArrayOfSubstrings! !

!String methodsFor: 'converting'!
trimSeparators

	"Return a copy of the receiver that has leading and trailing separators removed."

	| size index |
	(size := self size) == 0 ifTrue: [^self].
	index := 1.
	[(self at: index) isSeparator] whileTrue: [
		index = size ifTrue: [^''].
		index := index + 1].
	[(self at: size) isSeparator] whileTrue: [
		index = size ifTrue: [^''].
		size := size - 1].
	^self copyFrom: index to: size! !


!Symbol methodsFor: 'converting'!
asClass
	"Answer the class I designate. Return nil if no such class exists.
	<#String asClass>
	<'Array' asClass>
	<'foo' asClass>"

	^Smalltalk allClasses 
		detect: [:each | each name == self]  
		ifNone: [nil]! !

!Symbol methodsFor: 'printing'!
asSelectorStringWithArgumentsString
	"<#at:put: asSelectorStringWithArgumentsString>
	<#halt asSelectorStringWithArgumentsString>
	<#@ asSelectorStringWithArgumentsString>"
	"#SmallInterfaces2.0.0#Squeak(modified)"
	

	| writer |
	self isUnary ifTrue: [^self asString].
	self isBinary ifTrue: [^self asString, ' anObject'].

	writer := WriteStream on: String newLarge.
	self keywords withIndexDo: 
		[:each :index |
		writer 
			nextPutAll: each;
			nextPutAll: ' arg';
			nextPutAll: index printString;
			space].
	^writer contents! !

!Symbol methodsFor: 'testing'!
isBinary
	"if I am to be interpreted as a message selector, am I a binary method? (such as #<).
	<#@ isBinary>
	<#== isBinary>
	<#halt isBinary>
	<#at: isBinary>"
	"#SmallInterfaces2.0.0#Squeak(modified)"

	^self numArgs = 1 and: [(self endsWith: ':') not]! !


!SystemDictionary methodsFor: 'querying'!
allNonInterfaceBehaviors

	| classes metaclasses |
	self isEmpty ifTrue: [^Set new].
	classes := self allNonInterfaceClasses.
	metaclasses := classes collect: [:each | each class].
	^(classes, metaclasses)! !

!SystemDictionary methodsFor: 'querying'!
allNonInterfaceClasses

	^Smalltalk allClasses reject: [:each | each isInterface]! !

!SystemDictionary methodsFor: 'testing'!
isHeadless
	"Answer whether the image is headless."

	^self includesKey: #HeadlessImage! !


!UndefinedObject class methodsFor: 'compiling-st80'!
subclass: className
	instanceVariableNames: stringOfInstVars
	classVariableNames: stringOfClassVars
	poolDictionaries: stringOfPoolNames
	category: category
 	interfaces: stringOfInterfaces
	"Create or modify class so it would adhere to the declared Interfaces."

	| class |
	class := self
		subclass: className
		instanceVariableNames: stringOfInstVars
		classVariableNames: stringOfClassVars
		poolDictionaries: stringOfPoolNames
		category: category.
	class implementInterfacesNamed: stringOfInterfaces.		
	^class! !

!UndefinedObject class methodsFor: 'compiling-st80'!
variableByteSubclass: className
	instanceVariableNames: stringOfInstVars
	classVariableNames: stringOfClassVars
	poolDictionaries: stringOfPoolNames
	category: category
 	interfaces: stringOfInterfaces
	"Create or modify class so it would adhere to the declared Interfaces."

	| class |
	class := self
		variableByteSubclass: className
		instanceVariableNames: stringOfInstVars
		classVariableNames: stringOfClassVars
		poolDictionaries: stringOfPoolNames
		category: category.
	class implementInterfacesNamed: stringOfInterfaces.		
	^class! !

!UndefinedObject class methodsFor: 'compiling-st80'!
variableSubclass: className
	instanceVariableNames: stringOfInstVars
	classVariableNames: stringOfClassVars
	poolDictionaries: stringOfPoolNames
	category: category
 	interfaces: stringOfInterfaces
	"Create or modify class so it would adhere to the declared Interfaces."

	| class |
	class := self
		variableSubclass: className
		instanceVariableNames: stringOfInstVars
		classVariableNames: stringOfClassVars
		poolDictionaries: stringOfPoolNames
		category: category.
	class implementInterfacesNamed: stringOfInterfaces.		
	^class! !

