"Name:		SmallInterfaces
Date:			20 May 2000
Author:			Benny Sadeh
Porter:			Masashi Umezawa
Version:	2.0.0
Distribution:	World
Prerequisites: #('SmallInterfaces Squeak SysChanges.cs' )
Description:

SmallInterfaces Squeak Port. (core module) 
Originally developed by Benny Sadeh.

'SmallInterfaces' brings you long-awaited 'interface' in Smalltalk.
Please see Interface class >> documentaion protocol for details.

Copyright (C) 1999-2000 Benny Sadeh
Copyright (C) 1999-2000 Masashi Umezawa

URL: http://brain.cs.uiuc.edu/VisualWorks/SmallInterfaces (original)
URL: http://www.mars.dti.ne.jp/~umejava/smalltalk/stClasses/smallinterfaces/ (Squeak Port)

"!

Object subclass: #Interface
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallInterfaces'!
!Interface commentStamp: '<historical>' prior: 0!
An Interface is an abstract Type, unlike Class which is a concrete Type. An interface specifies a set of messages that an object of any class implementing it would respond to. Therefore, it is orthogonal to Class, which propagates the implementation of a message to its instances. In other words, an interface specifies which messages an object will execute, where a class specifies how those messages will be executed.

For a more comprehensive explanation and a tutorial, look at the ''documentation'' protocol (on the class side) !


"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Interface class
	instanceVariableNames: ''!

!Interface class methodsFor: 'accessing-interfaces'!
ancestors
	"Return all interfaces which I extend (in bredth-first order).
	In other words: the lineage chain from the roots of my 'family tree'."

	| family elders ancestors |
	family := self extended.
		"now put an order within this family; oldest generation first, youngen last..."
	ancestors := OrderedCollection new: family size.
	elders := family select: [:each | each isOrphan].	
	[ancestors addAll: elders.
	family removeAll: elders.
	family notEmpty] whileTrue:
		[elders := family select: [:each | ancestors includesAny: each parents]].
	^ancestors! !

!Interface class methodsFor: 'accessing-interfaces'!
children
	"Return my children, which are the interfaces immidiately extending me."

	| family children |
	family := self extenders.
	family isEmpty ifTrue: [^family].
		"now gradually trim the family's youngest generation, until only 
		the oldest generation remains. these are my immediate children."
	[children := Interface consolidate: family.
	family removeAll: children.
	family notEmpty] whileTrue.
	^children! !

!Interface class methodsFor: 'accessing-interfaces'!
descendants

	^self progeny! !

!Interface class methodsFor: 'accessing-interfaces'!
equivalents

	^self home allInterfaces select: [:each | each ~~ self and: [each isEquivalentTo: self]]! !

!Interface class methodsFor: 'accessing-interfaces'!
extended
	"Return all intefaces which I extend directly or indirectly.
	In other words: all my originating interfaces."

	self isEmpty ifTrue: [^Set new].
	^(self home allNonEmptyInterfaces select: [:each | self extends: each]) 
		reject: [:each | self isEquivalentTo: each	"remove circularities"]! !

!Interface class methodsFor: 'accessing-interfaces'!
extenders
	"Return all intefaces which extend me (directly or indirectly).
	In other words: all (composite) interfaces for which I am a progenator."

	self isEmpty ifTrue: [^Set new].
	^(self home allInterfaces select: 
		[:each | each extends: self]) reject: 
			[:each | self isEquivalentTo: each]	"remove circularities"! !

!Interface class methodsFor: 'accessing-interfaces'!
parents
	"Return my parents, which are the interfaces I immidiately extend.
	Note: a simple (root) interface is an orphan, while a composite 
	interface is a child of unlimitted number of parents."

	^Interface consolidate: self extended! !

!Interface class methodsFor: 'accessing-interfaces'!
progeny
	"Return all interfaces which extend me (in bredth-first order).
	In other words: the branches of my 'family tree' eminating from me."

	| family children descendants |
	family := self extenders.
		"now put an order within this family; my children first, their children after ..."
	descendants := OrderedCollection new: family size.
	[children := Interface consolidate: family.	
	children isEmpty 	"case of existing equivalents"
		ifTrue: [children := family select: [:each | each hasEquivalents]].
	descendants addAll: children.
	family removeAll: children.
	family notEmpty] whileTrue.
	^descendants reversed! !

!Interface class methodsFor: 'accessing-interfaces'!
semiSiblings
	
	^self home allInterfaces select: [:each | each ~~ self and: [each parents includesAll: self parents]]! !

!Interface class methodsFor: 'accessing-interfaces'!
siblings
	"My siblings are considered to be semiSiblings; they have to share all of my parents.
	Note: For siblings = only as those that have identical parents, => use trueSiblings instead.
	For siblings = only as those that share at list one of the parents, => use stepSiblings instead."

	^self semiSiblings! !

!Interface class methodsFor: 'accessing-interfaces'!
stepSiblings
	
	^self home allInterfaces select: [:each | each ~~ self and: [each parents includesAny: self parents]]! !

!Interface class methodsFor: 'accessing-interfaces'!
trueSiblings
	
	^self home allInterfaces select: [:each | each ~~ self and: [each parents consistsOf: self parents]]! !

!Interface class methodsFor: 'accessing-interfaces'!
twins
	
	^self equivalents! !


!Interface class methodsFor: 'class creation'!
asClass

	^self asClassNamed: self defaultClassName! !

!Interface class methodsFor: 'class creation'!
asClassNamed: aSymbol

	^self 
		asClassNamed: aSymbol 
		super: Object! !

!Interface class methodsFor: 'class creation'!
asClassNamed: aSymbol super: aClass 

	^self
		asClassNamed: aSymbol
		super: aClass
		namespace: self environment! !

!Interface class methodsFor: 'class creation' stamp: 'MU 5/6/2000 14:04'!
asClassNamed: aSymbol super: aClass namespace: env
	"Note: <environment> is ignored for Smalltalks without namespaces."
	"#SmallInterfaces2.0.0#Squeak(modified)"
	| defaultCategory |
	defaultCategory := Interface configuration defaultCategory.
	aClass isFixed 
		ifTrue:
			[^aClass
				subclass: aSymbol
				instanceVariableNames: ''
				classVariableNames: ''
				poolDictionaries: ''
				category: defaultCategory
				interfaces: self name asString].		
	^aClass isVariable 
		ifTrue:
			[aClass isBits
				ifTrue:
					[aClass
						variableByteSubclass: aSymbol
						instanceVariableNames: ''
						classVariableNames: ''
						poolDictionaries: ''
						category: defaultCategory
						interfaces: self name asString]
				ifFalse:
					[aClass  
						variableSubclass: aSymbol
						instanceVariableNames: ''
						classVariableNames: ''
						poolDictionaries: ''
						category: defaultCategory
						interfaces: self name asString]]
		ifFalse: [self error: 'Sorry, can not handle this type of class.']! !

!Interface class methodsFor: 'class creation'!
asMetaclass

	^self asMetaclassNamed: self defaultClassName! !

!Interface class methodsFor: 'class creation'!
asMetaclassNamed: aSymbol

	^self 
		asMetaclassNamed: aSymbol 
		super: Object! !

!Interface class methodsFor: 'class creation'!
asMetaclassNamed: aSymbol super: aClass 

	^self
		asMetaclassNamed: aSymbol
		super: aClass
		namespace: self environment! !

!Interface class methodsFor: 'class creation'!
asMetaclassNamed: aSymbol super: aClass namespace: aNameSpace 

	| meta |
		"a quick and dirty way..."
	meta := (self
				asClassNamed: aSymbol
				super: aClass
				namespace: aNameSpace) class.
	meta removeMethods: self repertoire.
	meta implement: self! !


!Interface class methodsFor: 'interface calculus'!
& anInterface

	^self intersectionWith: anInterface! !

!Interface class methodsFor: 'interface calculus'!
+ anInterface

	^self unionWith: anInterface! !

!Interface class methodsFor: 'interface calculus'!
-  anInterface

	^self differenceFrom: anInterface! !

!Interface class methodsFor: 'interface calculus'!
differenceFrom: anInterface
 
	^self 
		differenceFrom: anInterface 
		named: (self name , 'Minus' , anInterface name) asSymbol! !

!Interface class methodsFor: 'interface calculus'!
differenceFrom: anInterface named: aSymbol

	| repertoire interface |
	repertoire := self repertoire reject: [:each| anInterface repertoire includes: each].
	interface := (self home interfaceWithRepertoire: repertoire) 
		ifNil: 
			[Interface 
				newNamed: aSymbol 
				withSelectors: repertoire].
	^interface! !

!Interface class methodsFor: 'interface calculus'!
intersectionWith: anInterface 

	^self 
		intersectionWith: anInterface 
		named: (self name , 'IntersectionWith' , anInterface name) asSymbol! !

!Interface class methodsFor: 'interface calculus'!
intersectionWith: anInterface named: aSymbol

	| repertoire interface |
	repertoire := self repertoire & anInterface repertoire.
	interface := (self home interfaceWithRepertoire: repertoire) 
		ifNil: 
			[Interface 
				newNamed: aSymbol 
				withSelectors: repertoire].
	^interface! !

!Interface class methodsFor: 'interface calculus'!
unionWith: anInterface 

	^self 
		unionWith: anInterface 
		named: (self name , 'Plus' , anInterface name) asSymbol! !

!Interface class methodsFor: 'interface calculus' stamp: 'MU 4/26/2000 23:36'!
unionWith: anInterface named: aSymbol
	"#SmallInterfaces2.0.0#Squeak(modified) +operator is already defined in SQ"
	| repertoire interface |
	repertoire := self repertoire , anInterface repertoire.
	interface := (self home interfaceWithRepertoire: repertoire) 
		ifNil: 
			[Interface 
				newNamed: aSymbol 
				withSelectors: repertoire].
	^interface! !


!Interface class methodsFor: 'printing'!
ancestorsString

	| writer ancestors  |
	writer := String newLarge writeStream.
	(ancestors := self ancestors) notEmpty
		ifTrue:
			[writer 
				crtab; 
				nextPutAll: 'ancestors: {'.
			ancestors do: [:each | writer nextPutAll: each name asString; space].
			writer 
				skip: -1; 
				nextPutAll: '}'].
	^writer contents! !

!Interface class methodsFor: 'printing'!
definition

	| writer extendedInterfaces equivalentInterfaces |
	self == Interface ifTrue: [^super definition].
	
	extendedInterfaces := (self perform: self configuration selectorForInterfaceInheritance) asSortedCollection: Class sortBlock.
	equivalentInterfaces := self equivalents.
	writer := String newLarge writeStream.
	writer 
		nextPutAll: super definition; crtab; 
		nextPutAll: 'interfaces: '; 
		nextPut: $'.
	extendedInterfaces do: [:each | writer nextPutAll: each name asString; space].
	writer nextPut: $'.
	equivalentInterfaces notEmpty
		ifTrue:
			[writer cr; cr; nextPutAll: '"Equivalent interfaces:'.
			equivalentInterfaces do: [:each | writer cr; tab; nextPutAll: each name asString].
			writer cr; nextPut: $"].
	^writer contents! !

!Interface class methodsFor: 'printing'!
definitionWithInterfaces

	^self definition! !

!Interface class methodsFor: 'printing'!
familyString

	^String newLarge writeStream
		nextPutAll: self name asString;
		nextPutAll: self ancestorsString;
		nextPutAll: self progenyString;
		contents! !

!Interface class methodsFor: 'printing'!
progenyString

	| writer progeny |
	writer := String newLarge writeStream.
	(progeny := self progeny) notEmpty
		ifTrue:
			[writer 
				crtab; 
				nextPutAll: 'progeny: {'.
			progeny do: [:each | writer nextPutAll: each name asString; space].
			writer 
				skip: -1; 
				nextPutAll: '}'].
	^writer contents! !


!Interface class methodsFor: 'testing'!
extends: anInterface

	^(self ~~ anInterface) and:
	[self repertoire includesAll: anInterface repertoire]! !

!Interface class methodsFor: 'testing'!
hasEquivalents

	^self equivalents notEmpty! !

!Interface class methodsFor: 'testing'!
hasParents
	"A simple interface is an orphan"

	^self parents notEmpty! !

!Interface class methodsFor: 'testing'!
isEmbodiedIn: anObject

	^anObject conformsTo: self! !

!Interface class methodsFor: 'testing'!
isEmpty

	^self repertoire isEmpty! !

!Interface class methodsFor: 'testing'!
isEquivalentTo: anInterface

	^self repertoire consistsOf: anInterface repertoire! !

!Interface class methodsFor: 'testing'!
isExtended
	"Note: not using 'self extendingInterfaces notEmpty' for speed"

	^self home allInterfaces contains: [:each | self isExtendedBy: each]! !

!Interface class methodsFor: 'testing'!
isExtendedBy: anInterface

	^anInterface extends: self! !

!Interface class methodsFor: 'testing'!
isExtending

	^self home allInterfaces contains: [:each | self extends: each]! !

!Interface class methodsFor: 'testing'!
isImmutable
	"Can I be modified?
	Interfaces which should be left as is (such as ANSI Protocols) should override with ^true.
	See: #makeImmutable:"
	
	^false! !

!Interface class methodsFor: 'testing'!
isImplementedBy: aClass
	"Does <aClass> directly implement all of my repertoire?"

	^aClass repertoireImplemented includesAll: self repertoire! !

!Interface class methodsFor: 'testing'!
isInterface
	"All subclass are considered as interfeces, excluding the superclass - Interface."

	^self ~~ Interface! !

!Interface class methodsFor: 'testing'!
isOrphan
	"A composite interface has parents"
	
	^self hasParents not! !

!Interface class methodsFor: 'testing'!
isUnderstoodBy: aBehavior
	"Can <aBehavior> understand all of <anInterface> repertoire?"

	^aBehavior repertoire includesAll: self repertoire! !

!Interface class methodsFor: 'testing'!
mustBeImplemented
	"Indicating the way I should be implemented by a class. 
	Consider the case of #hash and #=. How can we indicate that if a class implements one, 
	it should implement the other? (By overriding this method to return true.)
	The default is false, which means it is sufficient that I am understood by the implementing class."

	^false! !

!Interface class methodsFor: 'testing'!
truelyExtends: anInterface

	^(self extends: anInterface) and: 
	[(self isEquivalentTo: anInterface) not]! !


!Interface class methodsFor: 'subclass creation' stamp: 'MU 5/1/2000 18:49'!
subclass: interfaceName instanceVariableNames: stringOfInstVars classVariableNames: stringOfClassVars poolDictionaries: stringOfPoolNames category: cat 
	"#SmallInterfaces2.0.0#Squeak(added)"
	| interface |
	self == Interface ifFalse: [self error: 'an interface must be a direct subclass of Interface!!'].
	interface := super
				subclass: interfaceName
				instanceVariableNames: stringOfInstVars
				classVariableNames: stringOfClassVars
				poolDictionaries: stringOfPoolNames
				category: cat.
	^ interface! !

!Interface class methodsFor: 'subclass creation' stamp: 'MU 4/30/2000 20:53'!
subclass: interfaceName
	instanceVariableNames: stringOfInstVars
	classVariableNames: stringOfClassVars
	poolDictionaries: stringOfPoolNames
	category: cat
 	interfaces: stringOfInterfaces
	"Create or modify an interface so it would adhere to the declared Interfaces."
	"#SmallInterfaces2.0.0#Squeak(modified)"
	
	| interface |
	interface := self
		subclass: interfaceName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: cat.
	interface implementInterfacesNamed: stringOfInterfaces.		
	^interface! !

!Interface class methodsFor: 'subclass creation' stamp: 'MU 4/30/2000 20:54'!
variableByteSubclass: interfaceName
	instanceVariableNames: stringOfInstVars
	classVariableNames: stringOfClassVars
	poolDictionaries: stringOfPoolNames
	category: cat
 	interfaces: stringOfInterfaces
	"Create or modify an interface so it would adhere to the declared Interfaces.
	Note: since an interface must be a subclass"
	"#SmallInterfaces2.0.0#Squeak(modified)"
	^self
		subclass: interfaceName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: cat
 		interfaces: stringOfInterfaces! !

!Interface class methodsFor: 'subclass creation' stamp: 'MU 4/30/2000 20:54'!
variableSubclass: interfaceName
	instanceVariableNames: stringOfInstVars
	classVariableNames: stringOfClassVars
	poolDictionaries: stringOfPoolNames
	category: cat
 	interfaces: stringOfInterfaces
	"Create or modify an interface so it would adhere to the declared Interfaces."
	"#SmallInterfaces2.0.0#Squeak(modified)"
	^self
		subclass: interfaceName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: category
 		interfaces: stringOfInterfaces! !


!Interface class methodsFor: 'code generation'!
createMethod: selector from: aClass

	| source category |
	source := self methodSourceFor: selector from: aClass.
	category := aClass isNil
		ifTrue: [#repertoire]
		ifFalse: [aClass organization categoryOfElement: selector].
	self compile: source classified: category! !

!Interface class methodsFor: 'code generation'!
createMethods: selectors

	self createMethods: selectors from: nil! !

!Interface class methodsFor: 'code generation'!
createMethods: selectors from: aClass

	selectors do: [:each | self createMethod: each from: aClass]! !

!Interface class methodsFor: 'code generation' stamp: 'MU 4/15/2000 23:56'!
createStubedMethodFor: selector from: aClass
	"Create a stub method implementation for a selector in <aClass> on behalf of me."
	"#SmallInterfaces2.0.0#Squeak(renamed)"

	| interfaceOwningSelector |
	interfaceOwningSelector := self ancestors 
		detect: [:each | each repertoire includes: selector] 
		ifNone: [self].
	aClass 
		compile: (self methodSourceFor: selector from: self)
		classified: (interfaceOwningSelector defaultClassification)! !

!Interface class methodsFor: 'code generation'!
implementationStringForInterface

	^String newLarge writeStream
		nextPutAll: '^self';
		space;
		nextPutAll: self selectorForStubedImplementation asString;
		space;
		nextPutAll: self name asString;
		contents! !

!Interface class methodsFor: 'code generation' stamp: 'MU 4/22/2000 22:16'!
methodSourceFor: selector from: aClass
	"#SmallInterfaces2.0.0#Squeak(modified)"
	^aClass isNil
		ifTrue: 
			[self
				methodSourceWithHeader: selector asSelectorStringWithArgumentsString 
				comment: ''
				implementation: Class implementationStringForInterface]
		ifFalse: 
			[self
				methodSourceWithHeader: (aClass>>selector) headerString	 
				comment: (aClass>>selector) commentString
				implementation: aClass implementationStringForInterface]! !

!Interface class methodsFor: 'code generation'!
methodSourceWithHeader: header comment: comment implementation: implementationString

	| writer |
	writer := String newLarge writeStream.
	writer nextPutAll: header.
	comment isEmpty ifFalse: [writer crtab; nextPutAll: comment].
	writer cr; crtab; nextPutAll: implementationString.
	^writer contents! !


!Interface class methodsFor: 'accessing-methods'!
compilableMetaSelectors

	^#(#isSealed #mustBeImplemented)! !

!Interface class methodsFor: 'accessing-methods'!
methodsImplementedBy: aBehavior

	^self repertoire & aBehavior repertoireImplemented! !

!Interface class methodsFor: 'accessing-methods'!
methodsNotImplementedBy: aBehavior

	^self repertoire - aBehavior repertoireImplemented! !

!Interface class methodsFor: 'accessing-methods'!
methodsNotUnderstoodBy: aBehavior

	^self repertoire - aBehavior repertoire! !

!Interface class methodsFor: 'accessing-methods'!
methodsStubedIn: aBehavior

	^self repertoire & (aBehavior whichSelectorsReferTo: self selectorForStubedImplementation)! !

!Interface class methodsFor: 'accessing-methods'!
methodsStubedIn: aBehavior for: anInterface
	"Note: this method is useful in the following scenario:
	interface A repertoire = #(a b c).
	interface B repertoire = #(b c d).
	class C was declared to understand both A and B, and as a result it has #(a b c d) stubbed.
	If we wanted to delete A without affecting <C understands: B>, we could use this method to
	figure out which methods are specifically stubbed for <anInterface>."
	
	^(anInterface methodsStubedIn: aBehavior) &
	(aBehavior allSelectorsReferencing: anInterface)! !


!Interface class methodsFor: 'accessing'!
configuration

	^InterfaceConfiguration current! !

!Interface class methodsFor: 'accessing'!
home

	^InterfaceHome current! !

!Interface class methodsFor: 'accessing'!
selectorsForRepertoire

	^self selectors asIdentitySet! !

!Interface class methodsFor: 'accessing'!
theExtentedFamily
	"All relevant Classes and Interfaces"

	^(Set with: self)
		addAll: self extended;
		addAll: self extenders;
		addAll: self allConformers;
		yourself! !


!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:42'!
anOntologyOfInterfacesForSmalltalk4

"Each interface specifies a set of messages, which together constitutes its Repertoire.
Repertoires are not mutually exclusive; a message can be part of many repertoires (N:N relationship).

Interfaces are organized in heterarchies, so that one interface can be declared as being a composite of other interfaces. A composite interface is an interface that extends other interfaces; it inherits messages from the interfaces it extends, and may add additional messages of its own. A composite interface also referred to as an extending interface. The above interface classifications are not mutually exclusive; an interface can be Extended and Extending at the same time.

At the top of the heterarchy are root interfaces, which are parentless interfaces; they extend no other interfaces. At the bottom of the heterarchy are the leaf interfaces, which are childless interfaces; no other interfaces extend them. These interface classifications also are not mutually exclusive. Consider the case where an environment contains a single interface; that singleton interface is both a root and a leaf at the same time.

An interface with no repertoire is referred to as an empty interface. In SmallInterfaces such interface is not very useful and is considered to be meaningless (though it causes no harm).

A behavior (class or metaclass) is an Understander (or a Conformer) of an interface if it can respond to all messages of the interface's repertoire. A behavior can conform to many interfaces, and an interface can be understood by many behaviors.

An object conformsTo: aType if < aType> is one of: a) its class (or one of its superclasses), or b) one of the interfaces its class understands.

Within a universe (such as the Smalltalk image), interfaces form a-cyclical directed graphs that are not necessarily connected. Since this universe is dynamic in Smalltalk, the inter-relationships among interfaces is dynamic as well, and may change whenever an interface is added to or removed from the universe, or whenever a message is added to or removed from an interface. The relationships between classes and interfaces is also dynamic, and may change for similar reasons. So the classes <--> interfaces relationships as well as interfaces <--> interfaces relationships are always inferred from the actual current composition of classes and interfaces in the universe."

	^self commentOnly! !

!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:41'!
interfacesAndSmalltalk3

"Interfaces are not about static typing. In principle, they are merely about typing. More specifically, they can be used to solely specify a behavior, detached from data. In other words, an interface specifies which messages an object will execute, unlike a class, which specifies how those messages will be executed.

Some OO languages have Interface and Class as two distinct concepts. For example, Java reifies both to some extent and includes syntax for defining and implementing interfaces, while Microsoft COM and OMG CORBA both have their own software object interface definition languages (IDLs).

In Smalltalk however, the interfaces of an object are implicit and folded into its class implementation. This is not to say that interfaces in Smalltalk can not be harvested and become tangible, first-class objects. On the contrary, this is exactly what SmallInterfaces sets to achieve - the reification of interfaces in Smalltalk.

For a dynamically typed language like Smalltalk it is sufficient for a message declaration to specify (implicitly) the number of arguments. That is what SmallInterfaces does. However, there exists another meaningful scheme for Smalltalk, where the type (interface or class) of each parameter (input variable) would be specified as well. With such scheme it is also desirable to specify the return value as well. Such scheme would be very useful for component-based development. Since SmallInterfaces aims to help the design and development process itself, it is taking the first and simpler approach."

	^self commentOnly! !

!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:42'!
someImportantImplementationNotes6

"The classes <--> interfaces relationships as well as interfaces <--> interfaces relationships are of a dynamic nature. This is to say that the web of relationships is always inferred from the actual composition of classes and interfaces every time a query is attempted. This is neat, but also very inefficient, especially when a query for all conforming classes is executed for an interface. Therefore, a caching scheme is implemented under the wings of RepertoireHome.

The notion that an interface is a composite is implicit; when a new interface is composed from other interfaces, the repertoires of these interfaces are copied over.

In SmallInterfaces, when a class is added/changed/removed to/from the environment, all relevant interfaces are immediately affected by the event (causal connection). The same holds true when an interface is added/changed/removed to/from the environment; all relevant classes and interfaces are immediately affected by the event. This causal connection is a byproduct of dynamically inferencing the relationships among classes and interfaces.

We refer to the activity which facilitates such causal connection as: synchronization.
The granularity of synchronization is controlled by (within InterfaceConfiguration):
	synchronize
	synchronizeWithConformers
	synchronizeWithExtenders
	removeOnlyStubbs
Where removeOnlyStubbs controls whether or not existing stubbed methods would be removed when a class declares it understands a different set of interfaces."

	^self commentOnly! !

!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:31'!
tutorial

"I guess the biggest question everybody has is: 
	'How would I use one of 'them interfaces in Smalltalk?'

Within the context of Smalltalk, interfaces can be useful in three major areas: 
	Design, Documentation, and Exploration.
	
For each aspect, see the corresponding tutorial.

Note that all interfaces are classes, and as such can be browsed and modified using a generic browser. So once you have created an interface, you can use a generic browser to add some methods, and/or remove other methods."

	^self commentOnly! !

!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:40'!
tutorialDesigning

"The concept of interfaces can help us do a better job with using Responsibility Driven Design by facilitating a higher level of abstraction than that of a class. The gist of Responsibility Driven Design is that when one designs an object (in the context of a society of objects), one wants to concentrate on what services an object needs to offer to its clients.  What we are really after is figuring out what are the pure 'working relationships' an object has within its society. We do not care about *how* the object is going to work it out, nor what state it needs to maintain in order to do so.  

Each service an object is contracted to supply, or said differently - each role it plays, is embodied within an interface.

So, designing using interfaces can be a mind liberating experience. With interfaces one can better concentrate on the question: 'What are the responsibilities of a given object?' in terms of: 'What should be its repertoire?' 

Designing is then broken down into searching for the answer to: 'What messages this and that are supposed to communicate and understand in order to get a certain functionality/aspect accomplished?'

If we follow this design mode, we have: 
	Responsibilities beget Roles, 
		Roles beget Interfaces,  
			Interfaces beget Objects.


With that in mind, let us put this theory into practice using an example.  

Say you have a domain where you need the ability to price some objects.  Some objects have no price, some have a fixed price, and some have a dynamic price.  The dynamic price may depend on various factors such as time and locale.  Also, that price needs to be quoted in a parameterized way, such as: in a currency, in goods (as in: 'I'll give you three loaves of bread for two of these'), or in services (as in: 'will be president for food').  

An important aspect of these objects is that they can be priced; they are priceable (clink, clink...). Some of these objects have the capability to compute their price; others would use a helper pricer to do the job, and others yet, might do one or the other - depending on the context. What they all have in common is that they all need to understand (and respond to) the following group of messages:
	price
	priceAsOf: aTemporal   (such as Date, Timestamp, etc.)
	priceAsOf: aTemporal quantifyUsing: aTradable   (such as Money, Product, WorkService)
	defaultTemporal
	defaultTradable
	pricer

This group of messages, this repertoire, has semantic meaning on its own, and as such it deserves to be a first class object. We call such reification - an interface. And the above repertoire defines the Priceable interface. From that point on, we no longer need to say: 'Money or Product or WorkService or whatever' - we can refer to those entities as Priceables.

As an exercise, you might want to try and define the Pricer interface.  Note that a pricer for: 5 cubic miles of irrigation water in the Sahara for the summer of 1999 might have very little in common with a pricer for: 10,000 IBM stocks on the New York Exchange for 1/1/2000.
If you still got energy, go a head and define the Tradable and Temporal interfaces. Note that the price answered is a Quantifiable. Define that too.

Can you start to see the advantages of 'thinking' in interfaces?

When you are done with this little design you end up with capturing the core of your business. Furthermore, there is enough content in this design to facilitate some simulations of business scenarios, and by doing so, validating your understanding of the problem at hand.

Maybe you can also imagine that the actual classes that will be used to implement this design would cross hierarchies. For example, some objects' prices will be date-specific, some location-specific, some will be time & location specific, some will have to consider all factors or none - depending on the context when they are asked (like by whom).

Now you can use SmallInterfaces to make the first step in implementing this design - declaring those interfaces.
Using SmallInterfaces you can create interfaces programmatically by one of three ways:
	1. define the interface and its methods directly
	2. define the interface and its methods from a group of interfaces
	3. define the interface and its methods from a class

Using the Priceable interface as an example for the first way, we would execute:
	<Interface 
		newNamed: #Priceable
		withSelectors: #(price priceAsOf: priceAsOf:quantifyUsing: defaultTemporal defaultTradable pricer)>

Using an example from the attached ToyInterfaces package to illustrate the second way, we would execute:
	<Interface 
		newNamed: #ReadableWritable
		extending: 'Readable Writable'
		additionalSelectors: #()>

Now assuming you want to use Magnitude as a template for the Quantifiable interface, you could use the third way and execute:
	<Magnitude asInterfaceNamed: #Quantifiable> 
or:
	<Interface 
		newNamed: #Quantifiable 
		from: Magnitude>

Remember that all interfaces are classes, and can be browsed and modified using the generic browser. So, for example, after creating Quantifiable from Magnitude, you might want to remove some methods, and/or add some others.

Now you are ready for the next step - implementing some classes based on those interfaces.
Assuming you had Priceable & Quantifiable in mind, and you wanted to create Price & Quantity, you would execute something like:
	<Priceable asClassNamed: #Price super: MyDomainObject>
and:
	<Quantifiable asClassNamed: #Quantity>

The Price & Quantity classes would now hold stubbed method implementations for Priceable & Quantifiable repertoires (including comments, if you had any).


All the above operations and more are easily performed using the Interface Browser GUI."

	^self commentOnly! !

!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:40'!
tutorialDocumenting

"Using interfaces we can write a better self-documenting code, since we have a higher level of abstraction at our disposal. We can use interfaces to make the implicit explicit, thus helping the original design surface and stay visible as the code evolves. Their presence should help providing direction for the refactoring process. 

Consider the following implementation snippet for the example in the design section:

WorkService>>priceAsOf: aTemporal quantifyUsing: aTradable		
	^self pricer 
		price: self
		in: self location
		asOf: aTemporal
		quotingIn: aTradable

Notice that pricer, aTemporal, aTradable, and possibly location, represent interfaces. The actual possible objects that can occupy each slot are not necessarily sharing the same hierarchy!!

Now consider how the very same code snippet would look without 'thinking in interfaces':

WorkService>>priceAsOf: aDateOrTimeOrTimestamp quantifyUsing: aMoneyOrProductOrWorkService		
	^self pricer 
		price: self
		in: self location
		asOf: aDateOrTimeOrTimestamp
		quotingIn: MoneyOrProductOrWorkService

Which version is better at self-documenting?


Interfaces can also be used in design validation. After specifying a #requiredInterfaces method on the class side of aClass, we now can use <aClass understandsRequiredInterfaces> in our code, like in a testcase: 
	<self should: [someObject class understandsRequiredInterfaces]>


A more explicit way of indicating the original design, is a variation of the #isSomething usage.
For example, all classes that understand Priceable will implement isPriceable to return true, while their superclasses will implement it to return false. Now we can have phrases such as:
	<anObject isPriceable
		ifTrue: [self charge: aCustomer for: anObject]
		ifFalse: [self giveAway: anObject to: aCustomer]>


Also, we can now sprinkle in our code statements like:
	<groupOfObjectsWhichSupposadlyAreKeyable do:
		[:each | 
		(each conformsTo: Keyable)
  			ifTrue: [each at: someplaceSafe put: somethingGood]
				ifFalse: [self giveMe: somethingGood]]>


Note that SmallInterfaces can automate some of these schemes, but does not."

	^self commentOnly! !

!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:40'!
tutorialExploring

"Interfaces can give us another mental navigation tool while browsing the environment. My contention is that if interfaces had been more explicit in Smalltalk then one could have used them to learn a Smalltalk environment faster.
Following are a few examples taken from the VisualWorks environment to illustrate how interfaces could be used in system exploration.

VisualWorks has Model which has a specialized implementation of the Observable interface found in Object. Now let's say you want to find out if the environment has a specialized Observable collection as well. You look and you don't seem to see one under the Model hierarchy. So you might want to create an Observable interface with #myDependents: and #myDependents as its repertoire and search for conformers (understanders) of Observable. 
To create an Observable interface you would execute:
	<Interface
	newNamed: # Observable
	withSelectors: #(#myDependents: #myDependents)>

To look for its conformers you would execute:
	<Observable conformers> 
Depending on your environment the result would be something like:
	{Object TableAdaptor TwoDList Model WidgetWrapper List ScrollWrapper WeakArray}.
Notice List and WeakArray - they might be what you were looking for. Notice the others too - if you are one of the curious kinds, it might be interesting for you to try and figure why the others have a specialized implementation of the Observable interface.

You can now refine your Observable interface by adding other messages you think are integral to it, and use it as a template to create other observable objects. For example, how about implementing an observable process: one that has dependents who can be notified when it is completed its task (like Promise or Future for asynchronous messaging). Such a class would probably implement a Runnable interface as well...


Another example is related to extending the environment:

If you happened to add your own extensions to base classes you probably extended Dictionary as well. When doing so, did you think it might be useful to add the same extension to KeyedCollection as well?  If not, why? After all, KeyedCollection and Dictionary are both keyable...

A similar connection exists between SequenceableCollection and Stream, especially String and Stream."

	^self commentOnly! !

!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:42'!
whatCanYouDoWithSmallInterfaces5

"1. Declare an interface and specify its behavior. This can be done in three distinct ways:
	- Directly, by specifying its name and selectors. (Look under the ''interface creation'' protocol under Interface.)
	- By composing from other interfaces.
	- By converting a class as a template (Look at the various asInterface* methods under ClassDescription.)

2. Declare a behavior as conforming to (or understanding) one or more interfaces. This can be done in two distinct ways:
	- By adding a 'interfaces: stringOfInterfaceNames' to the behavior 's definition. 
	  (Look under the 'compiling' protocols under Class and Metaclass.)
	- By sending implement: to the behavior 's definition. 
	  (Look at the various implement* methods under ClassDescription.)
	- By converting an interface to a class (Look at the various asClass* methods under Interface.)
	All methods will create stubbed methods as necessary on behalf of the to-be-understood interface(s)

3. Given a behavior (class or metaclass), you can ask: 
	- What interfaces it understands (conforms to)? (its interfaces)

4. Given an interface, you can ask: 
	- Which behaviors understand it? (its conformers)

5. Given an interface, you can ask: 
	- Which interfaces extend it? (its extenders)
	- Which interfaces it extends? (its extended)
	- Which interfaces immediately extend it? (its children) 
	- Which interfaces it immediately extends? (its parents)
	- Which interfaces share its parents? (its siblings)
	- Which interfaces are equivalent to it? (its twins)
	- Which interfaces it extends? (its lineage)
	- Which interfaces and behaviors are related to it? (its relatives)

6. Given an interface or a behavior, you can ask: 
	- What is the set of messages it is comprised of? (its repertoire)

7. Given an object, you can ask: 
	- Which are its types? (behaviors and interfaces)
	- Does it conform to a type?"

	^self commentOnly! !

!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:41'!
whatIsAnInterface2

" The concept of an interface is central to object-oriented methodologies, most commonly referred to as Type. An interface is an abstract type, unlike class, which is a concrete type. An interface specifies a set of messages, where a class implements a set of messages. Therefore, the concept of an interface is orthogonal to that of a class.

Said differently, an interface specifies which messages an object will execute but it has no method implementations for those messages, where a class specifies how those messages will be executed by specifying the method implementations for those messages.

Each interface specifies a set of messages, which together constitutes what we term as its repertoire. In turn, each message declaration specifies the message's name (selector), and its arguments. For various languages, specifying the arguments for a message can mean different things, and there are many opinions and debates with regard to this issue. Enough said."

	^self commentOnly! !

!Interface class methodsFor: 'documentation' stamp: 'MU 5/15/2000 00:41'!
whyShouldYouBotherWithInterfaces1

"In the realm of domain analysis, the use of roles (also known as facets) has emerged as an important technique for classifying the features of objects. Roles reflect the various aspects of the object they describe, and the different roles that an object may play in relationship to other objects. More so, the assignment of responsibilities to an object depends on the role(s) it plays in a system of objects. 

Roles serve as an increasingly important metaphor for communicating object-oriented software designs and recognition of their importance has grown in recent years. For example, the codification of object-oriented software design knowledge in design patterns is founded in part on using the metaphor of roles. Software design patterns describe reusable collaborations between design elements. Each design element plays an identifiable role with well-defined responsibilities.

Now, a design process and a programming language work well together when there is support and clean translation from the design process conceptual units, to the programming language abstraction and composition mechanisms. An interface is such programming language mechanism which maps the design process role concept. 

The mental process that leads us from design to implementation can be summarized as:
Responsibilities beget Roles, 
Roles beget Interfaces,  
Interfaces beget Objects.

So interfaces are a very basic mechanism for thinking in terms of objects; they both define and organize the services objects provide. This is why designing good interfaces for objects becomes such an important endeavor when building applications using objects."

	^self commentOnly! !


!Interface class methodsFor: 'defaults'!
defaultCategory

	^self configuration defaultCategory! !

!Interface class methodsFor: 'defaults'!
defaultClassName
	"This method is called for the purpose of generating a 'real' class from an interface."

	^(self name, 'Implementation') asSymbol! !

!Interface class methodsFor: 'defaults'!
defaultClassification

	^self name! !

!Interface class methodsFor: 'defaults'!
defaultInterfaceName
	"This method is called for the purpose of generating an interface from a 'real' class.
	Since I am an interface, reaching this method is probably an error."
	
	^self error: 'I am an interface; you should not create an inteface for me!!'! !

!Interface class methodsFor: 'defaults'!
selectorForImplementation
	"The selector to be embeded within any method of an Interface"

	^#implementorsResponsibility! !

!Interface class methodsFor: 'defaults'!
selectorForStubbedImplementation
	"This selector is embeded as a stub within a method as initial implementation for an Interface"

	^#shouldImplementFor:! !

!Interface class methodsFor: 'defaults'!
selectorForStubedImplementation
	"This selector is embeded as a stub within a method as initial implementation for an Interface"

	^#shouldImplementFor:! !


!Interface class methodsFor: 'accessing-behaviors'!
allConformers

	^self isEmpty
		ifTrue: [IdentitySet new]
		ifFalse: [self environment allNonInterfaceBehaviors select: [:each | each understands: self]]! !

!Interface class methodsFor: 'accessing-behaviors'!
conformers

	^self allConformers reject: 
		[:each | each superclass notNil and: [each superclass understands: self]]! !


!Interface class methodsFor: 'utilities' stamp: 'MU 4/28/2000 01:32'!
consolidate: interfaces
	"out of <interfaces>, return the minimal descriptive set.
	mostly, these are the interfaces which are not extended by any of the others."

	^interfaces select: [:each |(interfaces contains: [:interface | interface truelyExtends: each]) not]! !

!Interface class methodsFor: 'utilities'!
interfacesFromString: stringOfInterfaces

	^((stringOfInterfaces subStrings collect: 
		[:each | each asClass]) select: 
			[:each | each notNil]) select: 
				[:each | each isInterface]! !

!Interface class methodsFor: 'utilities'!
makeImmutable: anInterface
	"right now, this will no work.
	to make this work, we would need to indicate #isImmutable as a special selector that can be added to the meta Interface. 
	alternatively, we can register immutable interfaces as such in a special dictionary (such as in InterfaceHome)." 
	
	anInterface isInterface
		ifTrue: 
			[anInterface class
				compile: 'isImmutable\\	^true' withCRs
				classified: #testing
				notifying: nil]! !

!Interface class methodsFor: 'utilities'!
template
	"Answer an expression that can be edited and evaluated in order to define a new interface."

^'Interface subclass: #NameOfInterface
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''''
	interfaces: '''' '! !

!Interface class methodsFor: 'utilities'!
valueWithoutSynchronization: aBlock 

	| currentState |
	currentState := Interface configuration synchronize.
	Interface configuration synchronize: false.
	[aBlock value] ensure: [Interface configuration synchronize: currentState]! !


!Interface class methodsFor: 'errors'!
basicNew

	self error: 'I am an Interface. I can not be instantiated!!'! !

!Interface class methodsFor: 'errors'!
implementorsResponsibility
	"Besides being an indicator, this method is envoked via: 'anObject shouldImplementFor: anInterface'.
	As a consequence, this message sets up a framework for the behavior of the interface's implementors."

	self error: 'My implementing class should have overridden one of my messages.'


"to facilitate laissez faire programming, one implementation could be:
	1. open an editor on the calling method (allow the user to abort).
	2. accept the source and copile it in the original sending class.
	3. continue execution (restart?)"! !

!Interface class methodsFor: 'errors'!
new

	self error: 'I am an Interface. I can not be instantiated!!'! !


!Interface class methodsFor: 'synchronizing'!
conformersToSynchronize

	^Interface configuration synchronizeWithConformers 
		ifTrue: [self allConformers]
		ifFalse: [#()]! !

!Interface class methodsFor: 'synchronizing'!
extendersToSyncronize

	^Interface configuration synchronizeWithExtenders 
		ifTrue: [self extenders]
		ifFalse: [#()]! !

!Interface class methodsFor: 'synchronizing'!
relativesToSynchronize

	^IdentitySet new
		addAll: self conformersToSynchronize;
		addAll: self extendersToSyncronize;
		yourself.! !

!Interface class methodsFor: 'synchronizing'!
synchronizeChangeOf: aspect usingSelector: selector

	self relativesToSynchronize do:
		[:each | 
		(aspect == #removeClass:) ifTrue: [each unimplement: self].
		(aspect == #removeSelector:class:) 
			ifTrue: 
				[each 
					synchronizeWith: self
					removalOf: selector]].! !

!Interface class methodsFor: 'synchronizing'!
synchronizeWith: anInterface removalOf: selector 

	self removeMethods: (Array with: selector)! !


!Interface class methodsFor: 'interface creation' stamp: 'MU 4/29/2000 18:53'!
newNamed: aSymbol 

	"self isPrivateMethod."
	| interface |
	self ~~ Interface 
		ifTrue: [self error: 'an interface must be a direct subclass of Interface!!'].
	(self home includesInterfaceNamed: aSymbol) 
		ifTrue: [^self error: aSymbol asString, ' interface already exists!!'].

	interface := self 
		subclass: aSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: (Interface configuration defaultCategory)
		interfaces: ''.		
	interface resetRepertoire.
	^interface! !

!Interface class methodsFor: 'interface creation'!
newNamed: aSymbol extending: stringOfInterfaces

	^self
		newNamed: aSymbol 
		extending: stringOfInterfaces
		additionalSelectors: #()! !

!Interface class methodsFor: 'interface creation'!
newNamed: aSymbol extending: stringOfInterfaces additionalSelectors: selectors

	| interface |
	interface := self newNamed: aSymbol.
	Interface valueWithoutSynchronization:
		[interface extendAll: (Interface interfacesFromString: stringOfInterfaces).
		interface createMethods: selectors asSet].
	interface resetRepertoire.
	^interface! !

!Interface class methodsFor: 'interface creation'!
newNamed: aSymbol from: aClass

	| interface | 
	Interface valueWithoutSynchronization:
		[interface := self 
			newNamed: aSymbol 
			withSelectors: aClass selectors 
			from: aClass].
	^interface! !

!Interface class methodsFor: 'interface creation'!
newNamed: aSymbol withSelectors: selectors

	| interface | 
	Interface valueWithoutSynchronization:
		[interface := (self newNamed: aSymbol) createMethods: selectors].
	^interface! !

!Interface class methodsFor: 'interface creation'!
newNamed: aSymbol withSelectors: selectors from: aClass

	| interface | 
	Interface valueWithoutSynchronization:
		[interface := (self newNamed: aSymbol)
			createMethods: selectors 
			from: aClass].
	^interface! !

!Interface class methodsFor: 'interface creation'!
superclass: aClass 

	aClass ~~ Interface 
		ifTrue: [self error: 'an interface must be a direct subclass of Interface!!'].

	super superclass: aClass.! !

!Interface class methodsFor: 'interface creation'!
template: category 
	"Answer an expression that can be edited and evaluated in order to define a new Interface."

	^'Interface subclass: #NameOfInterface
	instanceVariableNames: ''''
	classVariableNames: ''''
	poolDictionaries: ''''
	category: ''', category asString storeString, '
	interfaces: '''' '! !


!Interface class methodsFor: 'compiling' stamp: 'MU 5/6/2000 16:23'!
compile: code classified: heading notifying: requestor
	"An interface method can not have an implementation. 
	This method attempts to enforce just that...	
	Note: if the method compiled is new, then the interface change is synchronized."

	| toSynchronize selector result |
	self isSealed ifTrue: [^nil].
	
	selector := SIParserUtil parseMethodSelectorOf: code. "#SmallInterfaces2.0.0#Squeak(modified)(need RB)"
	toSynchronize := (self selectors includes: selector) not
		ifTrue: [self relativesToSynchronize]
		ifFalse: [#()].
		
	result := super 
		compile: (self interfaceMethodSourceBasedOn: code)
		classified: heading
		notifying: requestor.

	self synchronizeChangeOf: selector with: toSynchronize.
	^result! !

!Interface class methodsFor: 'compiling' stamp: 'MU 5/6/2000 16:23'!
interfaceMethodSourceBasedOn: source 
	"#SmallInterfaces2.0.0#Squeak(modified)"
	
	^self
		methodSourceWithHeader: (SIParserUtil parseMethodHeaderOf: source)
		comment: (SIParserUtil parseMethodCommentOfWithQuote: source)
		implementation: Class implementationStringForInterface! !

!Interface class methodsFor: 'compiling'!
synchronizeChangeOf: selector with: behaviors

	(self selectToSynchronizeFrom: behaviors) do: 
		[:each | 
		(each repertoire includes: selector) not
			ifTrue: [self createStubedMethodFor: selector from: each]]! !


!Interface class methodsFor: 'message creation-removal'!
extend: anInterface

	anInterface repertoire do: 
		[:each | | classification |
		classification := anInterface organization categoryOfElement: each.
		(classification == #repertoire or:
		[(self environment hasClassNamed: classification) not])
			ifTrue: [classification := anInterface name].
		self 
			compile: (anInterface sourceCodeAt: each)
			classified: classification]! !

!Interface class methodsFor: 'message creation-removal'!
extendAll: interfaces 

	(self extended - interfaces) do: [:each | self unimplement: each].
	(Interface consolidate: interfaces) do: [:each | self extend: each]! !

!Interface class methodsFor: 'message creation-removal'!
implementAll: interfaces 

	self extendAll: interfaces! !

!Interface class methodsFor: 'message creation-removal'!
removeSelector: selector ifAbsent: exceptionHandler

	self isSealed ifTrue: [^self].
	^super removeSelector: selector ifAbsent: exceptionHandler! !

!Interface class methodsFor: 'message creation-removal'!
unimplement: anInterface
	"Remove implementations for all of <anInterface> repertoire I already implement."

	self removeMethods: anInterface repertoire.! !


!Interface class methodsFor: 'mutability'!
beMutable

	(self isInterface and: [self isSealed]) 
		ifTrue: 
			[self class 
				removeSelector: #isSealed;
				removeAllEmptyCategories]! !

!Interface class methodsFor: 'mutability' stamp: 'MU 4/29/2000 19:07'!
beSealed
	"Note: the current implementation uses method compilation. 
	alternatively, we can use a special registry (such as in InterfaceHome)."
	"#SmallInterfaces2.0.0#Squeak(modified)"
	(self isInterface and: [self isMutable])
		ifTrue: 
			[self class
				compile: 'isSealed\\	^true' withCRs
				classified: #testing
				notifying: nil]! !

!Interface class methodsFor: 'mutability'!
isMutable
	"Can I be modified?"
	
	^self isSealed not! !

!Interface class methodsFor: 'mutability'!
isSealed
	"Am I 'final'?"
	
	^false! !


Object subclass: #InterfaceConfiguration
	instanceVariableNames: 'defaultCategory discardInterfacesOnShutDown notifyOnDerivationDiscovery removeOnlyStubs selectorForInterfaceInheritance selectorForClassInterfaces synchronize synchronizeSelectively synchronizeWithConformers synchronizeWithExtenders '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallInterfaces'!
!InterfaceConfiguration commentStamp: 'MU 5/15/2000 00:29' prior: 0!
InterfaceConfiguration is responsible for the system-wide behavior of interfaces.

The current configuration can be switched by either configuring the current singletone, or by replacing the current singletone with another configuration.
Look at #reset and #setDefaults for suggested configurations.

Instance Variables:

discardInterfacesOnShutDown		<Boolean>	... should all cached repertoires be discarded each time the image is saved?
notifyOnDerivationDiscovery			<Boolean>	... should the user be notifyied upon a discovery of derivation from an interface? 
									this happen when a behavior had just added a new message and it is now inferred to derive from an interface.
removeOnlyStubbs					<Boolean>	... when unimplementing an interface for a class, should only stubbed methods be removed?
selectorForInterfaceInheritance		<Symbol>	determines which interfaces should be shown in the definition string (e.g.: all extended).
selectorForClassInterfaces			<Symbol>	determines which interfaces should be shown in the definition string (e.g. all understood).
synchronize							<Boolean>	... should an interface change be synchronized with affected relatives (conformers & extenders)?
synchronizeSelectively				<Boolean>	... selectively choose which relatives to upgrade/downgrade when synchronizing a change?
synchronizeWithConformers			<Boolean>	... synchronize an interface change with conforming behaviors?
synchronizeWithExtenders			<Boolean>	... synchronize an interface change with extending interfaces?
 
Class Instance Variables:

singleton	<InterfaceConfiguration>	holds the current InterfaceConfiguration.!


!InterfaceConfiguration methodsFor: 'configuring'!
initialize
 	
 	self setDefaults! !


!InterfaceConfiguration methodsFor: 'accessing' stamp: 'MU 5/15/2000 01:20'!
defaultCategory
	"#SmallInterfaces2.0.0#Squeak(modified)"
	^defaultCategory! !

!InterfaceConfiguration methodsFor: 'accessing' stamp: 'MU 5/15/2000 01:20'!
defaultCategory: aString
	"#SmallInterfaces2.0.0#Squeak(added)"
	defaultCategory := aString! !

!InterfaceConfiguration methodsFor: 'accessing'!
discardInterfacesOnShutDown 

	^discardInterfacesOnShutDown! !

!InterfaceConfiguration methodsFor: 'accessing'!
discardInterfacesOnShutDown: aBoolean

	discardInterfacesOnShutDown := aBoolean! !

!InterfaceConfiguration methodsFor: 'accessing'!
notifyOnDerivationDiscovery

	^notifyOnDerivationDiscovery and: [self environment isHeadless not]! !

!InterfaceConfiguration methodsFor: 'accessing'!
notifyOnDerivationDiscovery: aBoolean

	notifyOnDerivationDiscovery := aBoolean! !

!InterfaceConfiguration methodsFor: 'accessing'!
removeOnlyStubs

	^removeOnlyStubs! !

!InterfaceConfiguration methodsFor: 'accessing'!
removeOnlyStubs: aBoolean 

	removeOnlyStubs := aBoolean! !

!InterfaceConfiguration methodsFor: 'accessing'!
selectorForClassInterfaces

	^selectorForClassInterfaces! !

!InterfaceConfiguration methodsFor: 'accessing'!
selectorForClassInterfaces: selectorSymbol

	selectorForClassInterfaces := selectorSymbol! !

!InterfaceConfiguration methodsFor: 'accessing'!
selectorForInterfaceInheritance

	^selectorForInterfaceInheritance! !

!InterfaceConfiguration methodsFor: 'accessing'!
selectorForInterfaceInheritance: selectorSymbol

	selectorForInterfaceInheritance := selectorSymbol! !

!InterfaceConfiguration methodsFor: 'accessing'!
synchronize 
	"#SmallInterfaces2.0.0#Squeak(modified)"
	synchronize ifNil:[synchronize := true].
	^synchronize! !

!InterfaceConfiguration methodsFor: 'accessing'!
synchronize: aBoolean

	synchronize := aBoolean! !

!InterfaceConfiguration methodsFor: 'accessing'!
synchronizeSelectively

	^synchronizeSelectively and: [self environment isHeadless not]! !

!InterfaceConfiguration methodsFor: 'accessing'!
synchronizeSelectively: aBoolean

	synchronizeSelectively := aBoolean! !

!InterfaceConfiguration methodsFor: 'accessing'!
synchronizeWithConformers

	^synchronizeWithConformers! !

!InterfaceConfiguration methodsFor: 'accessing'!
synchronizeWithConformers: aBoolean 

	synchronizeWithConformers := aBoolean! !

!InterfaceConfiguration methodsFor: 'accessing'!
synchronizeWithExtenders

	^synchronizeWithExtenders! !

!InterfaceConfiguration methodsFor: 'accessing'!
synchronizeWithExtenders: aBoolean 

	synchronizeWithExtenders := aBoolean! !


!InterfaceConfiguration methodsFor: 'initialize-release'!
reset

	self
		discardInterfacesOnShutDown: true;
		notifyOnDerivationDiscovery: false;	
		removeOnlyStubs: true;
		selectorForClassInterfaces: #interfaces;
		selectorForInterfaceInheritance: #parents;
		synchronize: false;
		synchronizeSelectively: false;
		synchronizeWithConformers: false;
		synchronizeWithExtenders: false! !

!InterfaceConfiguration methodsFor: 'initialize-release' stamp: 'MU 5/15/2000 01:20'!
setDefaults
	"#SmallInterfaces2.0.0#Squeak(modified)"
	self
		defaultCategory: 'Interfaces';
		discardInterfacesOnShutDown: true;
		notifyOnDerivationDiscovery: true;	
		removeOnlyStubs: true;
		selectorForClassInterfaces: #interfaces;
		selectorForInterfaceInheritance: #parents;
		synchronize: true;
		synchronizeSelectively: true;
		synchronizeWithConformers: true;
		synchronizeWithExtenders: true! !


!InterfaceConfiguration methodsFor: 'helping' stamp: 'MU 4/20/2000 00:02'!
helpString
	
	"#SmallInterfaces2.0.0#Squeak(added)"
	
	^self class helpString! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

InterfaceConfiguration class
	instanceVariableNames: 'singleton '!

!InterfaceConfiguration class methodsFor: 'constants' stamp: 'MU 4/28/2000 02:30'!
configurationArray
	"#SmallInterfaces2.0.0#Squeak(added)"
	^#(defaultCategory discardInterfacesOnShutDown notifyOnDerivationDiscovery removeOnlyStubs selectorForInterfaceInheritance selectorForClassInterfaces synchronize synchronizeSelectively synchronizeWithConformers synchronizeWithExtenders)! !


!InterfaceConfiguration class methodsFor: 'class initialization' stamp: 'MU 4/22/2000 22:36'!
addSettings
	"#SmallInterfaces2.0.0#Squeak(not used)"
	"| launcher |
	UISettings addPreferenceSection: #addSmallInterfacesSectionTo:development:runtime:.
	launcher := Smalltalk at: #VisualLauncher ifAbsent: nil.
	(launcher respondsTo: #checkInstalledComponents)
		ifTrue: [launcher checkInstalledComponents]"! !

!InterfaceConfiguration class methodsFor: 'class initialization' stamp: 'MU 4/1/2000 01:26'!
initialize
	"#SmallInterfaces2.0.0#Squeak(modified)"
	"InterfaceConfiguration initialize."
	
	Interface configuration setDefaults.! !

!InterfaceConfiguration class methodsFor: 'class initialization' stamp: 'MU 4/1/2000 01:27'!
obsolete
	"#SmallInterfaces2.0.0#Squeak(modified)"

	super obsolete.
	self nuke.
	! !

!InterfaceConfiguration class methodsFor: 'class initialization' stamp: 'MU 4/22/2000 22:37'!
removeSettings
	"#SmallInterfaces2.0.0#Squeak(not used)"
	"| launcher |
	UISettings removePreferenceSection: #addSmallInterfacesSectionTo:development:runtime:.
	launcher := Smalltalk at: #VisualLauncher ifAbsent: nil.
	(launcher respondsTo: #checkInstalledComponents)
		ifTrue: [launcher checkInstalledComponents]"! !


!InterfaceConfiguration class methodsFor: 'instance creation'!
current

	^singleton ifNil: [singleton := super new initialize]! !

!InterfaceConfiguration class methodsFor: 'instance creation'!
new

	^self current! !

!InterfaceConfiguration class methodsFor: 'instance creation'!
nuke

	singleton := nil! !


!InterfaceConfiguration class methodsFor: 'helping' stamp: 'MU 4/15/2000 01:34'!
helpString
	"#SmallInterfaces2.0.0#Squeak(modified)(moved from UISettings)"
	
	^'SmallInterfaces Configuration Setup

Set the preferences for system-wide behavior of interfaces.

Discard Interfaces On Image Save
	- should all cached repertoires be discarded each time the image is saved?  

Remove Only Stubs
	- when unimplementing an interface for a class, which parts of the implementation should be removed?  

Synchronization On
	- should an interface change be synchronized with affected behaviors (conformers & extenders)?

Synchronize with Conformers
	- synchronize an interface change with conforming classes?

Synchronize with Extenders
	- synchronize an interface change with extending interfaces?

Selector for Interface Inheritance
	- which interfaces should be shown in the definition string?

Selector for Class Interfaces
	- which interfaces should be shown in the definition string?
'! !


!InterfaceConfiguration class methodsFor: 'setting' stamp: 'MU 4/1/2000 01:30'!
openSetting
	"#SmallInterfaces2.0.0#Squeak(added)"
	"InterfaceConfiguration openSetting"
	^InterfaceConfiguration current inspect! !


!InterfaceConfiguration class methodsFor: 'EM-Internal'!
initializeAfterLoad

	self nuke! !


Object subclass: #InterfaceHome
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SmallInterfaces'!
!InterfaceHome commentStamp: 'MU 5/15/2000 00:29' prior: 0!
InterfaceHome is the access point for all interfaces.
Its sole responsibility is to provide 'slice & dice' views on interfaces.

As of now it is a pseudo-repository. However, it has a hook to facilitate streaming in & out of interfaces.
This should be done via by implementing a different singltone access.


Class Instance Variables:

singleton	<InterfaceHome>	holds the current InterfaceHome.!


!InterfaceHome methodsFor: 'querying'!
allExtendedInterfaces

	^self allInterfaces select: [:each | each isExtended]! !

!InterfaceHome methodsFor: 'querying'!
allExtendingInterfaces

	^self allInterfaces select: [:each | each isExtending]! !

!InterfaceHome methodsFor: 'querying'!
allInterfaces

	^IdentitySet withAll: Interface subclasses! !

!InterfaceHome methodsFor: 'querying'!
allNonEmptyInterfaces

	^self allInterfaces reject: [:each | each isEmpty]! !

!InterfaceHome methodsFor: 'querying'!
childlessInterfaces

	^self leafInterfaces! !

!InterfaceHome methodsFor: 'querying'!
classesNotFullyImplemented

	| resultSet behaviors |
	behaviors := self environment allNonInterfaceBehaviors.
	resultSet := self allInterfaces 
		inject: Set new
		into: 
			[:set :eachInterface | | nonFullyImplemented |
			nonFullyImplemented := behaviors select: 
				[:eachBehavior | (eachInterface methodsStubedIn: eachBehavior) notEmpty].
			set addAll: nonFullyImplemented; yourself].
	^resultSet collect: [:each | each homeClass].! !

!InterfaceHome methodsFor: 'querying'!
emptyInterfaces

	^self allInterfaces select: [:each | each isEmpty]! !

!InterfaceHome methodsFor: 'querying'!
equivalentInterfaces
	"Return all interfaces who has other interfaces which are equivalent to them."

	^self allInterfaces select: [:each | each hasEquivalents]! !

!InterfaceHome methodsFor: 'querying'!
immutables

	^self allInterfaces select: [:each | each isSealed]! !

!InterfaceHome methodsFor: 'querying'!
includesInterfaceNamed: aSymbol

	^self allInterfaces contains: [:each | each name == aSymbol]! !

!InterfaceHome methodsFor: 'querying'!
interfaceNamed: aSymbol

	^self allInterfaces 
		detect: [:each | each name == aSymbol] 
		ifNone: [nil]! !

!InterfaceHome methodsFor: 'querying'!
interfaceWithRepertoire: repertoire

	^self allInterfaces 
		detect: [:each | each repertoire consistsOf: repertoire]
		ifNone: [nil]! !

!InterfaceHome methodsFor: 'querying'!
interfacesWithRepertoire: repertoire

	^self allInterfaces select: [:each | each repertoire includesAll: repertoire]! !

!InterfaceHome methodsFor: 'querying'!
leafInterfaces

	^Interface consolidate: self allInterfaces! !

!InterfaceHome methodsFor: 'querying'!
mutables

	^self allInterfaces reject: [:each | each isSealed]! !

!InterfaceHome methodsFor: 'querying'!
rootInterfaces

	^self rootsOfTheInterfaceWorld! !

!InterfaceHome methodsFor: 'querying'!
rootsOfTheInterfaceWorld

	^self allInterfaces select: [:each | each isOrphan]! !


!InterfaceHome methodsFor: 'actions'!
removeEmptyInterfaces

	self emptyInterfaces do: [:each | each removeFromSystem]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

InterfaceHome class
	instanceVariableNames: 'singleton '!

!InterfaceHome class methodsFor: 'class initialization'!
obsolete

	self nuke! !


!InterfaceHome class methodsFor: 'instance creation'!
current

	^singleton ifNil: [singleton := super new]! !

!InterfaceHome class methodsFor: 'instance creation'!
new

	^self current! !

!InterfaceHome class methodsFor: 'instance creation'!
nuke

	singleton := nil! !


!InterfaceHome class methodsFor: 'fileIn/Out'!
prerequisitesForLoading

	^super prerequisitesForLoading
		add: Interface;
		yourself! !


Object subclass: #RepertoireHome
	instanceVariableNames: 'classRepertoiresDictionary '
	classVariableNames: 'GenericMetaRepertoire '
	poolDictionaries: ''
	category: 'SmallInterfaces'!
!RepertoireHome commentStamp: 'MU 5/15/2000 00:30' prior: 0!
RepertoireHome has not been commented.  The comment should state the purpose of the class and also explain any unobvious aspects of the implementation.

Instance Variables:
	classRepertoiresDictionary	<Dictionary>	description of classRepertoiresDictionary

Class Variables:
	GenericMetaRepertoire	<>	description of GenericMetaRepertoire

!


!RepertoireHome methodsFor: 'initialize-release' stamp: 'MU 5/6/2000 15:08'!
aspectsOfInterest
	"For the purpose of synchronizing with interfaces, which system changes should we care about?
	Note: we are not interested in:
		#changeClass: - because a class' state has no effect on it behavior.
		#addClass: - because a class repertoire is retrieved lazily."
		
	^#(removeClass: 
	addSelector:class: 
	removeSelector:class:)! !

!RepertoireHome methodsFor: 'initialize-release' stamp: 'MU 4/15/2000 23:37'!
connectToChangeNotifications

	"#SmallInterfaces2.0.0#Squeak(modified)"
	ChangeSet addDependent: self.! !

!RepertoireHome methodsFor: 'initialize-release' stamp: 'MU 4/15/2000 23:37'!
disconnectFromChangeNotifications

	"#SmallInterfaces2.0.0#Squeak(modified)"
	ChangeSet removeDependent: self.! !

!RepertoireHome methodsFor: 'initialize-release'!
initialize

	self 
		computeRepertoires;
		connectToChangeNotifications! !

!RepertoireHome methodsFor: 'initialize-release'!
release

	super release.
	self disconnectFromChangeNotifications.! !


!RepertoireHome methodsFor: 'cache management'!
clearEntryFor: aBehavior

	classRepertoiresDictionary 
		removeKey: aBehavior
		ifAbsent: []! !

!RepertoireHome methodsFor: 'cache management'!
clearRepertoires

	classRepertoiresDictionary := IdentityDictionary new! !

!RepertoireHome methodsFor: 'cache management' stamp: 'MU 4/1/2000 00:57'!
computeRepertoires
	"#SmallInterfaces2.0.0#Squeak(modified)"
	Interface valueWithoutSynchronization: 
		[self clearRepertoires.
		(self progressWidgetAvailable)
			ifTrue: [self computeRepertoiresWithGuiFeedback]
			ifFalse: [self computeRepertoiresWithoutGuiFeedback]]! !

!RepertoireHome methodsFor: 'cache management' stamp: 'MU 4/1/2000 00:30'!
computeRepertoiresWithGuiFeedback

	"#SmallInterfaces2.0.0#Squeak(modified)"
	| classes counter |
	classes := self environment allNonInterfaceClasses.
	counter := 0.
	'Caching repertoires for all classes...'
		displayProgressAt: Sensor cursorPoint
		from: 0
		to: classes size
		during: [:bar | classes do: 
				[:each | 
				self createEntryFor: each.
				counter := counter + 1.
				bar value: counter]]! !

!RepertoireHome methodsFor: 'cache management'!
computeRepertoiresWithoutGuiFeedback

	self environment allNonInterfaceBehaviors do: [:each | self createEntryFor: each]! !

!RepertoireHome methodsFor: 'cache management'!
createEntryFor: aBehavior 

	classRepertoiresDictionary 
		at: aBehavior
		put: aBehavior selectorsForRepertoire! !


!RepertoireHome methodsFor: 'updating'!
update: aClassDefinitionChangedEvent
	"#SmallInterfaces2.0.0#Squeak(added)(moved from original: #update:with:from:)"

	| aspect sender |

	(aClassDefinitionChangedEvent class name == #SIClassDefinitionChangedEvent) ifFalse: [^nil].

	aspect := aClassDefinitionChangedEvent aspect.
	sender := aClassDefinitionChangedEvent source.

	(sender class == ChangeSet and: 
	[Interface configuration synchronize and:
	[self aspectsOfInterest includes: aspect]]) 
		ifTrue:
			[| selector behavior |
			selector := aClassDefinitionChangedEvent selector.
			behavior := aClassDefinitionChangedEvent behavior.

			behavior 
				synchronizeChangeOf: aspect 
				usingSelector: selector.
			self clearEntryFor: behavior].
			
	! !


!RepertoireHome methodsFor: 'querying'!
repertoireFor: aBehavior

	^classRepertoiresDictionary 
		at: aBehavior
		ifAbsentPut: [aBehavior selectorsForRepertoire]! !


!RepertoireHome methodsFor: 'testing' stamp: 'MU 4/1/2000 01:08'!
progressWidgetAvailable
	"#SmallInterfaces2.0.0#Squeak(added)"
	"Squeak can always show Progress"
	^((self environment includesKey: #ProgressWidgetView) and: 
		[self respondsTo: #computeRepertoiresWithGuiFeedback]) or:[true] 
	! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RepertoireHome class
	instanceVariableNames: 'singleton '!

!RepertoireHome class methodsFor: 'startUp/shutDown' stamp: 'MU 5/6/2000 16:10'!
shutDown 
	"#SmallInterfaces2.0.0#Squeak(modified)(moved from the instance side)"
	"The cached repertoires can take a huge amount of space (> 10 Mbytes). 
	To allow for a smaller size image, each time the image is exited the cache can be discarded.
	Note: nuking the singleton is followed by an implicit globalGarbageCollect."
	
	Interface configuration discardInterfacesOnShutDown ifTrue: [RepertoireHome nuke].
	
		"globalGarbageCollect is implide in the shut down process"! !


!RepertoireHome class methodsFor: 'class initialization' stamp: 'MU 4/30/2000 20:19'!
genericMetaRepertoire
	"Repertoire of the meta chain from Class up to Behavior.
	This one, all classes (instances of Metaclass) share."
	"#SmallInterfaces2.0.0#Squeak(modified)"
	GenericMetaRepertoire 
		ifNil:
			[GenericMetaRepertoire := (Class withAllSuperclasses remove: Object; remove: ProtoObject; yourself)	
				inject: (IdentitySet new: 700)
				into: [:set :each | set addAll: each selectors; yourself]].
	^GenericMetaRepertoire copy! !

!RepertoireHome class methodsFor: 'class initialization' stamp: 'MU 4/30/2000 20:22'!
initialize
	"#SmallInterfaces2.0.0#Squeak(modified)"
	"RepertoireHome initialize"
	GenericMetaRepertoire := nil.
	
	self nuke.
	self current.
	
	Smalltalk addToShutDownList: self.! !

!RepertoireHome class methodsFor: 'class initialization' stamp: 'MU 4/1/2000 02:49'!
obsolete
	"#SmallInterfaces2.0.0#Squeak(modified)"
	self nuke.
	
	Smalltalk removeFromShutDownList: self.! !


!RepertoireHome class methodsFor: 'instance creation'!
current

	^singleton ifNil: [singleton := super new initialize]! !

!RepertoireHome class methodsFor: 'instance creation'!
new

	^self current! !

!RepertoireHome class methodsFor: 'instance creation'!
nuke

	singleton notNil ifTrue: [singleton release].
	singleton := nil.! !

InterfaceConfiguration initialize!
RepertoireHome initialize!