"Name:		ANSI Interfaces
Date:			20 May 2000
Author:			Benny Sadeh
Version:	1.9
Distribution:	World
Prerequisites: #('SmallInterfaces.st' )
Description:

ANSI Smalltalk V1.9 interfaces (for Squeak).

Copyright (C) 1999-2000 Benny Sadeh
Copyright (C) 1999-2000 Masashi Umezawa

URL: http://brain.cs.uiuc.edu/VisualWorks/SmallInterfaces (original)
URL: http://www.mars.dti.ne.jp/~umejava/smalltalk/stClasses/smallinterfaces/ (Squeak Port)

"!

| currentState |
currentState := Interface configuration synchronize.
Interface configuration synchronize: false.!


Interface subclass: #numberProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

numberProtocol class
	instanceVariableNames: ''!

numberProtocol comment:
'Provides protocol for objects that represent  numeric  quantities and support operations performing arithmetic, arithmetic progressions, and conversion on numerical quantities. 
The descriptions of messages in this protocol reference specific arithmetic and numerical operations in the ISO/IEC 10967 standard, providing definition-by-reference for these operations.
Smalltalk provides for mixed-mode arithmetic with the receiver and argument having different numeric representations. Unless otherwise specified by an individual operation the receiver and argument are first converted to the same numeric representation according to the Default Conversion Table.
If multiple representations of <Float> are available, the representations are ordered from smallest to largest precision. This table contains multiple entries for <Float>, designated by a subscript, one for each designation of floating point literal representation. Values that are converted to <Float> are converted to the smallest precision of Float that can represent the number of digits in the original value.
An <integer> converted to a <scaledDecimal> will have the scale of the other operand with the fractional digits set to zero.  A <scaledDecimal> converted to a <Fraction> will be a fraction having the same numeric value but having an integer numerator and a denominator which is ten raised to the power of the <scaledDecimal>''s scale factor.
The result type of most numeric opeations is based upon the operaand type.  The Default Result Type for all operand types except <Fraction> is the type to which the operands have been converted according to the Default ConversionTable. If the converted operand type is <Fraction> the Default Result Type is <rational>. In all cases where the type of the return value differs from the default result type it is noted in the operation''s description.
Operations can produce results that are outside the set of representable numbers, or mathematically undefined. It is implementation defined whether errors are raised when results are not representable or if unrepresentable results are wrapped in implementation-defined continuation values or their equivalent. The effect of underflow and overflow is therefore implementation defined.
<number> conforms to <magnitude>. All object that implement the <number> protocol or any protocol that conforms to <number> are comparable.'!

Interface subclass: #monadicBlockProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

monadicBlockProtocol class
	instanceVariableNames: ''!

monadicBlockProtocol comment:
'This protocol describes the behavior for blocks with one argument.
Objects conforming to this protocol can be created only by the block constructor construct of the Smalltalk language.'!

Interface subclass: #ZeroDivideProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ZeroDivideProtocol class
	instanceVariableNames: ''!

ZeroDivideProtocol comment:
'This protocol describes the behavior of exceptions that are signalled when an attempt is made to divide some number (the dividend) by zero.'!

Interface subclass: #readableStringProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

readableStringProtocol class
	instanceVariableNames: ''!

readableStringProtocol comment:
'Provides protocol for string operations such as copying, comparing, replacing, converting, indexing, and matching. All objects that conform to the protocol <readableString> are comparable.'!

Interface subclass: #ByteArrayFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ByteArrayFactoryProtocol class
	instanceVariableNames: ''!

ByteArrayFactoryProtocol comment:
'This protocol defines the behavior of objects that can be used to create objects that conform to <ByteArray>.  These objects are created with a specified size.  If the element values are not explicitly provided, they default to 0.'!

Interface subclass: #collectionStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

collectionStreamProtocol class
	instanceVariableNames: ''!

collectionStreamProtocol comment:
'An object conforming to <collectionStream> has a <sequencedReadableCollection> as its stream backing store.'!

Interface subclass: #dyadicValuableProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

dyadicValuableProtocol class
	instanceVariableNames: ''!

dyadicValuableProtocol comment:
'This protocol describes the behavior for objects supporting the #value:value: selector.'!

Interface subclass: #SortedCollectionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

SortedCollectionProtocol class
	instanceVariableNames: ''!

SortedCollectionProtocol comment:
'Represents a variable sized collection of objects whose elements are ordered based on a sort order.  The sort order is specified by a <dyadicValuable> called the sort block. Elements may be added, removed or inserted, and can be accessed using external integer keys.'!

Interface subclass: #IntervalFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

IntervalFactoryProtocol class
	instanceVariableNames: ''!

IntervalFactoryProtocol comment:
'Represents protocol for creating a collection whose elements are numbers which form an arithmetic progression.'!

Interface subclass: #niladicBlockProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

niladicBlockProtocol class
	instanceVariableNames: ''!

niladicBlockProtocol comment:
'This protocol describes the behavior for blocks with no arguments.
Objects conforming to this protocol can be created only by the block constructor construct of the Smalltalk language.'!

Interface subclass: #WarningProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

WarningProtocol class
	instanceVariableNames: ''!

WarningProtocol comment:
'This protocol describes the behavior of instances of class Warning. These are used to represent exceptional conditions that might occur that are not considered errors but which should be reported to the user. Typically, the actual warning exceptions used by an application will be subclasses of this class.
As Warning is explicitly specified  to be subclassable, conforming implementations must implement its behavior in a non-fragile manner.'!

Interface subclass: #instantiatorProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

instantiatorProtocol class
	instanceVariableNames: ''!

instantiatorProtocol comment:
'This protocol defines the behavior of objects that can be used to create other objects without requiring any additional information.'!

Interface subclass: #FloatProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

FloatProtocol class
	instanceVariableNames: ''!

FloatProtocol comment:
'Represents a floating point representation for real numbers, whose value may be approximate. Provides protocol for performing trigonometry, exponentiation, and conversion on numerical quantities. 
Operations can produce results that are outside the set of representable numbers, or that are mathematically undefined. It is implementation defined whether errors are raised when results are not representable or if unrepresentable results are wrapped in implementation-defined continuation values or their equivalent. The effect of underflow and overflow is therefore implementation defined. It is erroneous if the result of an operation is mathematically undefined.'!

Interface subclass: #ArrayFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ArrayFactoryProtocol class
	instanceVariableNames: ''!

ArrayFactoryProtocol comment:
'This protocol defines the behavior of objects that can be used to create objects that conform to <Array>.  These objects are created with a specified size.  If element values are not explicitly provided they default to nil.'!

Interface subclass: #CharacterFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

CharacterFactoryProtocol class
	instanceVariableNames: ''!

CharacterFactoryProtocol comment:
'This protocol describes the behavior for accessing character objects.'!

Interface subclass: #sequencedStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

sequencedStreamProtocol class
	instanceVariableNames: ''!

sequencedStreamProtocol comment:
'An object conforming to <sequencedStream> has a finite number of past and future sequence values. It maintains a position on its sequence values and allows the position to be altered.'!

Interface subclass: #StringProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

StringProtocol class
	instanceVariableNames: ''!

StringProtocol comment:
'Provides protocol for string operations such as copying, storing, comparing, replacing, converting, indexing, and matching.  The element type of <String> is <Character>.  The range of codePoints of characters that may be elements of a <String> is implementation defined.'!

Interface subclass: #magnitudeProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

magnitudeProtocol class
	instanceVariableNames: ''!

magnitudeProtocol comment:
'Provides protocol for comparing objects which are linearly ordered with respect to some comparison operation.'!

Interface subclass: #FractionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

FractionProtocol class
	instanceVariableNames: ''!

FractionProtocol comment:
'An exact representation for rational numbers. It is unspecific whether the rational number are maintain in a reduced form but messages that reveal the numerator and denominator answer values as if the fraction was reduced.'!

Interface subclass: #extensibleCollectionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

extensibleCollectionProtocol class
	instanceVariableNames: ''!

extensibleCollectionProtocol comment:
'Provides protocol for adding elements to and removing elements from a variable sized collection.'!

Interface subclass: #BagFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

BagFactoryProtocol class
	instanceVariableNames: ''!

BagFactoryProtocol comment:
'This protocol defines the behavior of objects that can be used to create objects that conform to the protocol <Bag>.'!

Interface subclass: #rationalProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

rationalProtocol class
	instanceVariableNames: ''!

rationalProtocol comment:
'Rational numbers may be either integers or fractions.  An integer is logically a fraction whose denominator is one. This protocol is necessary because some integer and most fraction operations can produce results that may be either an integer or a fraction.'!

Interface subclass: #scaledDecimalProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

scaledDecimalProtocol class
	instanceVariableNames: ''!

scaledDecimalProtocol comment:
'Provides a numeric representation of fixed point decimal numbers. The representation must be able to accurately represent decimal fractions. The standard recommends that the implementation of this protocol support unbounded precision, with no limit to the number of digits before and after the decimal point. If a bounded implementation is provided, then any operation which exceeds the bounds has an implementation-specified  result.'!

Interface subclass: #SetProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

SetProtocol class
	instanceVariableNames: ''!

SetProtocol comment:
'Represents an unordered, variable sized collection whose elements can be added or removed, but cannot be individually accessed by external keys. A set is similar to a bag but cannot contain duplicate elements.'!

Interface subclass: #DurationProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

DurationProtocol class
	instanceVariableNames: ''!

DurationProtocol comment:
'Represents a length of time.'!

Interface subclass: #exceptionInstantiatorProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

exceptionInstantiatorProtocol class
	instanceVariableNames: ''!

exceptionInstantiatorProtocol comment:
'This protocol describes the instantiation behavior of objects that can create exceptions.'!

Interface subclass: #FileStreamFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

FileStreamFactoryProtocol class
	instanceVariableNames: ''!

FileStreamFactoryProtocol comment:
'<FileStream Factory> provides for the creation of objects conforming to the <readFileStream> or <writeFileStream>'!

Interface subclass: #ObjectClassProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ObjectClassProtocol class
	instanceVariableNames: ''!

ObjectClassProtocol comment:
'This protocol describes the behavior the class object whose global identifier is ''Object'', which is the traditional root of the class hierarchy.
This class must be implemented in such a way that it is not fragile. A class is said to be fragile if it is implemented in such a way that subclasses of that class can change the behavior of any standard-specified method without overriding the implementation of those methods. This can happen when a method is implemented to use an auxiliary method that is not specified in the standard, which the subclass then (possibly unintentionally) overrides. The inherited method will then invoke the subclass'' implementation of the auxiliary method rather than the expected implementation in the superclass.
One way to ensure that the implementation of a class is not fragile is to ensure that any message sent to self is either part of the specified behavior for that class or has a selector that begins with an underscore. Alternatively, an implementation may use implementation-specific means to implement these methods in a way that makes them non-fragile.'!

Interface subclass: #readFileStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

readFileStreamProtocol class
	instanceVariableNames: ''!

readFileStreamProtocol comment:
'Provides protocol for traversing and reading elements in an external file. The sequence values are provided by the external file which also serves as the stream backing store. '!

Interface subclass: #DateAndTimeFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

DateAndTimeFactoryProtocol class
	instanceVariableNames: ''!

DateAndTimeFactoryProtocol comment:
'Represents protocol for creating an abstraction for a particular day of the year.'!

Interface subclass: #ErrorClassProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ErrorClassProtocol class
	instanceVariableNames: ''!

ErrorClassProtocol comment:
'This protocol describe the behavior of the global Error. The value of the standard global Error is a class object that conforms to this protocol. The class Error is explicitly specified  to be subclassable in a standard conforming program. Conforming implementations must implement its behaviors in a non-fragile manner.
The signaled exceptions generated by this type of object conform to the protocol <Error>.'!

Interface subclass: #IntervalProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

IntervalProtocol class
	instanceVariableNames: ''!

IntervalProtocol comment:
'Represents a collection whose elements are numbers which form an arithmetic progression. Elements cannot be accessed externally.'!

Interface subclass: #niladicValuableProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

niladicValuableProtocol class
	instanceVariableNames: ''!

niladicValuableProtocol comment:
'This protocol describes the behavior for objects supporting the #value selector.'!

Interface subclass: #writeFileStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

writeFileStreamProtocol class
	instanceVariableNames: ''!

writeFileStreamProtocol comment:
'Provides protocol for storing elements in an external file. The sequence values are provided by the external file which also serves as the stream backing store. A <writeFileStream> is a write-back stream.'!

Interface subclass: #OrderedCollectionFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

OrderedCollectionFactoryProtocol class
	instanceVariableNames: ''!

OrderedCollectionFactoryProtocol comment:
'This protocol defines the behavior of objects that can be used to create fixed sized ordered collections of objects which can be accessed externally using integer keys.'!

Interface subclass: #ArrayProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ArrayProtocol class
	instanceVariableNames: ''!

ArrayProtocol comment:
'Represents a keyed collection of objects which can be accessed externally using sequential integer keys. The index of the first element is one (1).'!

Interface subclass: #CharacterProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

CharacterProtocol class
	instanceVariableNames: ''!

CharacterProtocol comment:
'This protocol describes the behavior that is common to character objects. Character objects serve as the element value for Smalltalk strings. The Smalltalk language provides a literal syntax for character objects. Character objects represent individual elements of an implementation defined execution character set whose individual elements are identified by integer values. These integers are called code points.  Each character object has an associated code point.
It is unspecified whether or not each code point is uniquely associated with a unique character object.
The execution character set is the character set used by an implementation during execution of a Smalltalk program.  It need not be the same as the character set used by that implementation to encode the definition of Smalltalk programs.'!

Interface subclass: #integerProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

integerProtocol class
	instanceVariableNames: ''!

integerProtocol comment:
'Represents an abstraction for integer numbers whose value is exact. Representations must provide unbounded precision and range, hence the ISO/IEC 10967 integer type parameter bounded is bound to false.'!

Interface subclass: #DictionaryFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

DictionaryFactoryProtocol class
	instanceVariableNames: ''!

DictionaryFactoryProtocol comment:
'This protocol defines the behavior of objects that can be used to create objects that conform to the protocol <Dictionary>.'!

Interface subclass: #symbolProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

symbolProtocol class
	instanceVariableNames: ''!

symbolProtocol comment:
'Represents an ordered, variable sized and immutable collection of characters. There is a unique object conforming to this protocol for every possible sequence of characters.  Symbols are identity objects.'!

Interface subclass: #abstractDictionaryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

abstractDictionaryProtocol class
	instanceVariableNames: ''!

abstractDictionaryProtocol comment:
'Provides protocol for accessing, adding, removing, and iterating over the elements of an unordered collection whose elements are accessed using an explicitly assigned external key.
Glossary Entries'!

Interface subclass: #SetFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

SetFactoryProtocol class
	instanceVariableNames: ''!

SetFactoryProtocol comment:
'This protocol defines the behavior of objects that can be used to create objects that conform to the protocol <Set>.'!

Interface subclass: #MessageNotUnderstoodProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

MessageNotUnderstoodProtocol class
	instanceVariableNames: ''!

MessageNotUnderstoodProtocol comment:
'This protocol describes the behavior of exceptions that are signalled if the receiver of a message does not have a method with a matching  selector.'!

Interface subclass: #WriteStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

WriteStreamProtocol class
	instanceVariableNames: ''!

WriteStreamProtocol comment:
'An object conforming to <WriteStream> has a positionable sequence of values to which new values may be written. The initial sequence values are provided by a collection that serves as the stream backing store. It is implementation defined whether a <WriteStream> is a write-back stream. Even if a <WriteStream> is not a write-back stream, its associated collection may be subject to modification in an unspecified manner as long as it is associated with the stream.'!

Interface subclass: #failedMessageProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

failedMessageProtocol class
	instanceVariableNames: ''!

failedMessageProtocol comment:
'This protocol describes the behavior of objects that represent a message that was sent to an object, but was not understood by that object.'!

Interface subclass: #gettableStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

gettableStreamProtocol class
	instanceVariableNames: ''!

gettableStreamProtocol comment:
'An object conforming to <gettableStream> can read objects from its future sequence values.'!

Interface subclass: #BagProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

BagProtocol class
	instanceVariableNames: ''!

BagProtocol comment:
'Represents an unordered, variable sized collection whose elements can be added or removed, but cannot be individually accessed by external keys. A bag is similar to a set but can contain duplicate elements. Elements are duplicates if they are equivalent.'!

Interface subclass: #selectorProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

selectorProtocol class
	instanceVariableNames: ''!

selectorProtocol comment:
'Defines the protocol supported by literal message selectors. No behavior is defined by this protocols but objects that conform to is can be used to perform dynamically generated message sends  using <Object> #perform: and related messages.'!

Interface subclass: #signaledExceptionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

signaledExceptionProtocol class
	instanceVariableNames: ''!

signaledExceptionProtocol comment:
'This protocol describes the messages that may be sent to the argument of a handler block. These message are used to explicitly control how execution will continue when it leaves the handler block.'!

Interface subclass: #valuableProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

valuableProtocol class
	instanceVariableNames: ''!

valuableProtocol comment:
'This protocol describes the behavior for objects that can be evaluated using variants of the #value message.'!

Interface subclass: #ExceptionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ExceptionProtocol class
	instanceVariableNames: ''!

ExceptionProtocol comment:
'This protocol describes the behavior of instances of class Exception. Typically, actual exceptions used by an application will be either direct or indirect subclasses of this class. Exception combines the behavior of <exceptionBuilder> and <signaledException>.  Instances are used to both supplied inform before an exception is signaled and to pass the information to an exception handler.
As Exception is explicitly specified  to be subclassable, conforming implementations must implement its behavior in a non-fragile manner.'!

Interface subclass: #FileStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

FileStreamProtocol class
	instanceVariableNames: ''!

FileStreamProtocol comment:
'Provides protocol for streams over external files. The external file serves as the stream backing store. When objects are read or written from a file stream they must be translated from or two an external data representation. File streams have an external stream type that is specified when the stream is created. The external stream type defines the data translation and the sequence value type for the stream. External stream types are specified using <symbol> objects. The standard defines the following external stream types:
#''binary''	The external data is treated as sequence of 8-bit bytes. The sequence value type is <integer> with values restricted to the range 0 to 255.
#''text'' 	The external data is treated as a sequenced of 8-bit characters encoded using an implementation defined external character set.  The sequence value type is <Character> restricted to those specific characters that may be represented in the external character set.
Implementations may define other external stream types.'!

Interface subclass: #IdentityDictionaryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

IdentityDictionaryProtocol class
	instanceVariableNames: ''!

IdentityDictionaryProtocol comment:
'This protocol defines the behavior of unordered collections whose elements can be accessed using an explicitly-assigned, external key.  Key equivalence is defined as sending the #== message.'!

Interface subclass: #ObjectProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ObjectProtocol class
	instanceVariableNames: ''!

ObjectProtocol comment:
'This protocol describe the behavior that is common to all objects.'!

Interface subclass: #ReadStreamFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ReadStreamFactoryProtocol class
	instanceVariableNames: ''!

ReadStreamFactoryProtocol comment:
'<ReadStream Factory> provides for the creation of objects conforming to the <ReadStream> protocol whose sequence values are supplied by a collection.'!

Interface subclass: #sequencedContractibleCollectionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

sequencedContractibleCollectionProtocol class
	instanceVariableNames: ''!

sequencedContractibleCollectionProtocol comment:
'Provides protocol for removing elements from an ordered collection of objects, whose elements can be accessed using external integer keys.'!

Interface subclass: #DateAndTimeProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

DateAndTimeProtocol class
	instanceVariableNames: ''!

DateAndTimeProtocol comment:
'This protocol describes the behavior that is common to date time objects. Date time objects represent individual points in Coordinated Universal Time (UTC) as represented in an implementation defined local time.  
The exact properties of local times are unspecified. Local times may differ in their offset from UTC.  A given local time may have different offsets from UTC at different points in time.
All dates and times in the UTC local time are in the Gregorian calendar. Date times prior to the adoption of the Gregorian calendar are given in the retrospective astronomical Gregorian calendar. The year 1 B.C. is astronomical Gregorian year 0. The year 2 B.C. is astronomical Gregorian year -1. The year 1 A.D. is astronomical Gregorian year 1. The offset of the UTC local time is zero.'!

Interface subclass: #ErrorProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ErrorProtocol class
	instanceVariableNames: ''!

ErrorProtocol comment:
'This protocol describes the behavior of instances of class Error. These are used to represent error conditions that prevent the normal continuation of processing. Actual error exceptions used by an application may be subclasses of this class.
As Error is explicitly specified  to be subclassable, conforming implementations must implement its behavior in a non-fragile manner.'!

Interface subclass: #exceptionSetProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

exceptionSetProtocol class
	instanceVariableNames: ''!

exceptionSetProtocol comment:
'This protocol describes the behavior of objects that may be used to group a set of <exceptionSelector> objects into a single <exceptionSelector>. This is useful for establishing a single exception handler that may deal with several different types of exceptions.'!

Interface subclass: #nilProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

nilProtocol class
	instanceVariableNames: ''!

nilProtocol comment:
'This protocol describes the behavior that is unique to the distinguished immutable, identity object that is the value of the reserved identifier ''nil''.'!

Interface subclass: #WriteStreamFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

WriteStreamFactoryProtocol class
	instanceVariableNames: ''!

WriteStreamFactoryProtocol comment:
'<WriteStream Factory> provides for the creation of objects conforming to the <WriteStream> protocol whose sequence values are supplied by a collection.'!

Interface subclass: #OrderedCollectionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

OrderedCollectionProtocol class
	instanceVariableNames: ''!

OrderedCollectionProtocol comment:
'Represents an ordered, variable sized collection of objects. Elements may be added, removed or inserted, and can be accessed using external integer keys.'!

Interface subclass: #ReadWriteStreamFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ReadWriteStreamFactoryProtocol class
	instanceVariableNames: ''!

ReadWriteStreamFactoryProtocol comment:
'<ReadWriteStreamfactory> provides for the creation of objects conforming to the <WriteStream> protocol whose sequence values are supplied by a collection.'!

Interface subclass: #classDescriptionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

classDescriptionProtocol class
	instanceVariableNames: ''!

classDescriptionProtocol comment:
'This protocol describes the behavior of class objects. It provides messages for identifying and locating class objects within the class hierarchy.'!

Interface subclass: #DictionaryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

DictionaryProtocol class
	instanceVariableNames: ''!

DictionaryProtocol comment:
'Represents an unordered collection whose elements can be accessed using an explicitly assigned external key. Key equivalence is defined as sending the #= message.'!

Interface subclass: #ExceptionClassProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ExceptionClassProtocol class
	instanceVariableNames: ''!

ExceptionClassProtocol comment:
'This protocol describe the behavior of class objects that are used to create, signal, and select exceptions that exist within a specialization hierarchy.
The value of the standard global Exception is a class object that conforms to this protocol. The class Exception is explicitly specified  to be subclassable. Conforming implementations must implement its behaviors in a non-fragile manner.'!

Interface subclass: #NotificationProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

NotificationProtocol class
	instanceVariableNames: ''!

NotificationProtocol comment:
'This protocol describes the behavior of instances of the class Notification. These are used to represent exceptional conditions that may occur but which are not considered errors. Actual notification exceptions used by an application may be subclasses of this class.
As Notification is explicitly specified  to be subclassable, conforming implementations must implement its behavior in a non-fragile manner.'!

Interface subclass: #TranscriptProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

TranscriptProtocol class
	instanceVariableNames: ''!

TranscriptProtocol comment:
'An object conforming to <Transcript> is a <puttableStream> for logging status messages from Smalltalk programs. The sequence value type of <Transcript> is <Character>. There may be an implementation defined stream backing store that receives characters written to the stream in an implementatiuon defined manner.'!

Interface subclass: #MessageNotUnderstoodSelectorProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

MessageNotUnderstoodSelectorProtocol class
	instanceVariableNames: ''!

MessageNotUnderstoodSelectorProtocol comment:
'This protocol describe the behavior of the value of the global named MessageNotUnderstood. This object  is used to as an exception selector to catch failed message sends. Message not understood exceptions are resumable so any message in this protocol that signal such an exception may ultimately return to their sender.
This object is not specifed as an <exceptionSignaler> or an <exceptionInstantiator>. It as assumed that message not understood exceptions are signaled by the implemention dependent implementaton of the message <Object> #doesNotUnderstand:.'!

Interface subclass: #ZeroDivideFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ZeroDivideFactoryProtocol class
	instanceVariableNames: ''!

ZeroDivideFactoryProtocol comment:
'This protocol describe the behavior of the global ZeroDivide. It is used to as an exception selector to catch zero divide exceptions and can also be used to signal that a division by zero error has occured. Zero divide exceptions are resumable so any message in this protocol that signal such an exception may ultimately return to their sender. The signaled exceptions generated by this type of object conform to the protocol <ZeroDivide> '!

Interface subclass: #IdentityDictionaryFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

IdentityDictionaryFactoryProtocol class
	instanceVariableNames: ''!

IdentityDictionaryFactoryProtocol comment:
'This protocol defines the behavior of objects that can be used to create objects that conform to the protocol <IdentityDictionary>.'!

Interface subclass: #booleanProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

booleanProtocol class
	instanceVariableNames: ''!

booleanProtocol comment:
'This protocol describes the behavior of the objects that are the values of the reserved identifiers ''true'' and ''false''.  These objects are identity objects.
Several message specifications include a truth table describing the result of the binary operation implemented by that message. In each table, the value of the receiver is used to locate a row and the value of the argument is used to locate a column, the result being located at the intersection of the row and column.'!

Interface subclass: #collectionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

collectionProtocol class
	instanceVariableNames: ''!

collectionProtocol comment:
'Provides protocol for manipulating and operating on a collection of objects, called elements, either individually or as a whole. A collection can be fixed or variable sized, ordered or unordered, and its elements may or may not be accessible by external keys.
Some implementations of collections may choose to use the hash values, as defined by either the message #hash or the message #identityHash, of either the elements of the collection or the keys by which those elements are accessed (if there are any). If the hash values of such objects are modified, the behavior of any message sent to such a collection is undefined until the message #rehash has been sent to the collection in order to restore the consistency of the collection.'!

Interface subclass: #sequencedCollectionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

sequencedCollectionProtocol class
	instanceVariableNames: ''!

sequencedCollectionProtocol comment:
'Provides protocol for writing to an ordered collection of objects, whose elements can be accessed using external integer keys.'!

Interface subclass: #monadicValuableProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

monadicValuableProtocol class
	instanceVariableNames: ''!

monadicValuableProtocol comment:
'This protocol describes the behavior for objects supporting the value: selector.'!

Interface subclass: #SortedCollectionFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

SortedCollectionFactoryProtocol class
	instanceVariableNames: ''!

SortedCollectionFactoryProtocol comment:
'Represents protocol for creating a variable sized collection of objects whose elements are ordered based on a sort order specified by a two parameter block called the sort block. Elements may be added, removed or inserted, and can be accessed using external integer keys.'!

Interface subclass: #WarningClassProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

WarningClassProtocol class
	instanceVariableNames: ''!

WarningClassProtocol comment:
'This protocol describe the behavior of the global Warning. The value of the standard global Warning is a class object that conforms to this protocol. The class Warning is explicitly specified  to be subclassable in a standard conforming program.  Conforming implementations must implement its behaviors in a non-fragile manner.
The signaled exceptions generated by this type of object conform to the protocol <Warning>.'!

Interface subclass: #exceptionSelectorProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

exceptionSelectorProtocol class
	instanceVariableNames: ''!

exceptionSelectorProtocol comment:
'This protocol describe the behavior of objects that are used to select an exception handler. In particular, objects that conform to this protocol may occur as the first argument to #on:do: message sent to blocks.'!

Interface subclass: #ByteArrayProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ByteArrayProtocol class
	instanceVariableNames: ''!

ByteArrayProtocol comment:
'Represents a keyed collection whose element type is <integer> and is limited to the range 0 to 255, inclusive. The elements can be accessed externally using sequential integer keys. The index of the first element is one (1).'!

Interface subclass: #floatCharacterizationProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

floatCharacterizationProtocol class
	instanceVariableNames: ''!

floatCharacterizationProtocol comment:
'Objects supporting this protocol characterize a floating point representation for real numbers. These characterizations are required by ISO/IEC 10967 for each precision of floating point numbers provided by an implementation.'!

Interface subclass: #initializableCollectionFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

initializableCollectionFactoryProtocol class
	instanceVariableNames: ''!

initializableCollectionFactoryProtocol comment:
'This protocol defines the behavior of objects that can be used to create non-empty collections.'!

Interface subclass: #ReadStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ReadStreamProtocol class
	instanceVariableNames: ''!

ReadStreamProtocol comment:
'An object conforming to <ReadStream> has a positionable sequence of values that can be read. The sequence values are provided by a sequenced collection that serves as the stream backing store. '!

Interface subclass: #sequencedReadableCollectionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

sequencedReadableCollectionProtocol class
	instanceVariableNames: ''!

sequencedReadableCollectionProtocol comment:
'Provides protocol for reading an ordered collection of objects whose elements can be accessed using external integer keys. The keys are between one (1) and the number of elements in the collection, inclusive.'!

Interface subclass: #StringFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

StringFactoryProtocol class
	instanceVariableNames: ''!

StringFactoryProtocol comment:
'This protocol defines the behavior of objects that can be used to create objects that conform to <String>.  These objects are created with a specified size.'!

Interface subclass: #exceptionBuilderProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

exceptionBuilderProtocol class
	instanceVariableNames: ''!

exceptionBuilderProtocol comment:
'This protocol describes the messages that may be used to set the information about an occurrence of an exception. This information may be retrieved using <exceptionDescription>  protocol. If an object conforming to this protocol is signaled as an exception, any information set in that object using this protocol''s messages will also be available for retrival from the signaled exception  that is passed to a handler block.'!

Interface subclass: #exceptionSignalerProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

exceptionSignalerProtocol class
	instanceVariableNames: ''!

exceptionSignalerProtocol comment:
'This protocol describes the behavior of signaling an exceptional condition, locating an exception handler, and executing an exception action.
'!

Interface subclass: #FractionFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

FractionFactoryProtocol class
	instanceVariableNames: ''!

FractionFactoryProtocol comment:
'Represents protocol for creating an exact representation for rational numbers.'!

Interface subclass: #NotificationClassProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

NotificationClassProtocol class
	instanceVariableNames: ''!

NotificationClassProtocol comment:
'This protocol describe the behavior of the global Notification. The value of the standard global Notification is a class object that conforms to this protocol. The class Notification is explicitly specified  to be subclassable in a standard conforming program. Conforming implementations must implement its behaviors in a non-fragile manner.
The signaled exceptions generated by this type of object conform to the protocol <Notification>.'!

Interface subclass: #puttableStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

puttableStreamProtocol class
	instanceVariableNames: ''!

puttableStreamProtocol comment:
'An object conforming to <puttableStream> allows objects to be added to its past sequence values.'!

Interface subclass: #ReadWriteStreamProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

ReadWriteStreamProtocol class
	instanceVariableNames: ''!

ReadWriteStreamProtocol comment:
'An object conforming to <ReadWriteStream> can read from its future sequence values or write to its past sequence values. The sequence values are provided by a collection that serves as the stream backing store. It is implementation defined whether a <ReadWriteStream> is a write-back stream. Even if a <ReadWriteStream> is not a write-back stream, its associated collection may be subject to modification in an unspecified manner as long as it is associated with the stream.'!

Interface subclass: #collectionFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

collectionFactoryProtocol class
	instanceVariableNames: ''!

collectionFactoryProtocol comment:
'Provides protocol for creating a collection of objects. A collection can be fixed or variable sized, ordered or unordered, and its elements may or may not be accessible by external keys.'!

Interface subclass: #DurationFactoryProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

DurationFactoryProtocol class
	instanceVariableNames: ''!

DurationFactoryProtocol comment:
'Represents protocol for creating a particular length of time.'!

Interface subclass: #exceptionDescriptionProtocol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ANSI Interfaces'!

exceptionDescriptionProtocol class
	instanceVariableNames: ''!

exceptionDescriptionProtocol comment:
'This protocol describe the messages that may be used to obtain information about an occurrence of an exception.'!

!numberProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!numberProtocol methodsFor: 'magnitudeProtocol'!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!numberProtocol methodsFor: 'repertoire'!

* operand 
	"Answer the result of multiplying the receiver by operand."

	^self implementorsResponsibility!

+ operand 
	"Answer the result of adding operand to the receiver."

	^self implementorsResponsibility!

- operand 
	"Answer the result of subtracting operand from the receiver."

	^self implementorsResponsibility!

/ operand 
	"Answer the result of dividing the receiver by operand."

	^self implementorsResponsibility!

// operand 
	"Answer the truncated quotient resulting from dividing the receiver by operand. The truncation is towards negative infinity."

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

abs
	"Answer the absolute value of the receiver."

	^self implementorsResponsibility!

asFloat
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatD
	"Answer a d precision floating-point number approximating the receiver ."

	^self implementorsResponsibility!

asFloatE
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatQ
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFraction
	"Answer a fraction approximating the receiver."

	^self implementorsResponsibility!

asInteger
	"Answer an integer approximating the receiver."

	^self implementorsResponsibility!

asScaledDecimal: scale 
	"Answer a scaled decimal number, with a fractional precision of scale, approximating the receiver."

	^self implementorsResponsibility!

ceiling
	"Answer the smallest integer greater than or equal to the receiver."

	^self implementorsResponsibility!

floor
	"Answer the largest integer less than or equal to the receiver."

	^self implementorsResponsibility!

fractionPart
	"Answer the fractional part of the receiver."

	^self implementorsResponsibility!

integerPart
	"Answer the integer part of the receiver."

	^self implementorsResponsibility!

negated
	"Answer the negation of the receiver."

	^self implementorsResponsibility!

negative
	"Answer true if the receiver is negative."

	^self implementorsResponsibility!

positive
	"Answer true if the receiver is positive or zero."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

quo: operand 
	"Answer the truncated integer quotient resulting from dividing the receiver by operand. Truncation is towards zero."

	^self implementorsResponsibility!

raisedTo: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

raisedToInteger: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

reciprocal
	"Answer the reciprocal of the receiver."

	^self implementorsResponsibility!

rem: operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility!

rounded
	"Answer the integer nearest the receiver."

	^self implementorsResponsibility!

roundTo: factor 
	"Answer the number nearest the receiver that is a multiple of factor."

	^self implementorsResponsibility!

sign
	"Answer the sign of the receiver."

	^self implementorsResponsibility!

sqrt
	"Answer the positive square root of the receiver."

	^self implementorsResponsibility!

squared
	"Answer the receiver squared."

	^self implementorsResponsibility!

strictlyPositive
	"Answer true if the receiver is greater than zero."

	^self implementorsResponsibility!

to: stop 
	"Answer an object conforming to <intervalProtocol> which represents an arithmetic progression from the receiver to stop in increments of 1."

	^self implementorsResponsibility!

to: stop by: step 
	"Answer an interval which represents an arithmetic progression from receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop by: step do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from the receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from receiver to stop in increments of 1."

	^self implementorsResponsibility!

truncated
	"Answer an integer equal to the receiver truncated towards zero."

	^self implementorsResponsibility!

truncateTo: factor 
	"Answer the number nearest the receiver truncated towards zero which is a multiple of factor."

	^self implementorsResponsibility!

\\ operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility! !


!monadicBlockProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!monadicBlockProtocol methodsFor: 'valuableProtocol'!

valueWithArguments: argumentArray 
	"Answers the value of the receiver when applied to the arguments in argumentArray."

	^self implementorsResponsibility! !

!monadicBlockProtocol methodsFor: 'monadicValuableProtocol'!

argumentCount
	"Answers the number of arguments needed to evaluate the receiver."

	^self implementorsResponsibility!

value: argument 
	"Answers the value of the receiver when applied to the argument."

	^self implementorsResponsibility! !


!ZeroDivideProtocol methodsFor: 'signaledExceptionProtocol'!

isNested
	"Determine whether the current exception handler is within the scope of another handler for the same exception."

	^self implementorsResponsibility!

outer
	"Evaluate the enclosing exception action for the receiver and return."

	^self implementorsResponsibility!

pass
	"Yield control to the enclosing exception action for the receiver."

	^self implementorsResponsibility!

resignalAs: replacementException 
	"Signal an alternative exception in place of the receiver."

	^self implementorsResponsibility!

resume
	"Return from the message that signaled the receiver."

	^self implementorsResponsibility!

resume: resumptionValue 
	"Return the argument as the value of the message that signaled the receiver."

	^self implementorsResponsibility!

retry
	"Abort an exception handler and re-evaluate its protected block."

	^self implementorsResponsibility!

retryUsing: alternativeBlock 
	"Abort an exception handler and evaluate a new block in place of the handler's protected block."

	^self implementorsResponsibility!

return
	"Return nil as the value of the block protected by the active exception handler."

	^self implementorsResponsibility!

return: returnValue 
	"Return the argument as the value of the block protected by the active exception handler."

	^self implementorsResponsibility! !

!ZeroDivideProtocol methodsFor: 'ErrorProtocol'!

defaultAction

	^self implementorsResponsibility! !

!ZeroDivideProtocol methodsFor: 'exceptionBuilderProtocol'!

messageText: signalerText 
	"Set an exception's message text."

	^self implementorsResponsibility! !

!ZeroDivideProtocol methodsFor: 'repertoire'!

dividend
	"Answer the number that was being divided by zero."

	^self implementorsResponsibility!

isResumable
	"Determine whether an exception is resumable."

	^self implementorsResponsibility! !


!readableStringProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!readableStringProtocol methodsFor: 'magnitudeProtocol'!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!readableStringProtocol methodsFor: 'sequencedReadableCollectionProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !

!readableStringProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!readableStringProtocol methodsFor: 'repertoire'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

asLowercase
	"Answer a new string which contains all of the elements of the receiver converted to their lower case equivalents."

	^self implementorsResponsibility!

asString
	"Answer a string containing the same characters as the receiver."

	^self implementorsResponsibility!

asSymbol
	"Answer a symbol containing the same characters as the receiver."

	^self implementorsResponsibility!

asUppercase
	"Answer a new string which contains all of the elements of the receiver converted to their upper case equivalents."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

sameAs: operand 
	"Answer true if the receiver collates the same as operand. Answer false otherwise."

	^self implementorsResponsibility!

subStrings: separators 
	"Answer an array containing the substrings in the receiver separated by the elements of separators."

	^self implementorsResponsibility! !


!ByteArrayFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ByteArrayFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility!

with: element1 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 with: element4 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing only the elements of newElements."

	^self implementorsResponsibility! !


!collectionStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!collectionStreamProtocol methodsFor: 'sequencedStreamProtocol'!

close
	"Disassociate a stream from its backing store."

	^self implementorsResponsibility!

isEmpty
	"Returns a Boolean indicating whether there are any sequence values in the receiver."

	^self implementorsResponsibility!

position
	"Returns the current position of the stream."

	^self implementorsResponsibility!

position: amount 
	"Sets the current position in a stream of values."

	^self implementorsResponsibility!

reset
	"Resets the position of the receiver to be at the beginning of the stream of values."

	^self implementorsResponsibility!

setToEnd
	"Set the position of the stream to its end."

	^self implementorsResponsibility! !

!collectionStreamProtocol methodsFor: 'repertoire'!

contents
	"Returns a collection containing the complete contents of the stream."

	^self implementorsResponsibility! !


!dyadicValuableProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!dyadicValuableProtocol methodsFor: 'valuableProtocol'!

valueWithArguments: argumentArray 
	"Answers the value of the receiver when applied to the arguments in argumentArray."

	^self implementorsResponsibility! !

!dyadicValuableProtocol methodsFor: 'repertoire'!

argumentCount
	"Answers the number of arguments needed to evaluate the receiver."

	^self implementorsResponsibility!

value: argument1 value: argument2 
	"Answers the value of the receiver when applied to the arguments."

	^self implementorsResponsibility! !


!SortedCollectionProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!SortedCollectionProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!SortedCollectionProtocol methodsFor: 'extensibleCollectionProtocol'!

addAll: newElements 
	"Add each element of newElements to the receiver's elements."

	^self implementorsResponsibility!

remove: oldElement 
	"Remove the first element of the receiver which is equivalent to oldElement and return the removed element."

	^self implementorsResponsibility!

remove: oldElement ifAbsent: exceptionHandler 
	"Remove the first element of the receiver which is equivalent to oldElement. If it is not found, answer the result of evaluating exceptionHandler."

	^self implementorsResponsibility!

removeAll: oldElements 
	"For each element in oldElements, remove the first element from the receiver which is equivalent to this element."

	^self implementorsResponsibility! !

!SortedCollectionProtocol methodsFor: 'sequencedContractibleCollectionProtocol'!

removeAtIndex: index 
	"Remove the element of the receiver at position index, and answer the removed element."

	^self implementorsResponsibility!

removeFirst
	"Remove and answer the first element of the receiver."

	^self implementorsResponsibility!

removeLast
	"Remove and answer the last element of the receiver."

	^self implementorsResponsibility! !

!SortedCollectionProtocol methodsFor: 'sequencedReadableCollectionProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !

!SortedCollectionProtocol methodsFor: 'repertoire'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

add: newElement 
	"Add newElement to the receiver's elements."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

sortBlock
	"Answer the receiver's sort block."

	^self implementorsResponsibility!

sortBlock: discriminator 
	"Set the receiver's sort block to discriminator."

	^self implementorsResponsibility! !


!IntervalFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!IntervalFactoryProtocol methodsFor: 'repertoire'!

from: start to: stop 
	"Answer an interval which represents an arithmetic progression from start to stop in increments of 1."

	^self implementorsResponsibility!

from: start to: stop by: step 
	"Answer an interval which represents an arithmetic progression from start to stop in increments of step."

	^self implementorsResponsibility! !


!niladicBlockProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!niladicBlockProtocol methodsFor: 'niladicValuableProtocol'!

argumentCount
	"Answers the number of arguments needed to evaluate the receiver."

	^self implementorsResponsibility!

value
	"Answers the value of the receiver."

	^self implementorsResponsibility!

whileFalse
	"Evaluates the receiver until it evaluates to true."

	^self implementorsResponsibility!

whileFalse: iterationBlock 
	"Evaluates iterationBlock zero or more times until the receiver evaluates to true."

	^self implementorsResponsibility!

whileTrue
	"Evaluates the receiver until it evaluates to false."

	^self implementorsResponsibility!

whileTrue: iterationBlock 
	"Evaluates iterationBlock zero or more times until the receiver evaluates to false."

	^self implementorsResponsibility! !

!niladicBlockProtocol methodsFor: 'valuableProtocol'!

valueWithArguments: argumentArray 
	"Answers the value of the receiver when applied to the arguments in argumentArray."

	^self implementorsResponsibility! !

!niladicBlockProtocol methodsFor: 'repertoire'!

ensure: terminationBlock 
	"Evaluate a termination block after evaluating the receiver."

	^self implementorsResponsibility!

ifCurtailed: terminationBlock 
	"Evaluating the receiver with an abnormal termination action."

	^self implementorsResponsibility!

on: selector do: action 
	"Evaluate the receiver in the scope of an exception handler."

	^self implementorsResponsibility! !


!WarningProtocol methodsFor: 'signaledExceptionProtocol'!

isNested
	"Determine whether the current exception handler is within the scope of another handler for the same exception."

	^self implementorsResponsibility!

outer
	"Evaluate the enclosing exception action for the receiver and return."

	^self implementorsResponsibility!

pass
	"Yield control to the enclosing exception action for the receiver."

	^self implementorsResponsibility!

resignalAs: replacementException 
	"Signal an alternative exception in place of the receiver."

	^self implementorsResponsibility!

resume
	"Return from the message that signaled the receiver."

	^self implementorsResponsibility!

resume: resumptionValue 
	"Return the argument as the value of the message that signaled the receiver."

	^self implementorsResponsibility!

retry
	"Abort an exception handler and re-evaluate its protected block."

	^self implementorsResponsibility!

retryUsing: alternativeBlock 
	"Abort an exception handler and evaluate a new block in place of the handler's protected block."

	^self implementorsResponsibility!

return
	"Return nil as the value of the block protected by the active exception handler."

	^self implementorsResponsibility!

return: returnValue 
	"Return the argument as the value of the block protected by the active exception handler."

	^self implementorsResponsibility! !

!WarningProtocol methodsFor: 'NotificationProtocol'!

isResumable

	^self implementorsResponsibility! !

!WarningProtocol methodsFor: 'exceptionBuilderProtocol'!

messageText: signalerText 
	"Set an exception's message text."

	^self implementorsResponsibility! !

!WarningProtocol methodsFor: 'repertoire'!

defaultAction
	"The default action taken if the exception is signaled."

	^self implementorsResponsibility! !


!instantiatorProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!instantiatorProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility! !


!FloatProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!FloatProtocol methodsFor: 'numberProtocol'!

* operand 
	"Answer the result of multiplying the receiver by operand."

	^self implementorsResponsibility!

+ operand 
	"Answer the result of adding operand to the receiver."

	^self implementorsResponsibility!

- operand 
	"Answer the result of subtracting operand from the receiver."

	^self implementorsResponsibility!

/ operand 
	"Answer the result of dividing the receiver by operand."

	^self implementorsResponsibility!

// operand 
	"Answer the truncated quotient resulting from dividing the receiver by operand. The truncation is towards negative infinity."

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

abs
	"Answer the absolute value of the receiver."

	^self implementorsResponsibility!

asFloat
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatD
	"Answer a d precision floating-point number approximating the receiver ."

	^self implementorsResponsibility!

asFloatE
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatQ
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFraction
	"Answer a fraction approximating the receiver."

	^self implementorsResponsibility!

asInteger
	"Answer an integer approximating the receiver."

	^self implementorsResponsibility!

asScaledDecimal: scale 
	"Answer a scaled decimal number, with a fractional precision of scale, approximating the receiver."

	^self implementorsResponsibility!

ceiling
	"Answer the smallest integer greater than or equal to the receiver."

	^self implementorsResponsibility!

floor
	"Answer the largest integer less than or equal to the receiver."

	^self implementorsResponsibility!

fractionPart
	"Answer the fractional part of the receiver."

	^self implementorsResponsibility!

integerPart
	"Answer the integer part of the receiver."

	^self implementorsResponsibility!

negated
	"Answer the negation of the receiver."

	^self implementorsResponsibility!

negative
	"Answer true if the receiver is negative."

	^self implementorsResponsibility!

positive
	"Answer true if the receiver is positive or zero."

	^self implementorsResponsibility!

quo: operand 
	"Answer the truncated integer quotient resulting from dividing the receiver by operand. Truncation is towards zero."

	^self implementorsResponsibility!

raisedTo: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

raisedToInteger: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

reciprocal
	"Answer the reciprocal of the receiver."

	^self implementorsResponsibility!

rem: operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility!

rounded
	"Answer the integer nearest the receiver."

	^self implementorsResponsibility!

roundTo: factor 
	"Answer the number nearest the receiver that is a multiple of factor."

	^self implementorsResponsibility!

sign
	"Answer the sign of the receiver."

	^self implementorsResponsibility!

sqrt
	"Answer the positive square root of the receiver."

	^self implementorsResponsibility!

squared
	"Answer the receiver squared."

	^self implementorsResponsibility!

strictlyPositive
	"Answer true if the receiver is greater than zero."

	^self implementorsResponsibility!

to: stop 
	"Answer an object conforming to <intervalProtocol> which represents an arithmetic progression from the receiver to stop in increments of 1."

	^self implementorsResponsibility!

to: stop by: step 
	"Answer an interval which represents an arithmetic progression from receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop by: step do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from the receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from receiver to stop in increments of 1."

	^self implementorsResponsibility!

truncated
	"Answer an integer equal to the receiver truncated towards zero."

	^self implementorsResponsibility!

truncateTo: factor 
	"Answer the number nearest the receiver truncated towards zero which is a multiple of factor."

	^self implementorsResponsibility!

\\ operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility! !

!FloatProtocol methodsFor: 'magnitudeProtocol'!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!FloatProtocol methodsFor: 'repertoire'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

arcCos
	"Answer the inverse cosine of the receiver in radians."

	^self implementorsResponsibility!

arcSin
	"Answer the inverse sine of the receiver in radians."

	^self implementorsResponsibility!

arcTan
	"Answer the inverse tangent of the receiver in radians."

	^self implementorsResponsibility!

cos
	"Answer the cosine of the receiver in radians."

	^self implementorsResponsibility!

degreesToRadians
	"Answer the receiver converted from degrees to radians."

	^self implementorsResponsibility!

exp
	"Answer the natural exponential of the receiver. This is the inverse of #ln."

	^self implementorsResponsibility!

floorLog: operand 
	"Answer the largest integer less than or equal to the logarithm to the base operand of the receiver."

	^self implementorsResponsibility!

ln
	"Answer the natural logarithm of the receiver."

	^self implementorsResponsibility!

log: operand 
	"Answer the logarithm to the base operand of the receiver."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

radiansToDegrees
	"Answer the receiver converted from radians to degrees."

	^self implementorsResponsibility!

sin
	"Answer the sine of the receiver."

	^self implementorsResponsibility!

tan
	"Answer the tangent of the receiver."

	^self implementorsResponsibility! !


!ArrayFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility!

with: element1 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 with: element4 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing only the elements of newElements."

	^self implementorsResponsibility! !

!ArrayFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !


!CharacterFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!CharacterFactoryProtocol methodsFor: 'repertoire'!

codePoint: integer 
	"Return a character whose encoding value is integer."

	^self implementorsResponsibility!

cr
	"Return a character representing a carriage-return."

	^self implementorsResponsibility!

lf
	"Return a character representing a line feed."

	^self implementorsResponsibility!

space
	"Return a character representing a space."

	^self implementorsResponsibility!

tab
	"Return a character representing a tab."

	^self implementorsResponsibility! !


!sequencedStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!sequencedStreamProtocol methodsFor: 'repertoire'!

close
	"Disassociate a stream from its backing store."

	^self implementorsResponsibility!

contents
	"Returns a collection containing the complete contents of the stream."

	^self implementorsResponsibility!

isEmpty
	"Returns a Boolean indicating whether there are any sequence values in the receiver."

	^self implementorsResponsibility!

position
	"Returns the current position of the stream."

	^self implementorsResponsibility!

position: amount 
	"Sets the current position in a stream of values."

	^self implementorsResponsibility!

reset
	"Resets the position of the receiver to be at the beginning of the stream of values."

	^self implementorsResponsibility!

setToEnd
	"Set the position of the stream to its end."

	^self implementorsResponsibility! !


!StringProtocol methodsFor: 'sequencedReadableCollectionProtocol'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !

!StringProtocol methodsFor: 'readableStringProtocol'!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

asLowercase
	"Answer a new string which contains all of the elements of the receiver converted to their lower case equivalents."

	^self implementorsResponsibility!

asSymbol
	"Answer a symbol containing the same characters as the receiver."

	^self implementorsResponsibility!

asUppercase
	"Answer a new string which contains all of the elements of the receiver converted to their upper case equivalents."

	^self implementorsResponsibility!

sameAs: operand 
	"Answer true if the receiver collates the same as operand. Answer false otherwise."

	^self implementorsResponsibility!

subStrings: separators 
	"Answer an array containing the substrings in the receiver separated by the elements of separators."

	^self implementorsResponsibility! !

!StringProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!StringProtocol methodsFor: 'magnitudeProtocol'!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!StringProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!StringProtocol methodsFor: 'sequencedCollectionProtocol'!

at: index put: newElement 
	"Replace the element in the receiver at index with newElement. Answer newElement."

	^self implementorsResponsibility!

atAll: indices put: newElement 
	"Replace the elements in the receiver specified by indices with newElement."

	^self implementorsResponsibility!

atAllPut: newElement 
	"Replace all the elements in the receiver with newElement."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements 
	"Replace the elements of the receiver between positions start and stop inclusive, with the elements of replacementElements in their original order. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements startingAt: replacementStart 
	"Replace the elements of the receiver between positions start and stop inclusive with the elements of replacementElements, in their original order, starting at position replacementStart. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop withObject: replacementElement 
	"Replace the elements of the receiver between start and stop inclusive with replacementElement. Answer the receiver."

	^self implementorsResponsibility! !

!StringProtocol methodsFor: 'repertoire'!

asString
	"Answer a string containing the same characters as the receiver."

	^self implementorsResponsibility! !


!magnitudeProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!magnitudeProtocol methodsFor: 'repertoire'!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !


!FractionProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!FractionProtocol methodsFor: 'numberProtocol'!

* operand 
	"Answer the result of multiplying the receiver by operand."

	^self implementorsResponsibility!

+ operand 
	"Answer the result of adding operand to the receiver."

	^self implementorsResponsibility!

- operand 
	"Answer the result of subtracting operand from the receiver."

	^self implementorsResponsibility!

/ operand 
	"Answer the result of dividing the receiver by operand."

	^self implementorsResponsibility!

// operand 
	"Answer the truncated quotient resulting from dividing the receiver by operand. The truncation is towards negative infinity."

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

abs
	"Answer the absolute value of the receiver."

	^self implementorsResponsibility!

asFloat
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatD
	"Answer a d precision floating-point number approximating the receiver ."

	^self implementorsResponsibility!

asFloatE
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatQ
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFraction
	"Answer a fraction approximating the receiver."

	^self implementorsResponsibility!

asInteger
	"Answer an integer approximating the receiver."

	^self implementorsResponsibility!

asScaledDecimal: scale 
	"Answer a scaled decimal number, with a fractional precision of scale, approximating the receiver."

	^self implementorsResponsibility!

ceiling
	"Answer the smallest integer greater than or equal to the receiver."

	^self implementorsResponsibility!

floor
	"Answer the largest integer less than or equal to the receiver."

	^self implementorsResponsibility!

fractionPart
	"Answer the fractional part of the receiver."

	^self implementorsResponsibility!

integerPart
	"Answer the integer part of the receiver."

	^self implementorsResponsibility!

negated
	"Answer the negation of the receiver."

	^self implementorsResponsibility!

negative
	"Answer true if the receiver is negative."

	^self implementorsResponsibility!

positive
	"Answer true if the receiver is positive or zero."

	^self implementorsResponsibility!

quo: operand 
	"Answer the truncated integer quotient resulting from dividing the receiver by operand. Truncation is towards zero."

	^self implementorsResponsibility!

raisedTo: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

raisedToInteger: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

reciprocal
	"Answer the reciprocal of the receiver."

	^self implementorsResponsibility!

rem: operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility!

rounded
	"Answer the integer nearest the receiver."

	^self implementorsResponsibility!

roundTo: factor 
	"Answer the number nearest the receiver that is a multiple of factor."

	^self implementorsResponsibility!

sign
	"Answer the sign of the receiver."

	^self implementorsResponsibility!

sqrt
	"Answer the positive square root of the receiver."

	^self implementorsResponsibility!

squared
	"Answer the receiver squared."

	^self implementorsResponsibility!

strictlyPositive
	"Answer true if the receiver is greater than zero."

	^self implementorsResponsibility!

to: stop 
	"Answer an object conforming to <intervalProtocol> which represents an arithmetic progression from the receiver to stop in increments of 1."

	^self implementorsResponsibility!

to: stop by: step 
	"Answer an interval which represents an arithmetic progression from receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop by: step do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from the receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from receiver to stop in increments of 1."

	^self implementorsResponsibility!

truncated
	"Answer an integer equal to the receiver truncated towards zero."

	^self implementorsResponsibility!

truncateTo: factor 
	"Answer the number nearest the receiver truncated towards zero which is a multiple of factor."

	^self implementorsResponsibility!

\\ operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility! !

!FractionProtocol methodsFor: 'magnitudeProtocol'!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!FractionProtocol methodsFor: 'repertoire'!

denominator
	"Answer the denominator of the receiver."

	^self implementorsResponsibility!

numerator
	"Answer the numerator of the receiver."

	^self implementorsResponsibility!

printString

	^self implementorsResponsibility! !


!extensibleCollectionProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!extensibleCollectionProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!extensibleCollectionProtocol methodsFor: 'repertoire'!

add: newElement 
	"Add newElement to the receiver's elements."

	^self implementorsResponsibility!

addAll: newElements 
	"Add each element of newElements to the receiver's elements."

	^self implementorsResponsibility!

remove: oldElement 
	"Remove the first element of the receiver which is equivalent to oldElement and return the removed element."

	^self implementorsResponsibility!

remove: oldElement ifAbsent: exceptionHandler 
	"Remove the first element of the receiver which is equivalent to oldElement. If it is not found, answer the result of evaluating exceptionHandler."

	^self implementorsResponsibility!

removeAll: oldElements 
	"For each element in oldElements, remove the first element from the receiver which is equivalent to this element."

	^self implementorsResponsibility! !


!BagFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!BagFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility!

with: element1 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 with: element4 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing only the elements of newElements."

	^self implementorsResponsibility! !


!rationalProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!rationalProtocol methodsFor: 'numberProtocol'!

* operand 
	"Answer the result of multiplying the receiver by operand."

	^self implementorsResponsibility!

+ operand 
	"Answer the result of adding operand to the receiver."

	^self implementorsResponsibility!

- operand 
	"Answer the result of subtracting operand from the receiver."

	^self implementorsResponsibility!

/ operand 
	"Answer the result of dividing the receiver by operand."

	^self implementorsResponsibility!

// operand 
	"Answer the truncated quotient resulting from dividing the receiver by operand. The truncation is towards negative infinity."

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

abs
	"Answer the absolute value of the receiver."

	^self implementorsResponsibility!

asFloat
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatD
	"Answer a d precision floating-point number approximating the receiver ."

	^self implementorsResponsibility!

asFloatE
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatQ
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFraction
	"Answer a fraction approximating the receiver."

	^self implementorsResponsibility!

asInteger
	"Answer an integer approximating the receiver."

	^self implementorsResponsibility!

asScaledDecimal: scale 
	"Answer a scaled decimal number, with a fractional precision of scale, approximating the receiver."

	^self implementorsResponsibility!

ceiling
	"Answer the smallest integer greater than or equal to the receiver."

	^self implementorsResponsibility!

floor
	"Answer the largest integer less than or equal to the receiver."

	^self implementorsResponsibility!

fractionPart
	"Answer the fractional part of the receiver."

	^self implementorsResponsibility!

integerPart
	"Answer the integer part of the receiver."

	^self implementorsResponsibility!

negated
	"Answer the negation of the receiver."

	^self implementorsResponsibility!

negative
	"Answer true if the receiver is negative."

	^self implementorsResponsibility!

positive
	"Answer true if the receiver is positive or zero."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

quo: operand 
	"Answer the truncated integer quotient resulting from dividing the receiver by operand. Truncation is towards zero."

	^self implementorsResponsibility!

raisedTo: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

raisedToInteger: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

reciprocal
	"Answer the reciprocal of the receiver."

	^self implementorsResponsibility!

rem: operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility!

rounded
	"Answer the integer nearest the receiver."

	^self implementorsResponsibility!

roundTo: factor 
	"Answer the number nearest the receiver that is a multiple of factor."

	^self implementorsResponsibility!

sign
	"Answer the sign of the receiver."

	^self implementorsResponsibility!

sqrt
	"Answer the positive square root of the receiver."

	^self implementorsResponsibility!

squared
	"Answer the receiver squared."

	^self implementorsResponsibility!

strictlyPositive
	"Answer true if the receiver is greater than zero."

	^self implementorsResponsibility!

to: stop 
	"Answer an object conforming to <intervalProtocol> which represents an arithmetic progression from the receiver to stop in increments of 1."

	^self implementorsResponsibility!

to: stop by: step 
	"Answer an interval which represents an arithmetic progression from receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop by: step do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from the receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from receiver to stop in increments of 1."

	^self implementorsResponsibility!

truncated
	"Answer an integer equal to the receiver truncated towards zero."

	^self implementorsResponsibility!

truncateTo: factor 
	"Answer the number nearest the receiver truncated towards zero which is a multiple of factor."

	^self implementorsResponsibility!

\\ operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility! !

!rationalProtocol methodsFor: 'magnitudeProtocol'!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!rationalProtocol methodsFor: 'repertoire'!

denominator
	"Answer the denominator of the receiver."

	^self implementorsResponsibility!

numerator
	"Answer the numerator of the receiver."

	^self implementorsResponsibility! !


!scaledDecimalProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!scaledDecimalProtocol methodsFor: 'numberProtocol'!

* operand 
	"Answer the result of multiplying the receiver by operand."

	^self implementorsResponsibility!

+ operand 
	"Answer the result of adding operand to the receiver."

	^self implementorsResponsibility!

- operand 
	"Answer the result of subtracting operand from the receiver."

	^self implementorsResponsibility!

/ operand 
	"Answer the result of dividing the receiver by operand."

	^self implementorsResponsibility!

// operand 
	"Answer the truncated quotient resulting from dividing the receiver by operand. The truncation is towards negative infinity."

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

abs
	"Answer the absolute value of the receiver."

	^self implementorsResponsibility!

asFloat
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatD
	"Answer a d precision floating-point number approximating the receiver ."

	^self implementorsResponsibility!

asFloatE
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatQ
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFraction
	"Answer a fraction approximating the receiver."

	^self implementorsResponsibility!

asInteger
	"Answer an integer approximating the receiver."

	^self implementorsResponsibility!

asScaledDecimal: scale 
	"Answer a scaled decimal number, with a fractional precision of scale, approximating the receiver."

	^self implementorsResponsibility!

ceiling
	"Answer the smallest integer greater than or equal to the receiver."

	^self implementorsResponsibility!

floor
	"Answer the largest integer less than or equal to the receiver."

	^self implementorsResponsibility!

fractionPart
	"Answer the fractional part of the receiver."

	^self implementorsResponsibility!

integerPart
	"Answer the integer part of the receiver."

	^self implementorsResponsibility!

negated
	"Answer the negation of the receiver."

	^self implementorsResponsibility!

negative
	"Answer true if the receiver is negative."

	^self implementorsResponsibility!

positive
	"Answer true if the receiver is positive or zero."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

quo: operand 
	"Answer the truncated integer quotient resulting from dividing the receiver by operand. Truncation is towards zero."

	^self implementorsResponsibility!

raisedTo: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

raisedToInteger: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

reciprocal
	"Answer the reciprocal of the receiver."

	^self implementorsResponsibility!

rem: operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility!

rounded
	"Answer the integer nearest the receiver."

	^self implementorsResponsibility!

roundTo: factor 
	"Answer the number nearest the receiver that is a multiple of factor."

	^self implementorsResponsibility!

sign
	"Answer the sign of the receiver."

	^self implementorsResponsibility!

sqrt
	"Answer the positive square root of the receiver."

	^self implementorsResponsibility!

squared
	"Answer the receiver squared."

	^self implementorsResponsibility!

strictlyPositive
	"Answer true if the receiver is greater than zero."

	^self implementorsResponsibility!

to: stop 
	"Answer an object conforming to <intervalProtocol> which represents an arithmetic progression from the receiver to stop in increments of 1."

	^self implementorsResponsibility!

to: stop by: step 
	"Answer an interval which represents an arithmetic progression from receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop by: step do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from the receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from receiver to stop in increments of 1."

	^self implementorsResponsibility!

truncated
	"Answer an integer equal to the receiver truncated towards zero."

	^self implementorsResponsibility!

truncateTo: factor 
	"Answer the number nearest the receiver truncated towards zero which is a multiple of factor."

	^self implementorsResponsibility!

\\ operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility! !

!scaledDecimalProtocol methodsFor: 'magnitudeProtocol'!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!scaledDecimalProtocol methodsFor: 'repertoire'!

scale
	"Answer a integer which represents the total number of digits used to represent the fraction part of the receiver, including trailing zeroes."

	^self implementorsResponsibility! !


!SetProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!SetProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!SetProtocol methodsFor: 'extensibleCollectionProtocol'!

remove: oldElement 
	"Remove the first element of the receiver which is equivalent to oldElement and return the removed element."

	^self implementorsResponsibility!

remove: oldElement ifAbsent: exceptionHandler 
	"Remove the first element of the receiver which is equivalent to oldElement. If it is not found, answer the result of evaluating exceptionHandler."

	^self implementorsResponsibility!

removeAll: oldElements 
	"For each element in oldElements, remove the first element from the receiver which is equivalent to this element."

	^self implementorsResponsibility! !

!SetProtocol methodsFor: 'repertoire'!

add: newElement 
	"Add newElement to the receiver's elements."

	^self implementorsResponsibility!

addAll: newElements 
	"Add each element of newElements to the receiver's elements."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility! !


!DurationProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!DurationProtocol methodsFor: 'magnitudeProtocol'!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!DurationProtocol methodsFor: 'repertoire'!

* operand 
	"Answer the result of multiplying the receiver by operand."

	^self implementorsResponsibility!

+ operand 
	"Answer the result of adding operand to the receiver. "

	^self implementorsResponsibility!

- operand 
	"Answer the result of subtracting the operand from the receiver."

	^self implementorsResponsibility!

/ operand 
	"Answer the result of dividing the receiver by operand. "

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

abs
	"Answer the absolute value of the receiver. "

	^self implementorsResponsibility!

asSeconds
	"Answer the total number of seconds in the length of time represented by the receiver. "

	^self implementorsResponsibility!

days
	"Answer the number of complete days in the receiver. "

	^self implementorsResponsibility!

hours
	"Answer the number of complete hours in the receiver. "

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

seconds
	"Answer the number of seconds in the receiver. "

	^self implementorsResponsibility! !


!exceptionInstantiatorProtocol methodsFor: 'exceptionSelectorProtocol'!

, anotherException 
	"Create an exception set."

	^self implementorsResponsibility!

handles: exception 
	"Determine whether an exception handler will accept a signaled exception."

	^self implementorsResponsibility! !

!exceptionInstantiatorProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!exceptionInstantiatorProtocol methodsFor: 'exceptionSignalerProtocol'!

signal: signalerText 
	"Signal the occurrence of an exceptional condition with a specified textual description."

	^self implementorsResponsibility! !

!exceptionInstantiatorProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

signal
	"Signal the occurrence on an exceptional condition."

	^self implementorsResponsibility! !


!FileStreamFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!FileStreamFactoryProtocol methodsFor: 'repertoire'!

read: fileId 
	"Returns a read file stream that reads text from the file with the given name."

	^self implementorsResponsibility!

read: fileId type: fileType 
	"Returns a read file stream that reads from the file with the given name."

	^self implementorsResponsibility!

write: fileId 
	"Returns a write file stream that writes text to the file with the given name."

	^self implementorsResponsibility!

write: fileId mode: mode 
	"Returns a write file stream that writes text to the file with the given name."

	^self implementorsResponsibility!

write: fileId mode: mode check: check type: fileType 
	"Returns a write file stream that writes to the file with the given name."

	^self implementorsResponsibility! !


!ObjectClassProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ObjectClassProtocol methodsFor: 'classDescriptionProtocol'!

allSubclasses
	"Return all subclasses of a class."

	^self implementorsResponsibility!

allSuperclasses
	"Return all superclasses of a class."

	^self implementorsResponsibility!

name
	"Return the name of a class."

	^self implementorsResponsibility!

subclasses
	"Return direct subclasses of a class."

	^self implementorsResponsibility!

superclass
	"Return the immediate superclass of a class."

	^self implementorsResponsibility! !

!ObjectClassProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility! !


!readFileStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!readFileStreamProtocol methodsFor: 'FileStreamProtocol'!

contents
	"Returns a collection containing the complete contents of the stream."

	^self implementorsResponsibility!

externalType
	"Returns a symbol that identifies the external stream type of the receiver."

	^self implementorsResponsibility!

isBinary
	"Answer wthether the receiver's data is binary."

	^self implementorsResponsibility!

isText
	"Answer whether the receiver's data is characters."

	^self implementorsResponsibility! !

!readFileStreamProtocol methodsFor: 'sequencedStreamProtocol'!

close
	"Disassociate a stream from its backing store."

	^self implementorsResponsibility!

isEmpty
	"Returns a Boolean indicating whether there are any sequence values in the receiver."

	^self implementorsResponsibility!

position
	"Returns the current position of the stream."

	^self implementorsResponsibility!

position: amount 
	"Sets the current position in a stream of values."

	^self implementorsResponsibility!

reset
	"Resets the position of the receiver to be at the beginning of the stream of values."

	^self implementorsResponsibility!

setToEnd
	"Set the position of the stream to its end."

	^self implementorsResponsibility! !

!readFileStreamProtocol methodsFor: 'gettableStreamProtocol'!

atEnd
	"Returns a Boolean indicating whether the receiver is at the end of its values."

	^self implementorsResponsibility!

do: operation 
	"Evaluates the argument with each receiver future sequence value, terminating evaluation when there are no more future sequence values."

	^self implementorsResponsibility!

next
	"Return the next object in the receiver."

	^self implementorsResponsibility!

nextLine
	"Reads the next line from the stream."

	^self implementorsResponsibility!

nextMatchFor: anObject 
	"Reads the next object from the stream and returns true if the object is equivalent to the argument and false if not."

	^self implementorsResponsibility!

peek
	"Returns the next object in the receiver's future sequence values without advancing the receiver's position. Returns nil if the receiver is at end of stream."

	^self implementorsResponsibility!

peekFor: anObject 
	"Peeks at the next object in the stream and returns true if it matches the argument, and false if not."

	^self implementorsResponsibility!

skip: amount 
	"Skips the next amount objects in the receiver's future sequence values."

	^self implementorsResponsibility!

skipTo: anObject 
	"Sets the stream to read the object just after the next occurrence of the argument and returns true. If the argument is not found before the end of the stream is encountered, false is returned."

	^self implementorsResponsibility! !

!readFileStreamProtocol methodsFor: 'repertoire'!

next: amount 
	"Returns a collection of the next amount objects in the stream."

	^self implementorsResponsibility!

upTo: anObject 
	"Returns a collection of all of the objects in the receiver up to, but not including, the next occurrence of the argument. Sets the stream to read the object just after the next occurrence of the argument. If the argument is not found and the end of the stream is encountered, an ordered collection of the objects read is returned."

	^self implementorsResponsibility! !


!DateAndTimeFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!DateAndTimeFactoryProtocol methodsFor: 'repertoire'!

clockPrecision
	"Answer a <Duration> such that after that period of time passes, #now is guaranteed to give a different result. "

	^self implementorsResponsibility!

now
	"Answer a <DateAndTime> representing the current date and time. "

	^self implementorsResponsibility!

year: year day: dayOfYear hour: hour minute: minute second: second 
	"Answer a <DateAndTime> which is the second second of the minute minute of the hour hour of the day dayOfYear of the year year of the astronomical Gregorian calendar in local time. "

	^self implementorsResponsibility!

year: year day: dayOfYear hour: hour minute: minute second: second offset: offset 
	"Answer a <DateAndTime> which is the second second of the minute minute of the hour hour of the day dayOfYear of the year year of the astronomical Gregorian calendar offset from UTC by offset. "

	^self implementorsResponsibility!

year: year month: month day: dayOfMonth hour: hour minute: minute second: second 
	"Answer a <DateAndTime> which is the second second of the minute minute of the hour hour of the day dayOfMonth of the month month of the year year of the astronomical Gregorian calendar in local time. "

	^self implementorsResponsibility!

year: year month: month day: dayOfMonth hour: hour minute: minute second: second offset: offset 
	"Answer a <DateAndTime> which is the second second of the minute minute of the hour hour of the day dayOfMonth of the month month of the year year of the astronomical Gregorian calendar offset from UTC by offset. "

	^self implementorsResponsibility! !


!ErrorClassProtocol methodsFor: 'classDescriptionProtocol'!

allSubclasses
	"Return all subclasses of a class."

	^self implementorsResponsibility!

allSuperclasses
	"Return all superclasses of a class."

	^self implementorsResponsibility!

name
	"Return the name of a class."

	^self implementorsResponsibility!

subclasses
	"Return direct subclasses of a class."

	^self implementorsResponsibility!

superclass
	"Return the immediate superclass of a class."

	^self implementorsResponsibility! !

!ErrorClassProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ErrorClassProtocol methodsFor: 'exceptionSignalerProtocol'!

signal: signalerText 
	"Signal the occurrence of an exceptional condition with a specified textual description."

	^self implementorsResponsibility! !

!ErrorClassProtocol methodsFor: 'ExceptionClassProtocol'!

handles: exception 

	^self implementorsResponsibility!

signal

	^self implementorsResponsibility! !

!ErrorClassProtocol methodsFor: 'exceptionSelectorProtocol'!

, anotherException 
	"Create an exception set."

	^self implementorsResponsibility! !

!ErrorClassProtocol methodsFor: 'repertoire'!

new

	^self implementorsResponsibility! !


!IntervalProtocol methodsFor: 'sequencedReadableCollectionProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !

!IntervalProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!IntervalProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!IntervalProtocol methodsFor: 'repertoire'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which excludes the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility! !


!niladicValuableProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!niladicValuableProtocol methodsFor: 'valuableProtocol'!

valueWithArguments: argumentArray 
	"Answers the value of the receiver when applied to the arguments in argumentArray."

	^self implementorsResponsibility! !

!niladicValuableProtocol methodsFor: 'repertoire'!

argumentCount
	"Answers the number of arguments needed to evaluate the receiver."

	^self implementorsResponsibility!

value
	"Answers the value of the receiver."

	^self implementorsResponsibility!

whileFalse
	"Evaluates the receiver until it evaluates to true."

	^self implementorsResponsibility!

whileFalse: iterationBlock 
	"Evaluates iterationBlock zero or more times until the receiver evaluates to true."

	^self implementorsResponsibility!

whileTrue
	"Evaluates the receiver until it evaluates to false."

	^self implementorsResponsibility!

whileTrue: iterationBlock 
	"Evaluates iterationBlock zero or more times until the receiver evaluates to false."

	^self implementorsResponsibility! !


!writeFileStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!writeFileStreamProtocol methodsFor: 'FileStreamProtocol'!

contents
	"Returns a collection containing the complete contents of the stream."

	^self implementorsResponsibility!

externalType
	"Returns a symbol that identifies the external stream type of the receiver."

	^self implementorsResponsibility!

isBinary
	"Answer wthether the receiver's data is binary."

	^self implementorsResponsibility!

isText
	"Answer whether the receiver's data is characters."

	^self implementorsResponsibility! !

!writeFileStreamProtocol methodsFor: 'sequencedStreamProtocol'!

close
	"Disassociate a stream from its backing store."

	^self implementorsResponsibility!

isEmpty
	"Returns a Boolean indicating whether there are any sequence values in the receiver."

	^self implementorsResponsibility!

position
	"Returns the current position of the stream."

	^self implementorsResponsibility!

position: amount 
	"Sets the current position in a stream of values."

	^self implementorsResponsibility!

reset
	"Resets the position of the receiver to be at the beginning of the stream of values."

	^self implementorsResponsibility!

setToEnd
	"Set the position of the stream to its end."

	^self implementorsResponsibility! !

!writeFileStreamProtocol methodsFor: 'puttableStreamProtocol'!

cr
	"Writes an end-of-line sequence to the receiver."

	^self implementorsResponsibility!

flush
	"Update a stream's backing store."

	^self implementorsResponsibility!

nextPut: anObject 
	"Writes the argument to the stream."

	^self implementorsResponsibility!

nextPutAll: aCollection 
	"Enumerates the argument, adding each element to the receiver"

	^self implementorsResponsibility!

space
	"Writes a space character to the receiver."

	^self implementorsResponsibility!

tab
	"Writes a tab character to the receiver."

	^self implementorsResponsibility! !


!OrderedCollectionFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!OrderedCollectionFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility!

with: element1 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 with: element4 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing only the elements of newElements."

	^self implementorsResponsibility! !


!ArrayProtocol methodsFor: 'sequencedReadableCollectionProtocol'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !

!ArrayProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ArrayProtocol methodsFor: 'sequencedCollectionProtocol'!

at: index put: newElement 
	"Replace the element in the receiver at index with newElement. Answer newElement."

	^self implementorsResponsibility!

atAll: indices put: newElement 
	"Replace the elements in the receiver specified by indices with newElement."

	^self implementorsResponsibility!

atAllPut: newElement 
	"Replace all the elements in the receiver with newElement."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements 
	"Replace the elements of the receiver between positions start and stop inclusive, with the elements of replacementElements in their original order. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements startingAt: replacementStart 
	"Replace the elements of the receiver between positions start and stop inclusive with the elements of replacementElements, in their original order, starting at position replacementStart. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop withObject: replacementElement 
	"Replace the elements of the receiver between start and stop inclusive with replacementElement. Answer the receiver."

	^self implementorsResponsibility! !

!ArrayProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !


!CharacterProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!CharacterProtocol methodsFor: 'repertoire'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

asLowercase
	"Return a character which is equivalent to the lowercase representation of the receiver."

	^self implementorsResponsibility!

asString
	"Return a new string whose sole element is equivalent to the receiver."

	^self implementorsResponsibility!

asUppercase
	"Return a character equivalent to the uppercase representation of the receiver."

	^self implementorsResponsibility!

codePoint
	"Return the encoding value of the receiver."

	^self implementorsResponsibility!

isAlphaNumeric
	"Test whether the receiver is a letter or digit."

	^self implementorsResponsibility!

isDigit
	"Test whether the receiver is a digit."

	^self implementorsResponsibility!

isLetter
	"Test whether the receiver is a letter."

	^self implementorsResponsibility!

isLowercase
	"Test whether the receiver is a lowercase letter."

	^self implementorsResponsibility!

isUppercase
	"Test whether the receiver is an uppercase letter."

	^self implementorsResponsibility! !


!integerProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!integerProtocol methodsFor: 'numberProtocol'!

* operand 
	"Answer the result of multiplying the receiver by operand."

	^self implementorsResponsibility!

+ operand 
	"Answer the result of adding operand to the receiver."

	^self implementorsResponsibility!

- operand 
	"Answer the result of subtracting operand from the receiver."

	^self implementorsResponsibility!

/ operand 
	"Answer the result of dividing the receiver by operand."

	^self implementorsResponsibility!

// operand 
	"Answer the truncated quotient resulting from dividing the receiver by operand. The truncation is towards negative infinity."

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

abs
	"Answer the absolute value of the receiver."

	^self implementorsResponsibility!

asFloat
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatD
	"Answer a d precision floating-point number approximating the receiver ."

	^self implementorsResponsibility!

asFloatE
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFloatQ
	"Answer a floating-point number approximating the receiver."

	^self implementorsResponsibility!

asFraction
	"Answer a fraction approximating the receiver."

	^self implementorsResponsibility!

asInteger
	"Answer an integer approximating the receiver."

	^self implementorsResponsibility!

ceiling
	"Answer the smallest integer greater than or equal to the receiver."

	^self implementorsResponsibility!

floor
	"Answer the largest integer less than or equal to the receiver."

	^self implementorsResponsibility!

fractionPart
	"Answer the fractional part of the receiver."

	^self implementorsResponsibility!

integerPart
	"Answer the integer part of the receiver."

	^self implementorsResponsibility!

negated
	"Answer the negation of the receiver."

	^self implementorsResponsibility!

negative
	"Answer true if the receiver is negative."

	^self implementorsResponsibility!

positive
	"Answer true if the receiver is positive or zero."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

quo: operand 
	"Answer the truncated integer quotient resulting from dividing the receiver by operand. Truncation is towards zero."

	^self implementorsResponsibility!

raisedTo: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

raisedToInteger: operand 
	"Answer the receiver raised to the power operand."

	^self implementorsResponsibility!

reciprocal
	"Answer the reciprocal of the receiver."

	^self implementorsResponsibility!

rem: operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility!

rounded
	"Answer the integer nearest the receiver."

	^self implementorsResponsibility!

roundTo: factor 
	"Answer the number nearest the receiver that is a multiple of factor."

	^self implementorsResponsibility!

sign
	"Answer the sign of the receiver."

	^self implementorsResponsibility!

sqrt
	"Answer the positive square root of the receiver."

	^self implementorsResponsibility!

squared
	"Answer the receiver squared."

	^self implementorsResponsibility!

strictlyPositive
	"Answer true if the receiver is greater than zero."

	^self implementorsResponsibility!

to: stop 
	"Answer an object conforming to <intervalProtocol> which represents an arithmetic progression from the receiver to stop in increments of 1."

	^self implementorsResponsibility!

to: stop by: step 
	"Answer an interval which represents an arithmetic progression from receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop by: step do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from the receiver to stop in increments of step."

	^self implementorsResponsibility!

to: stop do: operation 
	"Evaluate operation for each element of an interval which represents an arithmetic progression from receiver to stop in increments of 1."

	^self implementorsResponsibility!

truncated
	"Answer an integer equal to the receiver truncated towards zero."

	^self implementorsResponsibility!

truncateTo: factor 
	"Answer the number nearest the receiver truncated towards zero which is a multiple of factor."

	^self implementorsResponsibility!

\\ operand 
	"Answer the remainder after integer division of the receiver by the operand."

	^self implementorsResponsibility! !

!integerProtocol methodsFor: 'rationalProtocol'!

denominator
	"Answer the denominator of the receiver."

	^self implementorsResponsibility!

numerator
	"Answer the numerator of the receiver."

	^self implementorsResponsibility! !

!integerProtocol methodsFor: 'magnitudeProtocol'!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!integerProtocol methodsFor: 'repertoire'!

allMask: mask 
	"Answer true if all of the bits that are 1 in the binary representation of mask are 1 in the binary representation of the receiver. Answer false otherwise."

	^self implementorsResponsibility!

anyMask: mask 
	"Answer true if any of the bits that are 1 in the binary representation of mask are 1 in the binary representation of the receiver. Answer false otherwise."

	^self implementorsResponsibility!

asScaledDecimal: scale 
	"Answer a scaled decimal number, with a fractional precision of scale, approximating the receiver."

	^self implementorsResponsibility!

bitAnd: operand 
	"Answer the bit-wise logical and of the receiver and the operand."

	^self implementorsResponsibility!

bitAt: index 
	"Answer the value of the bit at index in the binary representation of the receiver."

	^self implementorsResponsibility!

bitAt: index put: value 
	"Set the value of the bit at index in the binary representation of the receiver."

	^self implementorsResponsibility!

bitOr: operand 
	"Answer the logical or of the receiver and operand."

	^self implementorsResponsibility!

bitShift: shift 
	"Answer the result of logically bit-wise shifting the binary representation of the receiver by shift bits."

	^self implementorsResponsibility!

bitXor: operand 
	"Answer bit-wise exclusive or of the receiver and the operand."

	^self implementorsResponsibility!

even
	"Answer true if the receiver is even."

	^self implementorsResponsibility!

factorial
	"Answer the factorial of the receiver."

	^self implementorsResponsibility!

gcd: operand 
	"Answer the greatest common divisor of the receiver and operand."

	^self implementorsResponsibility!

highBit
	"Answer the index of the most significant non-zero bit in the binary representation of the receiver."

	^self implementorsResponsibility!

lcm: operand 
	"Answer the least common multiple of the receiver and operand."

	^self implementorsResponsibility!

noMask: mask 
	"Answer true if none of the bits that are 1 in the binary representation of mask are 1 in the binary representation of the receiver. Answer false otherwise."

	^self implementorsResponsibility!

odd
	"Answer true if the receiver is odd."

	^self implementorsResponsibility!

printOn: output base: base showRadix: flag 
	"Write a sequence of characters that describes the receiver in radix base with optional radix specifier."

	^self implementorsResponsibility!

printStringRadix: base 
	"Answer a string which represents the receiver in radix base."

	^self implementorsResponsibility! !


!DictionaryFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!DictionaryFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing all the elements of newElements."

	^self implementorsResponsibility! !


!abstractDictionaryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!abstractDictionaryProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!abstractDictionaryProtocol methodsFor: 'repertoire'!

addAll: dictionary 
	"Store the elements of dictionary in the receiver at the corresponding keys from dictionary."

	^self implementorsResponsibility!

at: key 
	"Answer the element at key in the receiver."

	^self implementorsResponsibility!

at: key ifAbsent: operation 
	"Answer the element at key in the receiver. If key lookup for key fails, then answer the result of evaluating operation."

	^self implementorsResponsibility!

at: key ifAbsentPut: operation 
	"Answer the element at key in the receiver. If key lookup for key fails, then store and return the result of evaluating operation."

	^self implementorsResponsibility!

at: key put: newElement 
	"Store newElement at key in the receiver. Answer newElement."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

includesKey: key 
	"Answer true if the receiver contains an element stored at key. Answer false otherwise."

	^self implementorsResponsibility!

keyAtValue: value 
	"Answer a key such that the element stored at this key is equal to value. Answer nil if no such key is found."

	^self implementorsResponsibility!

keyAtValue: value ifAbsent: operation 
	"Answer a key such that the element stored at this key is equivalent to value. Answer the result of evaluating operation if no such key is found."

	^self implementorsResponsibility!

keys
	"Answer a collection of keys at which there is an element stored in the receiver."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Iteratively evaluate operation with each of the receiver's keys and values."

	^self implementorsResponsibility!

keysDo: operation 
	"Iteratively evaluate operation with each of the receiver's keys at which there are elements stored."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which excludes the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

removeAllKeys: keys 
	"Remove any elements from the receiver which are stored at the keys specified in keys."

	^self implementorsResponsibility!

removeAllKeys: keys ifAbsent: operation 
	"Remove any elements from the receiver which are stored at the keys specified in keys. For any element in keys which is not a valid key of the receiver, evaluate operation with that element as the argument, but do not stop the enumeration."

	^self implementorsResponsibility!

removeKey: key 
	"Remove the element which is stored at key in the receiver. Answer the removed element."

	^self implementorsResponsibility!

removeKey: key ifAbsent: operation 
	"Remove the element which is stored at key in the receiver and answer the removed element. Answer the result of evaluating operation if no such key is found in the receiver."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

values
	"Answer a collection of the receiver's elements."

	^self implementorsResponsibility! !


!symbolProtocol methodsFor: 'sequencedReadableCollectionProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !

!symbolProtocol methodsFor: 'readableStringProtocol'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

asLowercase
	"Answer a new string which contains all of the elements of the receiver converted to their lower case equivalents."

	^self implementorsResponsibility!

asUppercase
	"Answer a new string which contains all of the elements of the receiver converted to their upper case equivalents."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

sameAs: operand 
	"Answer true if the receiver collates the same as operand. Answer false otherwise."

	^self implementorsResponsibility!

subStrings: separators 
	"Answer an array containing the substrings in the receiver separated by the elements of separators."

	^self implementorsResponsibility! !

!symbolProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!symbolProtocol methodsFor: 'magnitudeProtocol'!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!symbolProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!symbolProtocol methodsFor: 'repertoire'!

asString
	"Answer a string containing the same characters as the receiver."

	^self implementorsResponsibility!

asSymbol
	"Answer a symbol containing the same characters as the receiver."

	^self implementorsResponsibility! !


!SetFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!SetFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility!

with: element1 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 with: element4 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing only the elements of newElements."

	^self implementorsResponsibility! !


!MessageNotUnderstoodProtocol methodsFor: 'signaledExceptionProtocol'!

isNested
	"Determine whether the current exception handler is within the scope of another handler for the same exception."

	^self implementorsResponsibility!

outer
	"Evaluate the enclosing exception action for the receiver and return."

	^self implementorsResponsibility!

pass
	"Yield control to the enclosing exception action for the receiver."

	^self implementorsResponsibility!

resignalAs: replacementException 
	"Signal an alternative exception in place of the receiver."

	^self implementorsResponsibility!

resume
	"Return from the message that signaled the receiver."

	^self implementorsResponsibility!

resume: resumptionValue 
	"Return the argument as the value of the message that signaled the receiver."

	^self implementorsResponsibility!

retry
	"Abort an exception handler and re-evaluate its protected block."

	^self implementorsResponsibility!

retryUsing: alternativeBlock 
	"Abort an exception handler and evaluate a new block in place of the handler's protected block."

	^self implementorsResponsibility!

return
	"Return nil as the value of the block protected by the active exception handler."

	^self implementorsResponsibility!

return: returnValue 
	"Return the argument as the value of the block protected by the active exception handler."

	^self implementorsResponsibility! !

!MessageNotUnderstoodProtocol methodsFor: 'ErrorProtocol'!

defaultAction

	^self implementorsResponsibility! !

!MessageNotUnderstoodProtocol methodsFor: 'exceptionBuilderProtocol'!

messageText: signalerText 
	"Set an exception's message text."

	^self implementorsResponsibility! !

!MessageNotUnderstoodProtocol methodsFor: 'repertoire'!

isResumable
	"Determine whether an exception is resumable."

	^self implementorsResponsibility!

message
	"Answer the selector and arguments of the message that failed."

	^self implementorsResponsibility!

receiver
	"Answer the receiver  the message that failed."

	^self implementorsResponsibility! !


!WriteStreamProtocol methodsFor: 'puttableStreamProtocol'!

cr
	"Writes an end-of-line sequence to the receiver."

	^self implementorsResponsibility!

flush
	"Update a stream's backing store."

	^self implementorsResponsibility!

nextPut: anObject 
	"Writes the argument to the stream."

	^self implementorsResponsibility!

nextPutAll: aCollection 
	"Enumerates the argument, adding each element to the receiver"

	^self implementorsResponsibility!

space
	"Writes a space character to the receiver."

	^self implementorsResponsibility!

tab
	"Writes a tab character to the receiver."

	^self implementorsResponsibility! !

!WriteStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!WriteStreamProtocol methodsFor: 'sequencedStreamProtocol'!

close
	"Disassociate a stream from its backing store."

	^self implementorsResponsibility!

isEmpty
	"Returns a Boolean indicating whether there are any sequence values in the receiver."

	^self implementorsResponsibility!

position
	"Returns the current position of the stream."

	^self implementorsResponsibility!

position: amount 
	"Sets the current position in a stream of values."

	^self implementorsResponsibility!

reset
	"Resets the position of the receiver to be at the beginning of the stream of values."

	^self implementorsResponsibility!

setToEnd
	"Set the position of the stream to its end."

	^self implementorsResponsibility! !

!WriteStreamProtocol methodsFor: 'collectionStreamProtocol'!

contents
	"Returns a collection containing the complete contents of the stream."

	^self implementorsResponsibility! !


!failedMessageProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!failedMessageProtocol methodsFor: 'repertoire'!

arguments
	"Answer the arguments of the message that could not be sent."

	^self implementorsResponsibility!

selector
	"Answer the selector of the message that could not be sent."

	^self implementorsResponsibility! !


!gettableStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!gettableStreamProtocol methodsFor: 'repertoire'!

atEnd
	"Returns a Boolean indicating whether the receiver is at the end of its values."

	^self implementorsResponsibility!

do: operation 
	"Evaluates the argument with each receiver future sequence value, terminating evaluation when there are no more future sequence values."

	^self implementorsResponsibility!

next
	"Return the next object in the receiver."

	^self implementorsResponsibility!

next: amount 
	"Returns a collection of the next amount objects in the stream."

	^self implementorsResponsibility!

nextLine
	"Reads the next line from the stream."

	^self implementorsResponsibility!

nextMatchFor: anObject 
	"Reads the next object from the stream and returns true if the object is equivalent to the argument and false if not."

	^self implementorsResponsibility!

peek
	"Returns the next object in the receiver's future sequence values without advancing the receiver's position. Returns nil if the receiver is at end of stream."

	^self implementorsResponsibility!

peekFor: anObject 
	"Peeks at the next object in the stream and returns true if it matches the argument, and false if not."

	^self implementorsResponsibility!

skip: amount 
	"Skips the next amount objects in the receiver's future sequence values."

	^self implementorsResponsibility!

skipTo: anObject 
	"Sets the stream to read the object just after the next occurrence of the argument and returns true. If the argument is not found before the end of the stream is encountered, false is returned."

	^self implementorsResponsibility!

upTo: anObject 
	"Returns a collection of all of the objects in the receiver up to, but not including, the next occurrence of the argument. Sets the stream to read the object just after the next occurrence of the argument. If the argument is not found and the end of the stream is encountered, an ordered collection of the objects read is returned."

	^self implementorsResponsibility! !


!BagProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!BagProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!BagProtocol methodsFor: 'extensibleCollectionProtocol'!

remove: oldElement 
	"Remove the first element of the receiver which is equivalent to oldElement and return the removed element."

	^self implementorsResponsibility!

remove: oldElement ifAbsent: exceptionHandler 
	"Remove the first element of the receiver which is equivalent to oldElement. If it is not found, answer the result of evaluating exceptionHandler."

	^self implementorsResponsibility!

removeAll: oldElements 
	"For each element in oldElements, remove the first element from the receiver which is equivalent to this element."

	^self implementorsResponsibility! !

!BagProtocol methodsFor: 'repertoire'!

add: newElement 
	"Add newElement to the receiver's elements."

	^self implementorsResponsibility!

add: newElement withOccurrences: count 
	"Add newElement count times to the receiver's elements."

	^self implementorsResponsibility!

addAll: newElements 
	"Add each element of newElements to the receiver's elements."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility! !


!selectorProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !


!signaledExceptionProtocol methodsFor: 'repertoire'!

isNested
	"Determine whether the current exception handler is within the scope of another handler for the same exception."

	^self implementorsResponsibility!

outer
	"Evaluate the enclosing exception action for the receiver and return."

	^self implementorsResponsibility!

pass
	"Yield control to the enclosing exception action for the receiver."

	^self implementorsResponsibility!

resignalAs: replacementException 
	"Signal an alternative exception in place of the receiver."

	^self implementorsResponsibility!

resume
	"Return from the message that signaled the receiver."

	^self implementorsResponsibility!

resume: resumptionValue 
	"Return the argument as the value of the message that signaled the receiver."

	^self implementorsResponsibility!

retry
	"Abort an exception handler and re-evaluate its protected block."

	^self implementorsResponsibility!

retryUsing: alternativeBlock 
	"Abort an exception handler and evaluate a new block in place of the handler's protected block."

	^self implementorsResponsibility!

return
	"Return nil as the value of the block protected by the active exception handler."

	^self implementorsResponsibility!

return: returnValue 
	"Return the argument as the value of the block protected by the active exception handler."

	^self implementorsResponsibility! !


!valuableProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!valuableProtocol methodsFor: 'repertoire'!

argumentCount
	"Answers the number of arguments needed to evaluate the receiver."

	^self implementorsResponsibility!

valueWithArguments: argumentArray 
	"Answers the value of the receiver when applied to the arguments in argumentArray."

	^self implementorsResponsibility! !


!ExceptionProtocol methodsFor: 'exceptionBuilderProtocol'!

messageText: signalerText 
	"Set an exception's message text."

	^self implementorsResponsibility! !

!ExceptionProtocol methodsFor: 'signaledExceptionProtocol'!

isNested
	"Determine whether the current exception handler is within the scope of another handler for the same exception."

	^self implementorsResponsibility!

outer
	"Evaluate the enclosing exception action for the receiver and return."

	^self implementorsResponsibility!

pass
	"Yield control to the enclosing exception action for the receiver."

	^self implementorsResponsibility!

resignalAs: replacementException 
	"Signal an alternative exception in place of the receiver."

	^self implementorsResponsibility!

resume
	"Return from the message that signaled the receiver."

	^self implementorsResponsibility!

resume: resumptionValue 
	"Return the argument as the value of the message that signaled the receiver."

	^self implementorsResponsibility!

retry
	"Abort an exception handler and re-evaluate its protected block."

	^self implementorsResponsibility!

retryUsing: alternativeBlock 
	"Abort an exception handler and evaluate a new block in place of the handler's protected block."

	^self implementorsResponsibility!

return
	"Return nil as the value of the block protected by the active exception handler."

	^self implementorsResponsibility!

return: returnValue 
	"Return the argument as the value of the block protected by the active exception handler."

	^self implementorsResponsibility! !


!FileStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!FileStreamProtocol methodsFor: 'sequencedStreamProtocol'!

close
	"Disassociate a stream from its backing store."

	^self implementorsResponsibility!

isEmpty
	"Returns a Boolean indicating whether there are any sequence values in the receiver."

	^self implementorsResponsibility!

position
	"Returns the current position of the stream."

	^self implementorsResponsibility!

position: amount 
	"Sets the current position in a stream of values."

	^self implementorsResponsibility!

reset
	"Resets the position of the receiver to be at the beginning of the stream of values."

	^self implementorsResponsibility!

setToEnd
	"Set the position of the stream to its end."

	^self implementorsResponsibility! !

!FileStreamProtocol methodsFor: 'repertoire'!

contents
	"Returns a collection containing the complete contents of the stream."

	^self implementorsResponsibility!

externalType
	"Returns a symbol that identifies the external stream type of the receiver."

	^self implementorsResponsibility!

isBinary
	"Answer wthether the receiver's data is binary."

	^self implementorsResponsibility!

isText
	"Answer whether the receiver's data is characters."

	^self implementorsResponsibility! !


!IdentityDictionaryProtocol methodsFor: 'abstractDictionaryProtocol'!

addAll: dictionary 
	"Store the elements of dictionary in the receiver at the corresponding keys from dictionary."

	^self implementorsResponsibility!

at: key 
	"Answer the element at key in the receiver."

	^self implementorsResponsibility!

at: key ifAbsent: operation 
	"Answer the element at key in the receiver. If key lookup for key fails, then answer the result of evaluating operation."

	^self implementorsResponsibility!

at: key ifAbsentPut: operation 
	"Answer the element at key in the receiver. If key lookup for key fails, then store and return the result of evaluating operation."

	^self implementorsResponsibility!

at: key put: newElement 
	"Store newElement at key in the receiver. Answer newElement."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

includesKey: key 
	"Answer true if the receiver contains an element stored at key. Answer false otherwise."

	^self implementorsResponsibility!

keyAtValue: value 
	"Answer a key such that the element stored at this key is equal to value. Answer nil if no such key is found."

	^self implementorsResponsibility!

keyAtValue: value ifAbsent: operation 
	"Answer a key such that the element stored at this key is equivalent to value. Answer the result of evaluating operation if no such key is found."

	^self implementorsResponsibility!

keys
	"Answer a collection of keys at which there is an element stored in the receiver."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Iteratively evaluate operation with each of the receiver's keys and values."

	^self implementorsResponsibility!

keysDo: operation 
	"Iteratively evaluate operation with each of the receiver's keys at which there are elements stored."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which excludes the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

removeAllKeys: keys 
	"Remove any elements from the receiver which are stored at the keys specified in keys."

	^self implementorsResponsibility!

removeAllKeys: keys ifAbsent: operation 
	"Remove any elements from the receiver which are stored at the keys specified in keys. For any element in keys which is not a valid key of the receiver, evaluate operation with that element as the argument, but do not stop the enumeration."

	^self implementorsResponsibility!

removeKey: key 
	"Remove the element which is stored at key in the receiver. Answer the removed element."

	^self implementorsResponsibility!

removeKey: key ifAbsent: operation 
	"Remove the element which is stored at key in the receiver and answer the removed element. Answer the result of evaluating operation if no such key is found in the receiver."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

values
	"Answer a collection of the receiver's elements."

	^self implementorsResponsibility! !

!IdentityDictionaryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!IdentityDictionaryProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !


!ObjectProtocol methodsFor: 'repertoire'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !


!ReadStreamFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ReadStreamFactoryProtocol methodsFor: 'repertoire'!

on: aCollection 
	"Returns a stream that reads from the given collection."

	^self implementorsResponsibility! !


!sequencedContractibleCollectionProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!sequencedContractibleCollectionProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!sequencedContractibleCollectionProtocol methodsFor: 'repertoire'!

removeAtIndex: index 
	"Remove the element of the receiver at position index, and answer the removed element."

	^self implementorsResponsibility!

removeFirst
	"Remove and answer the first element of the receiver."

	^self implementorsResponsibility!

removeLast
	"Remove and answer the last element of the receiver."

	^self implementorsResponsibility! !


!DateAndTimeProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!DateAndTimeProtocol methodsFor: 'magnitudeProtocol'!

<= operand 
	"Answer true if the receiver is less than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

>= operand 
	"Answer true if the receiver is greater than or equal to operand. Answer false otherwise."

	^self implementorsResponsibility!

between: min and: max 
	"Answer true if the receiver is less than or equal to max, and greater than or equal to min. Answer false otherwise."

	^self implementorsResponsibility!

max: operand 
	"Answer the receiver if it is greater than operand. Answer operand otherwise."

	^self implementorsResponsibility!

min: operand 
	"Answer the receiver if it is less than operand. Answer operand otherwise."

	^self implementorsResponsibility! !

!DateAndTimeProtocol methodsFor: 'repertoire'!

+ operand 
	"Answer the result of adding operand to the receiver. "

	^self implementorsResponsibility!

- operand 
	"Answer the result of adding operand to the receiver. "

	^self implementorsResponsibility!

< operand 
	"Answer true if the receiver is less than operand. Answer false otherwise."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

> operand 
	"Answer true if the receiver is greater than operand. Answer false otherwise."

	^self implementorsResponsibility!

asLocal
	"Answer a <DateAndTimeProtocol> that represents the same UTC time as the receiver but in the local time specified by the implementation. "

	^self implementorsResponsibility!

asUTC
	"Answer a <DateAndTimeProtocol> that represents the same absolute time as the receiver but in the local time UTC. "

	^self implementorsResponsibility!

dayOfMonth
	"Answer the number of the day in the month in the local time of the receiver which includes the receiver. "

	^self implementorsResponsibility!

dayOfWeek
	"Answer the number of the day in the week, in the local time of the receiver, which includes the receiver. "

	^self implementorsResponsibility!

dayOfWeekAbbreviation
	"Answer the abbreviation of the name, in the local time of the receiver, of the day of the week which includes the receiver. "

	^self implementorsResponsibility!

dayOfWeekName
	"Answer the name, in the local time of the receiver, of the day of the week which includes the receiver. "

	^self implementorsResponsibility!

dayOfYear
	"Answer the number of the day in the year, in the local time of the receiver, which includes the receiver. "

	^self implementorsResponsibility!

hour
	"Answer the number of the hour in the day, in the local time of the receiver, which includes the receiver. "

	^self implementorsResponsibility!

hour12
	"Answer the hour in the day in the 12-hour clock of the local time of the receiver. "

	^self implementorsResponsibility!

hour24
	"Answer the hour in the day in the 24-hour clock of the local time of the receiver. "

	^self implementorsResponsibility!

isLeapYear
	"Test for leap year. "

	^self implementorsResponsibility!

meridianAbbreviation
	"Answer the abbreviation, in the local time of the receiver, of the name of the half of the day, which includes the receiver. "

	^self implementorsResponsibility!

minute
	"Answer the minute of the hour in the local time of the receiver. "

	^self implementorsResponsibility!

month
	"Answer the number of the month in the year, in the local time of the receiver, which includes the receiver. "

	^self implementorsResponsibility!

monthAbbreviation
	"Answer the abbreviation of the name of the month, in the local time of the receiver, which includes the receiver. "

	^self implementorsResponsibility!

monthName
	"Answer the name of the month, in the local time of the receiver, which includes the receiver. "

	^self implementorsResponsibility!

offset
	"Answer the difference between the local time of the receiver and UTC at the time of the receiver. "

	^self implementorsResponsibility!

offset: offset 
	"Answer a <DateAndTimeProtocol> equivalent to the receiver but with its local time being offset from UTC by offset. "

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

second
	"Answer the second of the minute of the local time of the receiver. "

	^self implementorsResponsibility!

timeZoneAbbreviation
	"Answer the abbreviation of the name, in the local time of the receiver, of the time zone of the receiver. "

	^self implementorsResponsibility!

timeZoneName
	"Answer the name in the local time of the receiver of the time zone of the receiver. "

	^self implementorsResponsibility!

year
	"Answer the number of the year in the local time of the receiver which includes the receiver. "

	^self implementorsResponsibility! !


!ErrorProtocol methodsFor: 'signaledExceptionProtocol'!

isNested
	"Determine whether the current exception handler is within the scope of another handler for the same exception."

	^self implementorsResponsibility!

outer
	"Evaluate the enclosing exception action for the receiver and return."

	^self implementorsResponsibility!

pass
	"Yield control to the enclosing exception action for the receiver."

	^self implementorsResponsibility!

resignalAs: replacementException 
	"Signal an alternative exception in place of the receiver."

	^self implementorsResponsibility!

resume
	"Return from the message that signaled the receiver."

	^self implementorsResponsibility!

resume: resumptionValue 
	"Return the argument as the value of the message that signaled the receiver."

	^self implementorsResponsibility!

retry
	"Abort an exception handler and re-evaluate its protected block."

	^self implementorsResponsibility!

retryUsing: alternativeBlock 
	"Abort an exception handler and evaluate a new block in place of the handler's protected block."

	^self implementorsResponsibility!

return
	"Return nil as the value of the block protected by the active exception handler."

	^self implementorsResponsibility!

return: returnValue 
	"Return the argument as the value of the block protected by the active exception handler."

	^self implementorsResponsibility! !

!ErrorProtocol methodsFor: 'exceptionBuilderProtocol'!

messageText: signalerText 
	"Set an exception's message text."

	^self implementorsResponsibility! !

!ErrorProtocol methodsFor: 'repertoire'!

defaultAction

	^self implementorsResponsibility!

isResumable
	"Determine whether an exception is resumable."

	^self implementorsResponsibility! !


!exceptionSetProtocol methodsFor: 'exceptionSelectorProtocol'!

handles: exception 
	"Determine whether an exception handler will accept a signaled exception."

	^self implementorsResponsibility! !

!exceptionSetProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!exceptionSetProtocol methodsFor: 'repertoire'!

, anotherException 
	"Return an exception set that contains the receiver and the argument exception. This is commonly used to specify a set of exception selectors for an exception handler."

	^self implementorsResponsibility! !


!nilProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!nilProtocol methodsFor: 'repertoire'!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility! !


!WriteStreamFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!WriteStreamFactoryProtocol methodsFor: 'repertoire'!

with: aCollection 
	"Returns a stream that appends to the given collection."

	^self implementorsResponsibility! !


!OrderedCollectionProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!OrderedCollectionProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!OrderedCollectionProtocol methodsFor: 'extensibleCollectionProtocol'!

addAll: newElements 
	"Add each element of newElements to the receiver's elements."

	^self implementorsResponsibility!

remove: oldElement 
	"Remove the first element of the receiver which is equivalent to oldElement and return the removed element."

	^self implementorsResponsibility!

remove: oldElement ifAbsent: exceptionHandler 
	"Remove the first element of the receiver which is equivalent to oldElement. If it is not found, answer the result of evaluating exceptionHandler."

	^self implementorsResponsibility!

removeAll: oldElements 
	"For each element in oldElements, remove the first element from the receiver which is equivalent to this element."

	^self implementorsResponsibility! !

!OrderedCollectionProtocol methodsFor: 'sequencedContractibleCollectionProtocol'!

removeAtIndex: index 
	"Remove the element of the receiver at position index, and answer the removed element."

	^self implementorsResponsibility!

removeFirst
	"Remove and answer the first element of the receiver."

	^self implementorsResponsibility!

removeLast
	"Remove and answer the last element of the receiver."

	^self implementorsResponsibility! !

!OrderedCollectionProtocol methodsFor: 'sequencedReadableCollectionProtocol'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !

!OrderedCollectionProtocol methodsFor: 'sequencedCollectionProtocol'!

at: index put: newElement 
	"Replace the element in the receiver at index with newElement. Answer newElement."

	^self implementorsResponsibility!

atAll: indices put: newElement 
	"Replace the elements in the receiver specified by indices with newElement."

	^self implementorsResponsibility!

atAllPut: newElement 
	"Replace all the elements in the receiver with newElement."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements 
	"Replace the elements of the receiver between positions start and stop inclusive, with the elements of replacementElements in their original order. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements startingAt: replacementStart 
	"Replace the elements of the receiver between positions start and stop inclusive with the elements of replacementElements, in their original order, starting at position replacementStart. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop withObject: replacementElement 
	"Replace the elements of the receiver between start and stop inclusive with replacementElement. Answer the receiver."

	^self implementorsResponsibility! !

!OrderedCollectionProtocol methodsFor: 'repertoire'!

add: newElement 
	"Add newElement to the receiver's elements."

	^self implementorsResponsibility!

add: newElement after: target 
	"Add newElement to the receiver immediately following the first element which is equivalent to target."

	^self implementorsResponsibility!

add: newElement afterIndex: index 
	"Add newElement to the receiver immediately following the element at position index."

	^self implementorsResponsibility!

add: newElement before: target 
	"Add newElement to the receiver immediately before the first element which is equivalent to target."

	^self implementorsResponsibility!

add: newElement beforeIndex: index 
	"Add newElement to the receiver immediately before the element at position index."

	^self implementorsResponsibility!

addAll: newElements after: target 
	"Add each element of newElements to the receiver immediately after the first element in the receiver which is equivalent to target. Answer newElements."

	^self implementorsResponsibility!

addAll: newElements afterIndex: index 
	"Insert the elements of newElements in the receiver immediately after the element at position index. Answer newElements."

	^self implementorsResponsibility!

addAll: newElements before: target 
	"Add each element of newElements to the receiver immediately before the first element in the receiver which is equivalent to target. Answer newElements."

	^self implementorsResponsibility!

addAll: newElements beforeIndex: index 
	"Insert the elements of newElements in the receiver immediately before the element at position index. Answer newElements."

	^self implementorsResponsibility!

addAllFirst: newElements 
	"Add each element of newElements to the beginning of the receiver's elements. Answer newElements."

	^self implementorsResponsibility!

addAllLast: newElements 
	"Add each element of newElements to the end of the receiver's elements. Answer newElements."

	^self implementorsResponsibility!

addFirst: newElement 
	"Add newElement to the beginning of the receiver's elements. Answer newElement."

	^self implementorsResponsibility!

addLast: newElement 
	"Add newElement to the end of the receiver's elements. Answer newElement."

	^self implementorsResponsibility! !


!ReadWriteStreamFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ReadWriteStreamFactoryProtocol methodsFor: 'repertoire'!

with: aCollection 
	"Returns a stream that reads the elements of the given collection and can write new elements."

	^self implementorsResponsibility! !


!classDescriptionProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!classDescriptionProtocol methodsFor: 'repertoire'!

allSubclasses
	"Return all subclasses of a class."

	^self implementorsResponsibility!

allSuperclasses
	"Return all superclasses of a class."

	^self implementorsResponsibility!

name
	"Return the name of a class."

	^self implementorsResponsibility!

subclasses
	"Return direct subclasses of a class."

	^self implementorsResponsibility!

superclass
	"Return the immediate superclass of a class."

	^self implementorsResponsibility! !


!DictionaryProtocol methodsFor: 'abstractDictionaryProtocol'!

addAll: dictionary 
	"Store the elements of dictionary in the receiver at the corresponding keys from dictionary."

	^self implementorsResponsibility!

at: key 
	"Answer the element at key in the receiver."

	^self implementorsResponsibility!

at: key ifAbsent: operation 
	"Answer the element at key in the receiver. If key lookup for key fails, then answer the result of evaluating operation."

	^self implementorsResponsibility!

at: key ifAbsentPut: operation 
	"Answer the element at key in the receiver. If key lookup for key fails, then store and return the result of evaluating operation."

	^self implementorsResponsibility!

at: key put: newElement 
	"Store newElement at key in the receiver. Answer newElement."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

includesKey: key 
	"Answer true if the receiver contains an element stored at key. Answer false otherwise."

	^self implementorsResponsibility!

keyAtValue: value 
	"Answer a key such that the element stored at this key is equal to value. Answer nil if no such key is found."

	^self implementorsResponsibility!

keyAtValue: value ifAbsent: operation 
	"Answer a key such that the element stored at this key is equivalent to value. Answer the result of evaluating operation if no such key is found."

	^self implementorsResponsibility!

keys
	"Answer a collection of keys at which there is an element stored in the receiver."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Iteratively evaluate operation with each of the receiver's keys and values."

	^self implementorsResponsibility!

keysDo: operation 
	"Iteratively evaluate operation with each of the receiver's keys at which there are elements stored."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which excludes the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

removeAllKeys: keys 
	"Remove any elements from the receiver which are stored at the keys specified in keys."

	^self implementorsResponsibility!

removeAllKeys: keys ifAbsent: operation 
	"Remove any elements from the receiver which are stored at the keys specified in keys. For any element in keys which is not a valid key of the receiver, evaluate operation with that element as the argument, but do not stop the enumeration."

	^self implementorsResponsibility!

removeKey: key 
	"Remove the element which is stored at key in the receiver. Answer the removed element."

	^self implementorsResponsibility!

removeKey: key ifAbsent: operation 
	"Remove the element which is stored at key in the receiver and answer the removed element. Answer the result of evaluating operation if no such key is found in the receiver."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

values
	"Answer a collection of the receiver's elements."

	^self implementorsResponsibility! !

!DictionaryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!DictionaryProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !


!ExceptionClassProtocol methodsFor: 'classDescriptionProtocol'!

allSubclasses
	"Return all subclasses of a class."

	^self implementorsResponsibility!

allSuperclasses
	"Return all superclasses of a class."

	^self implementorsResponsibility!

name
	"Return the name of a class."

	^self implementorsResponsibility!

subclasses
	"Return direct subclasses of a class."

	^self implementorsResponsibility!

superclass
	"Return the immediate superclass of a class."

	^self implementorsResponsibility! !

!ExceptionClassProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ExceptionClassProtocol methodsFor: 'exceptionSelectorProtocol'!

, anotherException 
	"Create an exception set."

	^self implementorsResponsibility! !

!ExceptionClassProtocol methodsFor: 'exceptionSignalerProtocol'!

signal: signalerText 
	"Signal the occurrence of an exceptional condition with a specified textual description."

	^self implementorsResponsibility! !

!ExceptionClassProtocol methodsFor: 'repertoire'!

handles: exception 

	^self implementorsResponsibility!

new

	^self implementorsResponsibility!

signal

	^self implementorsResponsibility! !


!NotificationProtocol methodsFor: 'signaledExceptionProtocol'!

isNested
	"Determine whether the current exception handler is within the scope of another handler for the same exception."

	^self implementorsResponsibility!

outer
	"Evaluate the enclosing exception action for the receiver and return."

	^self implementorsResponsibility!

pass
	"Yield control to the enclosing exception action for the receiver."

	^self implementorsResponsibility!

resignalAs: replacementException 
	"Signal an alternative exception in place of the receiver."

	^self implementorsResponsibility!

resume
	"Return from the message that signaled the receiver."

	^self implementorsResponsibility!

resume: resumptionValue 
	"Return the argument as the value of the message that signaled the receiver."

	^self implementorsResponsibility!

retry
	"Abort an exception handler and re-evaluate its protected block."

	^self implementorsResponsibility!

retryUsing: alternativeBlock 
	"Abort an exception handler and evaluate a new block in place of the handler's protected block."

	^self implementorsResponsibility!

return
	"Return nil as the value of the block protected by the active exception handler."

	^self implementorsResponsibility!

return: returnValue 
	"Return the argument as the value of the block protected by the active exception handler."

	^self implementorsResponsibility! !

!NotificationProtocol methodsFor: 'exceptionBuilderProtocol'!

messageText: signalerText 
	"Set an exception's message text."

	^self implementorsResponsibility! !

!NotificationProtocol methodsFor: 'repertoire'!

defaultAction

	^self implementorsResponsibility!

isResumable

	^self implementorsResponsibility! !


!TranscriptProtocol methodsFor: 'puttableStreamProtocol'!

cr
	"Writes an end-of-line sequence to the receiver."

	^self implementorsResponsibility!

flush
	"Update a stream's backing store."

	^self implementorsResponsibility!

nextPut: anObject 
	"Writes the argument to the stream."

	^self implementorsResponsibility!

nextPutAll: aCollection 
	"Enumerates the argument, adding each element to the receiver"

	^self implementorsResponsibility!

space
	"Writes a space character to the receiver."

	^self implementorsResponsibility!

tab
	"Writes a tab character to the receiver."

	^self implementorsResponsibility! !

!TranscriptProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !


!MessageNotUnderstoodSelectorProtocol methodsFor: 'repertoire'!

handles: exception 
	"Determine whether an exception handler will accept a signaled exception."

	^self implementorsResponsibility! !


!ZeroDivideFactoryProtocol methodsFor: 'exceptionSelectorProtocol'!

, anotherException 
	"Create an exception set."

	^self implementorsResponsibility!

handles: exception 
	"Determine whether an exception handler will accept a signaled exception."

	^self implementorsResponsibility! !

!ZeroDivideFactoryProtocol methodsFor: 'exceptionInstantiatorProtocol'!

new
	"Create a new object."

	^self implementorsResponsibility! !

!ZeroDivideFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ZeroDivideFactoryProtocol methodsFor: 'exceptionSignalerProtocol'!

signal: signalerText 
	"Signal the occurrence of an exceptional condition with a specified textual description."

	^self implementorsResponsibility! !

!ZeroDivideFactoryProtocol methodsFor: 'repertoire'!

dividend: argument 
	"Signal the occurance of a division by zero."

	^self implementorsResponsibility!

signal

	^self implementorsResponsibility! !


!IdentityDictionaryFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing all the elements of newElements."

	^self implementorsResponsibility! !


!booleanProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!booleanProtocol methodsFor: 'repertoire'!

& operand 
	"Logical and - Boolean conjunction."

	^self implementorsResponsibility!

and: operand 
	"'Short circuit' logical and."

	^self implementorsResponsibility!

eqv: operand 
	"Boolean equivalence."

	^self implementorsResponsibility!

ifFalse: operand 
	"Evaluate the argument if receiver is false."

	^self implementorsResponsibility!

ifFalse: falseOperand ifTrue: trueOperand 
	"Selectively evaluate one of the arguments."

	^self implementorsResponsibility!

ifTrue: operand 
	"Evaluate the argument if the receiver is true."

	^self implementorsResponsibility!

ifTrue: trueOperand ifFalse: falseOperand 
	"Selectively evaluate one of the arguments."

	^self implementorsResponsibility!

not
	"Logical not - Boolean negation."

	^self implementorsResponsibility!

or: operand 
	"'Short circuit' logical or."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

xor: operand 
	"Boolean exclusive or."

	^self implementorsResponsibility!

| operand 
	"Logical or - Boolean disjunction."

	^self implementorsResponsibility! !


!collectionProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!collectionProtocol methodsFor: 'repertoire'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !


!sequencedCollectionProtocol methodsFor: 'sequencedReadableCollectionProtocol'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !

!sequencedCollectionProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!sequencedCollectionProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!sequencedCollectionProtocol methodsFor: 'repertoire'!

at: index put: newElement 
	"Replace the element in the receiver at index with newElement. Answer newElement."

	^self implementorsResponsibility!

atAll: indices put: newElement 
	"Replace the elements in the receiver specified by indices with newElement."

	^self implementorsResponsibility!

atAllPut: newElement 
	"Replace all the elements in the receiver with newElement."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements 
	"Replace the elements of the receiver between positions start and stop inclusive, with the elements of replacementElements in their original order. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements startingAt: replacementStart 
	"Replace the elements of the receiver between positions start and stop inclusive with the elements of replacementElements, in their original order, starting at position replacementStart. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop withObject: replacementElement 
	"Replace the elements of the receiver between start and stop inclusive with replacementElement. Answer the receiver."

	^self implementorsResponsibility! !


!monadicValuableProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!monadicValuableProtocol methodsFor: 'valuableProtocol'!

valueWithArguments: argumentArray 
	"Answers the value of the receiver when applied to the arguments in argumentArray."

	^self implementorsResponsibility! !

!monadicValuableProtocol methodsFor: 'repertoire'!

argumentCount
	"Answers the number of arguments needed to evaluate the receiver."

	^self implementorsResponsibility!

value: argument 
	"Answers the value of the receiver when applied to the argument."

	^self implementorsResponsibility! !


!SortedCollectionFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!SortedCollectionFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility!

sortBlock: sortBlock 
	"Create a new sorted collection with sortBlock as the sort block."

	^self implementorsResponsibility!

with: element1 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 with: element4 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing only the elements of newElements."

	^self implementorsResponsibility! !


!WarningClassProtocol methodsFor: 'classDescriptionProtocol'!

allSubclasses
	"Return all subclasses of a class."

	^self implementorsResponsibility!

allSuperclasses
	"Return all superclasses of a class."

	^self implementorsResponsibility!

name
	"Return the name of a class."

	^self implementorsResponsibility!

subclasses
	"Return direct subclasses of a class."

	^self implementorsResponsibility!

superclass
	"Return the immediate superclass of a class."

	^self implementorsResponsibility! !

!WarningClassProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!WarningClassProtocol methodsFor: 'exceptionSignalerProtocol'!

signal: signalerText 
	"Signal the occurrence of an exceptional condition with a specified textual description."

	^self implementorsResponsibility! !

!WarningClassProtocol methodsFor: 'ExceptionClassProtocol'!

handles: exception 

	^self implementorsResponsibility!

signal

	^self implementorsResponsibility! !

!WarningClassProtocol methodsFor: 'exceptionSelectorProtocol'!

, anotherException 
	"Create an exception set."

	^self implementorsResponsibility! !

!WarningClassProtocol methodsFor: 'repertoire'!

new

	^self implementorsResponsibility! !


!exceptionSelectorProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!exceptionSelectorProtocol methodsFor: 'repertoire'!

, anotherException 
	"Create an exception set."

	^self implementorsResponsibility!

handles: exception 
	"Determine whether an exception handler will accept a signaled exception."

	^self implementorsResponsibility! !


!ByteArrayProtocol methodsFor: 'sequencedReadableCollectionProtocol'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !

!ByteArrayProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ByteArrayProtocol methodsFor: 'sequencedCollectionProtocol'!

at: index put: newElement 
	"Replace the element in the receiver at index with newElement. Answer newElement."

	^self implementorsResponsibility!

atAll: indices put: newElement 
	"Replace the elements in the receiver specified by indices with newElement."

	^self implementorsResponsibility!

atAllPut: newElement 
	"Replace all the elements in the receiver with newElement."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements 
	"Replace the elements of the receiver between positions start and stop inclusive, with the elements of replacementElements in their original order. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop with: replacementElements startingAt: replacementStart 
	"Replace the elements of the receiver between positions start and stop inclusive with the elements of replacementElements, in their original order, starting at position replacementStart. Answer the receiver."

	^self implementorsResponsibility!

replaceFrom: start to: stop withObject: replacementElement 
	"Replace the elements of the receiver between start and stop inclusive with replacementElement. Answer the receiver."

	^self implementorsResponsibility! !

!ByteArrayProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !


!floatCharacterizationProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!floatCharacterizationProtocol methodsFor: 'repertoire'!

denormalized
	"Indication of whether the characterized floating point object representation allows denormalized values."

	^self implementorsResponsibility!

e
	"The closest floating point approximation of the irrational number e."

	^self implementorsResponsibility!

emax
	"The largest exponent of the characterized floating point object representation."

	^self implementorsResponsibility!

emin
	"The smallest exponent of the characterized floating point object representation."

	^self implementorsResponsibility!

epsilon
	"The maximum relative spacing in the characterized floating point object representation."

	^self implementorsResponsibility!

fmax
	"The largest value allowed by the characterized floating point object representation."

	^self implementorsResponsibility!

fmin
	"The minimum value allowed by the characterized floating point object representation."

	^self implementorsResponsibility!

fminDenormalized
	"The minimum denormalized value allowed by the characterized floating point object representation."

	^self implementorsResponsibility!

fminNormalized
	"The minimum normalized value allowed by the characterized floating point object representation."

	^self implementorsResponsibility!

pi
	"The closest floating point approximation to Pi."

	^self implementorsResponsibility!

precision
	"The precision of the characterized floating point object representation."

	^self implementorsResponsibility!

radix
	"The radix of the characterized floating point object representation."

	^self implementorsResponsibility! !


!initializableCollectionFactoryProtocol methodsFor: 'collectionFactoryProtocol'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility! !

!initializableCollectionFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!initializableCollectionFactoryProtocol methodsFor: 'repertoire'!

with: element1 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 with: element4 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing all the elements of newElements."

	^self implementorsResponsibility! !


!ReadStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ReadStreamProtocol methodsFor: 'gettableStreamProtocol'!

atEnd
	"Returns a Boolean indicating whether the receiver is at the end of its values."

	^self implementorsResponsibility!

do: operation 
	"Evaluates the argument with each receiver future sequence value, terminating evaluation when there are no more future sequence values."

	^self implementorsResponsibility!

next
	"Return the next object in the receiver."

	^self implementorsResponsibility!

nextLine
	"Reads the next line from the stream."

	^self implementorsResponsibility!

nextMatchFor: anObject 
	"Reads the next object from the stream and returns true if the object is equivalent to the argument and false if not."

	^self implementorsResponsibility!

peek
	"Returns the next object in the receiver's future sequence values without advancing the receiver's position. Returns nil if the receiver is at end of stream."

	^self implementorsResponsibility!

peekFor: anObject 
	"Peeks at the next object in the stream and returns true if it matches the argument, and false if not."

	^self implementorsResponsibility!

skip: amount 
	"Skips the next amount objects in the receiver's future sequence values."

	^self implementorsResponsibility!

skipTo: anObject 
	"Sets the stream to read the object just after the next occurrence of the argument and returns true. If the argument is not found before the end of the stream is encountered, false is returned."

	^self implementorsResponsibility! !

!ReadStreamProtocol methodsFor: 'sequencedStreamProtocol'!

close
	"Disassociate a stream from its backing store."

	^self implementorsResponsibility!

isEmpty
	"Returns a Boolean indicating whether there are any sequence values in the receiver."

	^self implementorsResponsibility!

position
	"Returns the current position of the stream."

	^self implementorsResponsibility!

position: amount 
	"Sets the current position in a stream of values."

	^self implementorsResponsibility!

reset
	"Resets the position of the receiver to be at the beginning of the stream of values."

	^self implementorsResponsibility!

setToEnd
	"Set the position of the stream to its end."

	^self implementorsResponsibility! !

!ReadStreamProtocol methodsFor: 'collectionStreamProtocol'!

contents
	"Returns a collection containing the complete contents of the stream."

	^self implementorsResponsibility! !

!ReadStreamProtocol methodsFor: 'repertoire'!

next: amount 
	"Returns a collection of the next amount objects in the stream."

	^self implementorsResponsibility!

upTo: anObject 
	"Returns a collection of all of the objects in the receiver up to, but not including, the next occurrence of the argument. Sets the stream to read the object just after the next occurrence of the argument. If the argument is not found and the end of the stream is encountered, an ordered collection of the objects read is returned."

	^self implementorsResponsibility! !


!sequencedReadableCollectionProtocol methodsFor: 'ObjectProtocol'!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!sequencedReadableCollectionProtocol methodsFor: 'collectionProtocol'!

allSatisfy: discriminator 
	"Return true if the discriminator evaluates to true for every element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

anySatisfy: discriminator 
	"Return true if the discriminator evaluates to true for any element of the receiver. Otherwise return false."

	^self implementorsResponsibility!

asArray
	"Answer an array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asBag
	"Answer a bag with the same elements as the receiver."

	^self implementorsResponsibility!

asByteArray
	"Answer a byte array whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asOrderedCollection
	"Answer an ordered collection whose elements are the elements of the receiver."

	^self implementorsResponsibility!

asSet
	"Answer a set with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection
	"Answer a sorted collection with the same elements as the receiver."

	^self implementorsResponsibility!

asSortedCollection: sortBlock 
	"Answer a sorted collection with the same elements as the receiver. The parameter sortBlock is used as the sort block."

	^self implementorsResponsibility!

collect: transformer 
	"Answer a new collection constructed by gathering the results of evaluating transformer with each element of the receiver."

	^self implementorsResponsibility!

detect: discriminator 
	"Return the first element of the receiver which causes discriminator to evaluate to true when the element is used as the argument."

	^self implementorsResponsibility!

detect: discriminator ifNone: exceptionHandler 
	"Return the first element of the receiver which causes discriminator to evaluate to true when used as the argument to the evaluation. Answer the result of evaluating exceptionHandler if no such element is found."

	^self implementorsResponsibility!

do: operation separatedBy: separator 
	"Evaluate operation with each element of the receiver interspersed by evaluation of separator."

	^self implementorsResponsibility!

includes: target 
	"Answer true if an element of the receiver is equivalent to target. Answer false otherwise."

	^self implementorsResponsibility!

inject: initialValue into: operation 
	"Answer the final result of evaluating operation using each element of the receiver and the previous evaluation result as the parameters."

	^self implementorsResponsibility!

isEmpty
	"Return true if the receiver contains no elements. Return false otherwise."

	^self implementorsResponsibility!

notEmpty
	"Return true if the receiver contains elements. Return false otherwise."

	^self implementorsResponsibility!

occurrencesOf: target 
	"Answer the number of elements of the receiver which are equivalent to target."

	^self implementorsResponsibility!

rehash
	"Re-establish hash invariants, if any."

	^self implementorsResponsibility!

reject: discriminator 
	"Answer a new collection which includes only the elements in the receiver which cause discriminator to evaluate to false."

	^self implementorsResponsibility!

select: discriminator 
	"Answer a new collection which contains only the elements in the receiver which cause discriminator to evaluate to true."

	^self implementorsResponsibility!

size
	"Answer the number of elements in the receiver."

	^self implementorsResponsibility! !

!sequencedReadableCollectionProtocol methodsFor: 'repertoire'!

, operand 
	"Answer a new collection which is the concatenation of the receiver and operand."

	^self implementorsResponsibility!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

after: target 
	"Answer the object immediately following the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

at: index 
	"Answer the element at the position index in the receiver."

	^self implementorsResponsibility!

at: index ifAbsent: exceptionBlock 
	"Answer the element at the position index in the receiver.  If there is no position corresponding to index in the receiver, then answer the result of evaluating exceptionBlock."

	^self implementorsResponsibility!

before: target 
	"Answer the object immediately preceding the first element which is equivalent to target in the receiver."

	^self implementorsResponsibility!

copyFrom: start to: stop 
	"Answer a new collection containing all of the elements of the receiver between the indices start and stop inclusive. If stop < start, the result has a size of zero."

	^self implementorsResponsibility!

copyReplaceAll: targetElements with: replacementElements 
	"Answer a new collection in which all subsequences of elements in the receiver matching targetElements are replaced in the new collection by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop with: replacementElements 
	"Answer a new collection, containing the same elements as the receiver, but with the elements in the receiver between start and stop inclusive replaced by the elements in replacementElements."

	^self implementorsResponsibility!

copyReplaceFrom: start to: stop withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which the elements of the receiver between start and stop inclusive have been replaced with replacementElement."

	^self implementorsResponsibility!

copyReplacing: targetElement withObject: replacementElement 
	"Answer a new collection conforming to the same protocols as the receiver, in which any occurrences of targetElement are replaced by replacementElement."

	^self implementorsResponsibility!

copyWith: newElement 
	"Answer a new collection containing the same elements as the receiver, with newElement added."

	^self implementorsResponsibility!

copyWithout: oldElement 
	"Answer a new collection, containing the same elements as the receiver in their original order omitting any elements equivalent to oldElement."

	^self implementorsResponsibility!

do: operation 
	"Evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

findFirst: discriminator 
	"Answer the index of the first element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

findLast: discriminator 
	"Answer the index of the last element of the receiver which causes discriminator to evaluate to true when the element is used as the parameter. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

first
	"Answer the first element of the receiver."

	^self implementorsResponsibility!

from: start to: stop do: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with each element of the receiver."

	^self implementorsResponsibility!

from: start to: stop keysAndValuesDo: operation 
	"For those elements of the receiver between positions start and stop, inclusive, evaluate operation with an element of the receiver as the first argument and the element's position (index) as the second."

	^self implementorsResponsibility!

indexOf: target 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer zero (0) if no such element is found."

	^self implementorsResponsibility!

indexOf: target ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is equivalent to target. Answer the result of evaluating exceptionHandler with no parameters if no such element is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer 0 if no such subsequence is found."

	^self implementorsResponsibility!

indexOfSubCollection: targetSequence startingAt: start ifAbsent: exceptionHandler 
	"Answer the index of the first element of the receiver which is the start of a subsequence which matches targetSequence. Start searching at index start in the receiver. Answer the result of evaluating exceptionHandler with no parameters if no such subsequence is found."

	^self implementorsResponsibility!

keysAndValuesDo: operation 
	"Evaluate operation with the index of each element of the receiver, in order, together with the element itself."

	^self implementorsResponsibility!

last
	"Answer the last element of the receiver."

	^self implementorsResponsibility!

reverse
	"Answer a collection with the elements of the receiver arranged in reverse order."

	^self implementorsResponsibility!

reverseDo: operation 
	"Evaluate operation with each element of the receiver in the reverse of the receiver's standard traversal order."

	^self implementorsResponsibility!

with: otherCollection do: operation 
	"Evaluate operation with each element of the receiver and the corresponding element of otherCollection as parameters."

	^self implementorsResponsibility! !


!StringFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!StringFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility!

with: element1 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

with: element1 with: element2 with: element3 with: element4 
	"Create a collection initially containing the argument elements"

	^self implementorsResponsibility!

withAll: newElements 
	"Create a collection containing only the elements of newElements."

	^self implementorsResponsibility! !


!exceptionBuilderProtocol methodsFor: 'repertoire'!

messageText: signalerText 
	"Set an exception's message text."

	^self implementorsResponsibility! !


!exceptionSignalerProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!exceptionSignalerProtocol methodsFor: 'repertoire'!

signal
	"Signal the occurrence of an exceptional condition."

	^self implementorsResponsibility!

signal: signalerText 
	"Signal the occurrence of an exceptional condition with a specified textual description."

	^self implementorsResponsibility! !


!FractionFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!FractionFactoryProtocol methodsFor: 'repertoire'!

numerator: top denominator: bottom 
	"Answer a new fraction whose numerator is top, and whose denominator is bottom."

	^self implementorsResponsibility! !


!NotificationClassProtocol methodsFor: 'classDescriptionProtocol'!

allSubclasses
	"Return all subclasses of a class."

	^self implementorsResponsibility!

allSuperclasses
	"Return all superclasses of a class."

	^self implementorsResponsibility!

name
	"Return the name of a class."

	^self implementorsResponsibility!

subclasses
	"Return direct subclasses of a class."

	^self implementorsResponsibility!

superclass
	"Return the immediate superclass of a class."

	^self implementorsResponsibility! !

!NotificationClassProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!NotificationClassProtocol methodsFor: 'exceptionSignalerProtocol'!

signal: signalerText 
	"Signal the occurrence of an exceptional condition with a specified textual description."

	^self implementorsResponsibility! !

!NotificationClassProtocol methodsFor: 'ExceptionClassProtocol'!

handles: exception 

	^self implementorsResponsibility!

signal

	^self implementorsResponsibility! !

!NotificationClassProtocol methodsFor: 'exceptionSelectorProtocol'!

, anotherException 
	"Create an exception set."

	^self implementorsResponsibility! !

!NotificationClassProtocol methodsFor: 'repertoire'!

new
	"Return a newly created object initialized to a standard initial state."

	^self implementorsResponsibility! !


!puttableStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!puttableStreamProtocol methodsFor: 'repertoire'!

cr
	"Writes an end-of-line sequence to the receiver."

	^self implementorsResponsibility!

flush
	"Update a stream's backing store."

	^self implementorsResponsibility!

nextPut: anObject 
	"Writes the argument to the stream."

	^self implementorsResponsibility!

nextPutAll: aCollection 
	"Enumerates the argument, adding each element to the receiver"

	^self implementorsResponsibility!

space
	"Writes a space character to the receiver."

	^self implementorsResponsibility!

tab
	"Writes a tab character to the receiver."

	^self implementorsResponsibility! !


!ReadWriteStreamProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!ReadWriteStreamProtocol methodsFor: 'gettableStreamProtocol'!

atEnd
	"Returns a Boolean indicating whether the receiver is at the end of its values."

	^self implementorsResponsibility!

do: operation 
	"Evaluates the argument with each receiver future sequence value, terminating evaluation when there are no more future sequence values."

	^self implementorsResponsibility!

next
	"Return the next object in the receiver."

	^self implementorsResponsibility!

nextLine
	"Reads the next line from the stream."

	^self implementorsResponsibility!

nextMatchFor: anObject 
	"Reads the next object from the stream and returns true if the object is equivalent to the argument and false if not."

	^self implementorsResponsibility!

peek
	"Returns the next object in the receiver's future sequence values without advancing the receiver's position. Returns nil if the receiver is at end of stream."

	^self implementorsResponsibility!

peekFor: anObject 
	"Peeks at the next object in the stream and returns true if it matches the argument, and false if not."

	^self implementorsResponsibility!

skip: amount 
	"Skips the next amount objects in the receiver's future sequence values."

	^self implementorsResponsibility!

skipTo: anObject 
	"Sets the stream to read the object just after the next occurrence of the argument and returns true. If the argument is not found before the end of the stream is encountered, false is returned."

	^self implementorsResponsibility! !

!ReadWriteStreamProtocol methodsFor: 'sequencedStreamProtocol'!

close
	"Disassociate a stream from its backing store."

	^self implementorsResponsibility!

isEmpty
	"Returns a Boolean indicating whether there are any sequence values in the receiver."

	^self implementorsResponsibility!

position
	"Returns the current position of the stream."

	^self implementorsResponsibility!

position: amount 
	"Sets the current position in a stream of values."

	^self implementorsResponsibility!

reset
	"Resets the position of the receiver to be at the beginning of the stream of values."

	^self implementorsResponsibility!

setToEnd
	"Set the position of the stream to its end."

	^self implementorsResponsibility! !

!ReadWriteStreamProtocol methodsFor: 'collectionStreamProtocol'!

contents
	"Returns a collection containing the complete contents of the stream."

	^self implementorsResponsibility! !

!ReadWriteStreamProtocol methodsFor: 'ReadStreamProtocol'!

next: amount 
	"Returns a collection of the next amount objects in the stream."

	^self implementorsResponsibility!

upTo: anObject 
	"Returns a collection of all of the objects in the receiver up to, but not including, the next occurrence of the argument. Sets the stream to read the object just after the next occurrence of the argument. If the argument is not found and the end of the stream is encountered, an ordered collection of the objects read is returned."

	^self implementorsResponsibility! !

!ReadWriteStreamProtocol methodsFor: 'puttableStreamProtocol'!

cr
	"Writes an end-of-line sequence to the receiver."

	^self implementorsResponsibility!

flush
	"Update a stream's backing store."

	^self implementorsResponsibility!

nextPut: anObject 
	"Writes the argument to the stream."

	^self implementorsResponsibility!

nextPutAll: aCollection 
	"Enumerates the argument, adding each element to the receiver"

	^self implementorsResponsibility!

space
	"Writes a space character to the receiver."

	^self implementorsResponsibility!

tab
	"Writes a tab character to the receiver."

	^self implementorsResponsibility! !


!collectionFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!collectionFactoryProtocol methodsFor: 'repertoire'!

new
	"Create a new object."

	^self implementorsResponsibility!

new: count 
	"Create a new collection. The parameter count constrains the number of elements in the result."

	^self implementorsResponsibility! !


!DurationFactoryProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!DurationFactoryProtocol methodsFor: 'repertoire'!

days: days hours: hours minutes: minutes seconds: seconds 
	"Answer a <Duration> of the number of days, hours, minutes, and seconds. "

	^self implementorsResponsibility!

seconds: seconds 
	"Answer a <Duration> which is seconds in length "

	^self implementorsResponsibility!

zero
	"Answer a <Duration> of zero length. "

	^self implementorsResponsibility! !


!exceptionDescriptionProtocol methodsFor: 'ObjectProtocol'!

= comparand 
	"Object equivalence test."

	^self implementorsResponsibility!

== comparand 
	"Object identity test."

	^self implementorsResponsibility!

class
	"Determine the class of the receiver."

	^self implementorsResponsibility!

copy
	"Return a copy of the receiver."

	^self implementorsResponsibility!

doesNotUnderstand: message 
	"A message was sent to the receiver for which the receiver has no behavior."

	^self implementorsResponsibility!

error: signalerText 
	"Announce an error"

	^self implementorsResponsibility!

hash
	"Return an integer hash code that can be used in conjunction with an #= comparison."

	^self implementorsResponsibility!

identityHash
	"Return an integer hash code that can be used in conjunction with an #== (identity) comparison."

	^self implementorsResponsibility!

isKindOf: candidateClass 
	"Classify an object."

	^self implementorsResponsibility!

isMemberOf: candidateClass 
	"Determine whether the receiver is an instance of the argument."

	^self implementorsResponsibility!

isNil
	"Determine if the receiver is the value of the reserved identifier nil."

	^self implementorsResponsibility!

notNil
	"Determine if the receiver is not the value of the reserved identifier nil."

	^self implementorsResponsibility!

perform: selector 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector with: argument1 with: argument2 with: argument3 
	"Send a message using a computed message selector."

	^self implementorsResponsibility!

perform: selector withArguments: arguments 
	"Send a message using a computed message selector and a collection of arguments."

	^self implementorsResponsibility!

printOn: target 
	"Write a textual description of the receiver to a stream."

	^self implementorsResponsibility!

printString
	"Return a string that describes the receiver."

	^self implementorsResponsibility!

respondsTo: selector 
	"Determine if the receiver can respond to a specific message selector."

	^self implementorsResponsibility!

yourself
	"No operation. Return the receiver as the result."

	^self implementorsResponsibility!

~= comparand 
	"Object inequality test."

	^self implementorsResponsibility!

~~ comparand 
	"Negated object identity test."

	^self implementorsResponsibility! !

!exceptionDescriptionProtocol methodsFor: 'repertoire'!

defaultAction
	"The default action taken if the exception is signaled."

	^self implementorsResponsibility!

description
	"Return a textual description of the exception."

	^self implementorsResponsibility!

isResumable
	"Determine whether an exception is resumable."

	^self implementorsResponsibility!

messageText
	"Return an exception's message text."

	^self implementorsResponsibility!

tag
	"Return an exception's tag value."

	^self implementorsResponsibility! !



Interface configuration synchronize: currentState.!

